#!/usr/bin/env python3
import os
import sys
import argparse

#
#
#
from daemon.utils import FsMgr
from daemon.core  import SquidUser, DaemonPath

#
# action dispatcher
#
def perform_action(args):

    # get the paths
    source_dir = args.config
    target_dir = DaemonPath.etc_dir()

    # create actor
    fsmgr = FsMgr()

    # copy out only known subfolders
    for name in ["antivirus", "safety", "squid"]:

        fsmgr.copy_folder(
            os.path.join(source_dir, name),
            os.path.join(target_dir, name)
        )

    # copy out the known top level files
    for name in ["config.json"]:

        fsmgr.copy_file(
            os.path.join(source_dir, name),
            os.path.join(target_dir, name)
        )

    # finally reset the owner
    fsmgr.reset_owner(target_dir, SquidUser.name(), SquidUser.name())

#
# main
#
def main():

    # create parser
    parser = argparse.ArgumentParser(description='ConfMgr.py configuration manipulation script.')

    # add args
    parser.add_argument(
        "--config", required=True, help="full path to folder with generated configuration"
    )
    
    # parse actual arguments
    args = parser.parse_args()

    # ok let's go
    perform_action(args)

if __name__ == "__main__":

    # call the entry code
    main()

    # if we got here everything is fine
    sys.exit(0)
