import os
import platform

#
#
#
from .system import System

#
#
#
class Distrib:

    WS_WINDOWSX = "windowsx"
    WS_UBUNTU20 = "ubuntu20"
    WS_DEBIAN11 = "debian11"
    WS_REDHAT8  = "redhat8"
    
    @staticmethod
    def name():

        # return distrib based on system
        if System.WS_WINDOWS == System.name():
            return Distrib.WS_WINDOWSX
    
        if System.WS_LINUX == System.name():

            if os.path.isfile("/etc/redhat-release"):
                return Distrib.WS_REDHAT8

            if os.path.isfile("/etc/os-release"):

                with open("/etc/os-release", "r") as fin:
                    data = fin.read() 
                    if data.find("ubuntu") == -1:
                        return Distrib.WS_DEBIAN11

            # otherwise we consider this to be Ubuntu 20
            return Distrib.WS_UBUNTU20

        raise Exception("Cannot determine distrib for: %s" % System.name())

    @staticmethod
    def arch():

        # assume default
        value = "amd64"

        # for debugging
        if System.WS_WINDOWS == System.name():
            return value

        # it may also be arm
        try:
            with open("/proc/cpuinfo", "r") as fin:
                data = fin.read()

            if data.find("ARMv7") != -1:
                value = "armhf"
        except Exception as e:
            pass

        # and return
        return value