#
# helper script to get the list of available YouTube categories, run manually from time to time (between each release?)
#
import os
import json
import shutil
import requests
import tempfile


class CategoriesDownloader(object):

    def __init__(self, temp_dir):
        self.temp_dir = temp_dir

    def download(self):

        # construct get
        url  = "https://www.googleapis.com/youtube/v3/videoCategories"
        url += "?key=" + "AIzaSyCOFWadiilSpHXB0ECOIssmrxxkG0GluBY"
        url += "&part=snippet"
        url += "&regionCode=US"

        # set the the proxy
        proxies = {
            "http" : "http://proxy.diladele.lan:3128", 
            "https": "http://proxy.diladele.lan:3128"
        }
        r = requests.get(url, proxies=proxies)

        if r.status_code != 200:

            # server failed, raise and error
            raw_text = r.text
            raw_text = raw_text.replace("\n", "_")
            raw_text = raw_text.replace("\r", "_")

            raise Exception("YouTube APIv3 call failed; status %d, response (as one line - %s)" % (r.status_code, raw_text))

        # get the output
        data = r.json()

        # do minimal validation
        kind = data.get("kind", "")
        if kind != "youtube#videoCategoryListResponse":
            raise Exception("Unexpected kind %s" % kind)

        items = data.get("items", [])
        if len(items) < 10:
            raise Exception("Unexpected number of categories returned (%d)" % len(items))

        # write the json into file
        name = os.path.join(self.temp_dir, 'data.json')
        with open(name, 'w') as fout:
            fout.write(json.dumps(data))

        # and return the name
        return name

#
#
#
class CategoriesGenerator(object):

    def generate_prefix(self):

        return '''
#
# youtube category list - generated by m:/websafety/src.res/opt/websafety/var/console/_domain/safety/youtube.py 
#
class YouTubeCategories:

    # numeric categories
    CATEGORY_NONE                      = 0
'''

    def generate(self, source_file, target_file):

        # read the source file
        with open(source_file, 'r') as fin:
            data = json.loads(fin.read())

        # the text we combine
        output = ""    
        output += self.generate_prefix()

        # get the items            
        items = data["items"]

        if True:

            for item in items:

                kind = item["kind"]
                if kind != "youtube#videoCategory":
                    raise Exception("Unexpected category item kind %s" % kind)

                category_id = item["id"]
                snippet     = item["snippet"]
                channel_id  = snippet["channelId"]
                title       = snippet["title"]

                # special case - two comedies
                if title.upper() == "COMEDY" and category_id == "34":
                    title = "comedy1"
                
                output += "    CATEGORY_%-25s = %s\n" % (self.convert_title(title), category_id)


        if True:

            output += "\n\n"
            output += "    # list of available categories as array\n"
            output += "    CATEGORY_NAMES = [\n"

            for item in items:

                snippet = item["snippet"]
                title   = snippet["title"]
                output += "        \"%s\",\n" % self.convert_title(title).lower()

            output += "    ]\n"
            output += "\n\n"

        if True:

            output += "\n\n"
            output += "    @staticmethod\n"
            output += "    def map_name(name):\n"

            for item in items:

                category_id = item["id"]
                snippet     = item["snippet"]
                title       = snippet["title"]
                title       = self.convert_title(title).lower()

                output += "        if name == \"%s\":\n" % title
                output += "            return %d\n" % int(category_id)

            output += "        return 0\n"
            output += "\n\n"

            

            '''
            
            

                name_lower = name.lower()
                if name_lower == "adult_themes_sexuality":
                    return BuildInCategories.CATEGORY_ADULT_THEMES_SEXUALITY;

            @staticmethod
            def unmap_name(v):

                result = []
                if v & BuildInCategories.CATEGORY_ADULT_THEMES_SEXUALITY:
                    result.append("adult_themes_sexuality")
            '''


        # will write into out file
        with open(target_file, 'w') as fout:
            fout.write(output)

    def convert_title(self, title):

        assert(len(title) > 0)

        title = title.replace(" & ", "_and_")
        title = title.replace("/", "_")
        title = title.replace("-", "_")
        title = title.replace(" ", "_")
        title = title.upper()

        return title


#
#
#
def recreate_dir(dir_name):

    # remove old folder
    if os.path.exists(dir_name):
        
        print ("Removing old folder " + dir_name)
        shutil.rmtree(dir_name)
    
    os.makedirs(dir_name)

#
#
#
class YouTubeCategories(object):

    def generate(self, target_dir):

        # create temp dir name
        temp_dir = tempfile.mkdtemp(prefix="ytc")

        try:
            # this is the name of the final file
            name = "youtube_categories.py"

            # create the list of youtube categories in the temp dir
            self.generate_list(temp_dir, name)

            # copy temp into target
            shutil.copyfile( 
                os.path.join(temp_dir, name), 
                os.path.join(target_dir, name) 
            )

            # remove temp folder
            shutil.rmtree(temp_dir)

        except Exception as e:

            # error, dump it
            print ("")        
            print ("")
            print ("FATAL ERROR: see %s" % temp_dir)
            print ("")
            print ("")
            
            # and rethrow to stop execution
            raise

    def generate_list(self, temp_dir, name):

        # recreate folder
        recreate_dir(temp_dir)

        # download the list
        downloader = CategoriesDownloader(temp_dir)
        original   = downloader.download()
        
        # generate our python file on top of it
        l = CategoriesGenerator()
        l.generate(original, os.path.join(temp_dir, name))
        

def main():

    # generate final dirs
    base_dir = os.path.dirname(__file__)
    
    # generate
    YouTubeCategories().generate(base_dir)

    # dump the final output
    print ("")
    print ("")   
    print ("SUCCESS")
    print ("SUCCESS YOUTUBE CATEGORIES")
    print ("SUCCESS")
    print ("")
    print ("\tGenerated at: %s" % base_dir)
    print ("")
    print ("SUCCESS")
    print ("SUCCESS IWF")
    print ("SUCCESS")

if __name__ == '__main__':

    main()