import os
import logging

#
#
#
from daemon.core  import UiPath, SquidPath
from daemon.utils import JsonReader, ConfFileReader

#
#
#
class ConfigRealTime:

    def __init__(self, data):

        self.last_line_count       = data.get("last_line_count", 1000)
        self.regeneration_interval = data.get("regeneration_interval", 60)

#
#
#
class ConfigLogging:

    def __init__(self, data):

        self.dummy = data.get("dummy", False)

#
#
#
class ConfigCollector:

    def __init__(self, data):

        self.enable                         = data.get("enable", False)
        self.ignore_error_invalid_request   = data.get("ignore_error_invalid_request", True)
        self.ignore_error_transaction_end   = data.get("ignore_error_transaction_end", True)
        self.ignore_module_ads              = data.get("ignore_module_ads", False)
        self.ignore_module_adult_heuristics = data.get("ignore_module_adult_heuristics", False)
        self.ignore_module_adult_image      = data.get("ignore_module_adult_image", False)
        self.ignore_module_adult_phrases    = data.get("ignore_module_adult_phrases", False)
        self.ignore_module_adult_safesearch = data.get("ignore_module_adult_safesearch", False)
        self.ignore_module_adult_youtube    = data.get("ignore_module_adult_youtube", False)
        self.ignore_module_apps             = data.get("ignore_module_apps", False)
        self.ignore_module_clean            = data.get("ignore_module_clean", False)
        self.ignore_module_privacy          = data.get("ignore_module_privacy", False)
        self.ignore_module_sslbump          = data.get("ignore_module_sslbump", False)
        self.ignore_status_407              = data.get("ignore_status_407", True)
        self.ignore_status_non_aborted      = data.get("ignore_status_non_aborted", True)
        self.folders                        = data.get("folders", [])
        self.cluster                        = data.get("cluster", True)
        self.log_retention_days             = data.get("log_retention_days", 30)
        self.anonymized_policies            = data.get("anonymized_policies", [])

    def dump(self):

        logging.info("Collector configuration:")
        logging.info("\tenable               : %s" % self.enable)
        logging.info("\tanonymized_policies  : %s" % ", ".join(self.anonymized_policies))
        


#
#
#
class ConfigReporter:

    def __init__(self, data):

        self.max_detailed_users   = data.get("max_detailed_users", 50)
        self.max_detailed_domains = data.get("max_detailed_domains", 100)

    def dump(self):

        logging.info("Reporter configuration:")
        logging.info("\tmax_detailed_users   : %d" % self.max_detailed_users)
        logging.info("\tmax_detailed_domains : %d" % self.max_detailed_domains)

#
#
#
class Config:

    def __init__(self, logs_dir = "", stats_dir = "", reports_dir = ""):

        # set the folders
        self.logs_dir    = logs_dir
        self.stats_dir   = stats_dir
        self.reports_dir = reports_dir

        # verify if each one is NOT set explicitly, then set the default values
        if not self.logs_dir:
            self.logs_dir = SquidPath.log_dir()

        if not self.stats_dir:
            self.stats_dir = os.path.join(UiPath.var_dir(), "stats")

        if not self.reports_dir:
            self.reports_dir = os.path.join(UiPath.var_dir(), "reports")

        # cool, not read the configuration file from disk
        self.data = JsonReader(
            os.path.join(UiPath.etc_dir(), "traffic", "traffic.json")
        ).read_object()

        self.realtime  = ConfigRealTime(self.data.get("realtime", {}))
        self.logging   = ConfigLogging(self.data.get("logging", {}))
        self.collector = ConfigCollector(self.data.get("collector", {}))
        self.reporter  = ConfigReporter(self.data.get("reporter", {}))

    def dump(self):

        logging.info("Active configuration:")
        logging.info("\tLogs folder    : %s" % self.logs_dir)
        logging.info("\tStats folder   : %s" % self.stats_dir)
        logging.info("\tReports folder : %s" % self.reports_dir)

        self.collector.dump()
        self.reporter.dump()


    '''

        # read squid log format
        self.log_format = self.init_logformat()

    def init_logformat(self):

        # default log formats
        log_format_squid = r'%ts.%03tu %6tr %>a %Ss/%03>Hs %<st %rm %ru %[un %Sh/%<a %mt'

        file = os.path.join(DaemonPath.etc_dir(), "squid", "logfile.conf")
        if not os.path.isfile(file):

            # the log format file is not found, most probably this is NOT the
            # web safety managed instance of squid, then we will assume the 
            # default squid log format
            return log_format_squid

        # ok file is found, load it and see if web safety format is there
        prefix = "logformat websafety"

        lines = ConfFileReader(file).read_lines(skip_comments=True)
        for line in lines:
            if line.startswith(prefix):
                return line[len(prefix):].strip()

        # if we got here it is NOT there, return default then
        return log_format_squid
    '''
