#
#
#
import tld
from   urllib.parse import urlparse

#
#
#
from daemon.safety import BuildInCategories


#
# some utility functions
#
def contans_dot(domain):

    if domain.find(".") == -1:
        return False 
    return True

def try_parse_error(entry):

    if entry.request_url.startswith("error:"):

        # debug check
        assert entry.request_url in [
            "error:transaction-end-before-headers",
            "error:invalid-request"
        ]
        return entry.request_url

    # otherwise nothing
    return ""

def try_parse_url(entry):

    # for convinience
    value = entry.request_url

    # see if this is a cache protocol
    if value.startswith("cache_object://"):

        # imitate
        value = value.replace("cache_object://", "http://")

    # assume for all other methods we can use URL parsing library
    o = urlparse(value)

    # and assign
    domain = o.netloc

    # and return
    return domain


#
#
#
def is_offensive(categories_flags):

    #
    # raise the offensive flag - note BuildInCategories.CATEGORY_GAMES | \ is NOT offensive any more
    #
    offensive_flags = \
        BuildInCategories.CATEGORY_ADULT_THEMES_SEXUALITY    | \
        BuildInCategories.CATEGORY_ALCOHOL_TOBACCO           | \
        BuildInCategories.CATEGORY_CRACKS_HACKING_ILLEGAL    | \
        BuildInCategories.CATEGORY_DATING                    | \
        BuildInCategories.CATEGORY_DRUGS                     | \
        BuildInCategories.CATEGORY_ENTERTAINMENT_HUMOR_HOBBY | \
        BuildInCategories.CATEGORY_GAMBLING                  | \
        BuildInCategories.CATEGORY_NUDITY_PORNOGRAPHY        | \
        BuildInCategories.CATEGORY_P2P_FILE_SHARING          | \
        BuildInCategories.CATEGORY_PROXY_ANONYMIZER

    if categories_flags & offensive_flags:
        return 1
    else:
        return 0

#
#
#
class Enricher:

    def enrich(self, entry):

        self.enrich_user(entry)
        self.enrich_url(entry)
        self.enrich_offensive(entry)
        
    def enrich_user(self, entry):

        # first see if user name is set
        user_id = entry.requestor_name.strip()
        if user_id == "-":
            user_id = ""

        # then try to adapt if not
        if not user_id:
            user_id = entry.requestor_ip.strip()
        if not user_id:
            user_id = entry.requestor_mac.strip()

        # finally if nothing was set, put a dash for better looking reports
        if not user_id:
            user_id = "-"

        # save
        entry.user_id = user_id

    def enrich_offensive(self, entry):

        if is_offensive(entry.ws_categories):
            entry.ws_offensive = 1
        else:
            entry.ws_offensive = 0

    def enrich_url(self, entry):

        # set default values
        domain = ""
        
        # ok let's go
        if entry.request_method == "CONNECT":

            # asssume for now
            domain = entry.request_url

            # cut off the :443 port
            pos = domain.rfind(":")
            if pos != -1:
                domain = domain[:pos]

        else:

            domain = try_parse_error(entry)
            if not domain:
                domain = try_parse_url(entry)
                if not domain:
                    raise Exception("Enricher cannot parse the URL '%s' into domain, fldomain" % entry.request_url)

        # parse the first level domain out of domain
        fldomain = self.parse_fldomain(domain)

        # and assign
        entry.domain   = domain.lower()
        entry.fldomain = fldomain.lower()

    def parse_fldomain(self, domain):

        # debug check domain is valid
        assert(-1 == domain.find("://"))
        assert(len(domain) > 0)

        # no dot no parsing
        if not contans_dot(domain):
            return domain

        # try to parse using tld library
        obj = tld.get_tld("http://%s" % domain, as_object=True, fail_silently=True)
        if obj is not None:
            return obj.fld
        else:
            return domain

