import json
import datetime

#
#
#
from tld import get_fld

#
#
#
from daemon.safety import BuildInModules, BuildInCategories

#
#
#
class Entry:

    def __init__(self):

        # set default values of all members
        self.epoch             = 0
        self.timestamp         = ''
        self.date              = ''
        self.hour_of_day       = 0
        self.response_time     = 0
        self.user_id           = '-'

        self.requestor_ip      = ''   # the requestor_ fields are parsed from access log but not saved into stats storage
        self.requestor_mac     = ''   # instead we generate the user_id field out of these three; it is possible to adjust
        self.requestor_name    = ''   # the generation logic if needed by the administrator, see enricher.py

        self.squid_status      = ''
        self.status_code       = 0
        self.reply_size        = 0
        self.request_size      = 0    # total request payload (like body for POSTS) we use it in the upload reporting
        self.request_method    = ''
        self.request_url       = ''
        self.request_params    = ''
        self.domain            = ''
        self.fldomain          = ''
        self.mime_type         = ''
        self.ws_iid            = 0
        self.ws_duration       = 0
        self.ws_timing         = 0
        self.ws_mtime          = 0
        self.ws_scanflags      = 0
        self.ws_categories     = 0
        self.ws_trusted        = 0

        # verdict as reported by the web safety and more fields to ease working with pandas
        self.ws_verdict        = 'unknown'
        self.ws_verdict_pass   = 0
        self.ws_verdict_adapt  = 0
        self.ws_verdict_block  = 0
        self.ws_verdict_rescan = 0
        self.ws_verdict_skip   = 0
        self.ws_offensive      = 0

        # other members
        self.ws_policy         = '-'
        self.ws_member         = '-'
        self.ws_module         = 0
        self.ws_msgtype        = 0
        self.ws_param1         = ''
        self.ws_param2         = ''
        self.ws_debug          = ''

    @staticmethod
    def csv_header():
        return [
            'epoch',
            'timestamp', 
            'date',
            'hour_of_day',
            'response_time',
            'user_id', 
            'squid_status',
            'status_code', 
            'reply_size',
            'request_size',
            'request_method',
            'request_url',
            'request_params',
            'domain',
            'fldomain',
            'mime_type',
            'ws_iid',
            'ws_duration',
            'ws_timing',
            'ws_mtime',
            'ws_scanflags',
            'ws_categories',
            'ws_trusted',
            'ws_verdict',
            'ws_verdict_pass',
            'ws_verdict_adapt',
            'ws_verdict_block',
            'ws_verdict_rescan',
            'ws_verdict_skip',
            'ws_offensive',
            'ws_policy',
            'ws_member',
            'ws_module',
            'ws_msgtype',
            'ws_param1',
            'ws_param2',
            'ws_debug'
        ]

    def csv_row(self):
        return [
            self.epoch,
            self.timestamp.isoformat(), 
            self.date,
            self.hour_of_day,
            self.response_time,
            self.user_id, 
            self.squid_status,
            self.status_code,
            self.reply_size,
            self.request_size,
            self.request_method,
            self.request_url,
            self.request_params,
            self.domain,
            self.fldomain,
            self.mime_type,
            self.ws_iid,
            self.ws_duration,
            self.ws_timing,
            self.ws_mtime,
            self.ws_scanflags,
            self.ws_categories,
            self.ws_trusted,
            self.ws_verdict,
            self.ws_verdict_pass,
            self.ws_verdict_adapt,
            self.ws_verdict_block,
            self.ws_verdict_rescan,
            self.ws_verdict_skip,
            self.ws_offensive,
            self.ws_policy,
            self.ws_member,
            self.ws_module,
            self.ws_msgtype,
            self.ws_param1,
            self.ws_param2,
            self.ws_debug
        ]

    #
    # helper methods
    #
    def ws_categories_str(self):
        return ", ".join(BuildInCategories.unmap_name(self.ws_categories))

    def ws_module_str(self):
        return BuildInModules().unmap(self.ws_module)

    def contents(self):

        return " ".join(
            [
                self.requestor_ip, 
                self.requestor_mac, 
                self.requestor_name, 
                self.squid_status,
                self.request_url,
                str(self.ws_iid),
                self.ws_verdict,
                self.ws_policy,
                self.ws_member,
                self.ws_param1,
                self.ws_param2
            ]
        )


    