#
#
#
from daemon.safety import BuildInModules

#
#
#
class Excludes:

    def __init__(self, config):

        # save config        
        self.config  = config
        self.modules = BuildInModules()

        # fill errors check
        self.skip_errors = []
        if True:
            if config.collector.ignore_error_invalid_request:
                self.skip_errors.append("error:invalid-request")
            if config.collector.ignore_error_transaction_end:
                self.skip_errors.append("error:transaction-end-before-headers")

    def shall_skip(self, entry):

        # for now we skip some error entries
        if entry.request_url in self.skip_errors:

            # yes skip it
            return True

        # skipping NONE_ABORTED status (happens when client closes the connection before sending out any data at all)
        if self.config.collector.ignore_status_non_aborted:

            if entry.squid_status in ['NONE_ABORTED']:
                return True

        # skip 407 Proxy Authorization required records
        if self.config.collector.ignore_status_407:

            if entry.status_code in [407]:
                return True

        # let the child classes do the exclusion too
        return self.shall_skip_impl(entry)

#
#
#
class CronExcludes(Excludes):

    def shall_skip_impl(self, entry):

        # see what module we have
        module = self.modules.unmap(entry.ws_module)

        # now see if this module is to be excluded from collector
        if self.config.collector.ignore_module_ads:

            if module == "adblock":
                return True

        if self.config.collector.ignore_module_privacy:

            if module == "privacy":
                return True

        if self.config.collector.ignore_module_adult_heuristics:

            if module == "adult_heuristics":
                return True

        if self.config.collector.ignore_module_adult_image:

            if module == "adult_image":
                return True

        if self.config.collector.ignore_module_adult_phrases:

            if module == "adult_phrases":
                return True

        if self.config.collector.ignore_module_adult_safesearch:

            if module == "adult_safesearch":
                return True

        if self.config.collector.ignore_module_adult_youtube:

            if module == "adult_youtube":
                return True

        if self.config.collector.ignore_module_clean:

            if module == "clean":
                return True

        if self.config.collector.ignore_module_sslbump:

            if module == "sslbump":
                return True

        if self.config.collector.ignore_module_apps:

            if module == "apps":
                return True

        # no entry is fine, process it
        return False

#
#
#
class NowExcludes(Excludes):

    def shall_skip_impl(self, entry):        
        return False
