import os
import re

#
# constructs array of all available log files available in the file system
#
class Logs:

    def __init__(self, folders):

        # save the folders to enumerate
        self.folders = folders
        self.files   = self._enumerate(folders)

    def _enumerate(self, folders):

        files = []
        for folder in folders:
            files.extend(
                self._enumerate_folder(folder)
            )

        # now sort the files
        files = self._sorted(files)

        # and return nicely
        return files

    def _enumerate_folder(self, folder):

        # no folder, no entries
        if not os.path.isdir(folder):
            return []

        # this is the value to return
        files = []

        # list it
        for entry in os.listdir(folder):

            if "access.log" == entry:
                files.append(os.path.join(folder, entry))
            elif entry in ["access.log.0", "access.log.1"]:
                files.append(os.path.join(folder, entry))
            else:
                matched = re.match( r'^access\.log\.\d+\.gz$', entry, re.M|re.I )
                if matched:
                    files.append(os.path.join(folder, entry))

        return self._sorted(files)

    def by_age(self, file):

        matched = re.match( r'.*access\.log\.(\d+)\.gz', file, re.M|re.I )
        if matched:
            return int(matched.group(1)) + 1

        if file.endswith("access.log.1"):
            return 2

        if file.endswith("access.log.0"):
            return 1
            
        return 0

    def _sorted(self, files):

        # do in place sort
        files.sort(reverse=True, key=self.by_age)

        # and return it
        return files
