import os
import shutil
import logging
import datetime

#
#
#
from .history import History


#
# walks the statistics dir and purges all old CSV files
#
class CronPurger:

    def __init__(self, config):

        # store dirs        
        self.config    = config
        self.stats_dir = config.stats_dir

    def purge(self):

        logging.info("Purging statistics older than %d days ...", self.config.collector.log_retention_days)

        # these are folders to purge
        to_purge = []

        # create history
        history = History(self.stats_dir)

        # loop over years
        for year in history.years():

            # loop over months
            for month in history.months(year):

                # loop over days
                for day in history.days(year, month):

                    # construct the date
                    date_obj = datetime.datetime(year, month, day)
                    date_str = date_obj.strftime("%Y-%m-%d")

                    # see if we need to purge
                    if not self.needs_purging(date_obj):

                        # log it
                        logging.info("\tDate %s still fresh, skipping...", date_str)

                        # and skip
                        continue

                    # yes we must purge it, construct the folder name
                    folder = os.path.join(self.stats_dir, date_str)

                    # and append
                    if os.path.isdir(folder):
                        to_purge.append(folder)

        # now just purge all
        for folder in to_purge:

            # log it
            logging.info("\tDeleting folder %s...", folder)

            # and drop
            shutil.rmtree(folder)

        logging.info("Purging successfully completed")

    def needs_purging(self, date_obj):

        diff = abs((datetime.datetime.today() - date_obj).days)
        if diff > self.config.collector.log_retention_days:
            return True

        return False

