import os
import hashlib

#
#
#
from daemon.utils import JsonReader, JsonWriter

#
# represents the state of the collector - we do NOT need to re-collect access logs that were already collected
#
class State:

    def __init__(self, stats_dir):

        # save the state dir
        self.state_dir  = stats_dir
        self.state_file = os.path.join(stats_dir, "collector.state")

        # this is the state data
        self.data = {}

        # see if the state file exists
        if os.path.exists(self.state_file):

            # and load it
            self.data = JsonReader(self.state_file).read_object()

    def mark_as_collected(self, files):

        # optimization
        if len(files) == 0:
            return 

        # fill the data
        for (name, checksum) in files:
            self.data[checksum] = name

        # and save
        self.save()

    def save(self):

        # and save it
        w = JsonWriter(self.state_file)
        w.write(self.data)

    def shall_collect(self, file):

        # these are the values to return
        (name, checksum, collect) = (file, "", False)

        # if no such file exists, no need to collect
        if not os.path.isfile(file):

            # no need to collect
            return (name, checksum, collect)

        # calculate the checksum of the file contents
        checksum = self.calculate_checksum(file)

        # see if it is in the state file
        if self.was_collected(file, checksum):

            # no need to collect
            return (name, checksum, False)

        # othewise we need to parse it
        return (name, checksum, True)

    def calculate_checksum(self, file):

        # open the file
        md5 = hashlib.md5()
        with open(file, "rb") as fin:

            # read it over by 4K blocks
            for byte_block in iter(lambda: fin.read(4096), b""):
                md5.update(byte_block)

        # and calculate digest
        return md5.hexdigest().upper()

    def was_collected(self, file, checksum):

        if checksum in self.data:
            return True
        else:
            return False
