import os
import csv
import gzip
import shutil

#
#
#
from .entry import Entry

#
#
#
def make_folder(dir_name):

    if os.path.exists(dir_name):
        shutil.rmtree(dir_name)

    # create it
    os.mkdir(dir_name)



#
#
#
class Writer:

    def __init__(self, out_dir, date):

        # debug just in case
        assert(os.path.isdir(out_dir))

        # write will be done into temporary folder
        tmp_dir = os.path.join(out_dir, date) + ".temp"

        # make it
        make_folder(tmp_dir)

        # create csv file in the temp dir
        name = os.path.join(tmp_dir, 'data.csv')

        # open it
        fout = open(name, 'w', newline='', encoding='utf-8')
        if not fout:
            raise Exception("Cannot open the '%s' for writing." % name)

        # create writer
        writer = csv.writer(fout, delimiter='\t', quotechar='|', quoting=csv.QUOTE_MINIMAL)

        # and write the header
        writer.writerow(Entry.csv_header())

        # good we are fully initialized, save members
        self.date    = date
        self.out_dir = out_dir
        self.tmp_dir = tmp_dir
        self.fout    = fout
        self.writer  = writer
        self.name    = name

    def append(self, entry):

        # see if the date is the same
        assert(self.date == entry.date)

        # otherwise write to csv directly
        self.writer.writerow(entry.csv_row())

    def flush(self):

        # close the csv file
        self.fout.close()

        # repack it as gzip
        if True:
            with open(self.name, 'rb') as fin:
                with gzip.open(self.name + '.gz', 'wb') as fout:
                    fout.writelines(fin)

        # remove the original
        os.unlink(self.name)

        # construct the actual output folder
        cur_dir = os.path.join(self.out_dir, self.date)
        
        # remove it
        if os.path.exists(cur_dir):
            shutil.rmtree(cur_dir)

        # rename the temp dir into current
        os.rename(self.tmp_dir, cur_dir)