import os
import logging

#
#
#
from daemon.core  import UiPath
from daemon.utils import JsonReader, ConfFileReader

#
#
#
class ConfigMailer:

    def __init__(self, data):

        self.use_auth = data.get("use_auth", True)
        self.server   = data.get("server", "")
        self.port     = data.get("port", 587)
        self.password = data.get("password", "")
        self.username = data.get("username", "")

    def dump(self):

        logging.info("Mailer configuration:")
        logging.info("\tserver   : %s" % self.server)
        logging.info("\tport     : %s" % self.port)
        logging.info("\tusername : %s" % self.username)
        logging.info("\tpassword : %s" % "*** masked ***") # self.password
        logging.info("\tuse_auth : %s" % self.use_auth)
        


#
#
#
class ConfigFilters:

    def __init__(self, data):

        # exclusions
        self.exclude_domains  = data.get("exclude_domains", [])
        self.exclude_policies = data.get("exclude_policies", [])
        self.exclude_users    = data.get("exclude_users", [])

        # inclusions
        self.include_domains  = data.get("include_domains", [])
        self.include_policies = data.get("include_policies", [])
        self.include_users    = data.get("include_users", [])

        # record limits
        self.limit_n_drilldown = data.get("limit_n_drilldown", 50)
        self.limit_n_entries   = data.get("limit_n_entries", 50)

#
#
#
class ConfigTimeFrame:

    def __init__(self, data):

        self.time_from       = data.get("from", None)
        self.time_to         = data.get("to", None)
        self.time_type       = data.get("type", "last_n_days")
        self.include_current = data.get("include_current", True)
        self.last_value      = data.get("value", 7)

#
#
#
class ConfigReport:

    def __init__(self, data, mailer):

        # save mailer
        self.mailer = mailer

        # init top level properties
        self.name        = data.get("name", "")
        self.comments    = data.get("comments", "")
        self.report_type = data.get("type", "")

        # debug check everything is fine
        if not self.name:
            raise Exception("cannot construct report with empty name")

        if not self.report_type:
            raise Exception("cannot construct report with unknown or empty type")

        # read actions
        actions = data.get("actions", {})

        # and set other parameters
        self.send_to_emails = actions.get("send_to_emails", [])
        self.filters        = ConfigFilters(data.get("filters", {}))
        self.timeframe      = ConfigTimeFrame(data.get("timeframe", {}))

    def dump(self):

        logging.debug("Report configuration:")
        logging.debug("\tname           : %s" % self.name)
        logging.debug("\treport_type    : %s" % self.report_type)
        logging.debug("\tsend_to_emails : %s" % self.send_to_emails)
    


#
#
#
class ConfigPeriodic:

    def __init__(self, data, mailer):

        self.reports = []
        for item in data:
            self.reports.append(ConfigReport(item, mailer))

    def dump(self):

        for item in self.reports:
            item.dump()


#
#
#
class Config:

    def __init__(self, stats_dir = "", reports_dir = ""):

        logging.debug("Constructing configuration object ...")

        # set the folders
        self.stats_dir   = stats_dir
        self.reports_dir = reports_dir

        # verify if each one is NOT set explicitly, then set the default values
        if not self.stats_dir:
            self.stats_dir = os.path.join(UiPath.var_dir(), "stats")

        if not self.reports_dir:
            self.reports_dir = os.path.join(UiPath.var_dir(), "reports", "periodic")

        # mailer settings are stored in traffic.json
        data0 = JsonReader(
            os.path.join(UiPath.etc_dir(), "traffic", "traffic.json")
        ).read_object()

        self.mailer = ConfigMailer(data0.get("mailer", {}))

        # list of report are in periodic_reports.json
        data1 = JsonReader(
            os.path.join(UiPath.etc_dir(), "traffic", "periodic_reports.json")
        ).read_array()

        self.periodic = ConfigPeriodic(data1, self.mailer)

    def dump(self):

        logging.info("Active configuration:")
        logging.info("\tStats folder   : %s" % self.stats_dir)
        logging.info("\tReports folder : %s" % self.reports_dir)

        self.mailer.dump()
        self.periodic.dump()
