import smtplib

#
#
#
from email.mime.multipart import MIMEMultipart
from email.mime.text      import MIMEText

#
#
#
class Mailer:

    def __init__(self, config):

        self.config = config

    def send(self, recipients, report_html, subject):

        # the type of the recipients shall be a string, for example user1@domain, user2@domain etc
        assert(type(recipients) == str)

        # create message container - the correct MIME type is multipart/alternative.
        msg = MIMEMultipart('alternative')
        
        msg['From']    = self.config.username
        msg['To']      = recipients 
        msg['Subject'] = subject

        # get the html contents
        contents = ""
        with open(report_html, "r") as fin:
            contents = fin.read()

        # see the code below to use template as body
        body_text = "The text message part is empty, please see the HTML message part."
        body_html = contents

        # create the bodies of the message (a plain-text and an HTML version).
        part1 = MIMEText(body_text, 'plain')
        part2 = MIMEText(body_html, 'html')

        # according to RFC 2046, the last part of a multipart message, is best and preferred
        msg.attach(part1)
        msg.attach(part2)

        # create SMTP connection
        mail = smtplib.SMTP(
            self.config.server,
            self.config.port,
            timeout=20
        )

        # convert the recipients to array of strings
        addresses = recipients.split(",")

        # we always start TLS on the SMTP connection
        mail.starttls()

        # login if asked to
        if self.config.use_auth:
            mail.login(self.config.username, self.config.password)

        # and send message
        mail.sendmail(self.config.username, addresses, msg.as_string())

        # ok then
        mail.quit()
