import os
import numpy  as np

#
#
#
from django.template import loader, Context

#
#
#
from daemon.utils import JsonWriter

#
#
#
from .reporter import Reporter
from .renderer import Renderer
from .loader   import Loader

#
#
#
class BlockedUsers(Reporter):

    template_path = "blocked_users.html"

    def report_impl(self, folder, config, df):

        # generate
        (view, totals) = self.generate(config, df)

        # save view
        view.to_csv(os.path.join(folder, "data.csv"), encoding='utf-8')

        # save totals
        JsonWriter(os.path.join(folder, "data.json")).write(totals)

        # now render them as html
        text = Renderer().render(
            self.template_path, {
                "view"   : view,
                "totals" : totals,
                "config" : config,
                "meta"   : self.get_meta()
            }
        )

        # and save
        with open(os.path.join(folder, "index.html"), "w") as fout:
            fout.write(text)

    def generate(self, config, df):

        # calculate view
        view   = self.calculate_view(df)
        totals = self.calculate_totals(view)

        # get only top N
        view = view[:config.filters.limit_n_entries]

        # and return
        return (view, totals)

    def calculate_view(self, df):

        # select ONLY blocked sites
        view = df.loc[
            df["ws_verdict_block"] == 1
        ]

        # group records by users with accumulated reply_size
        view = view.groupby(
            ["user_id"]
        ).agg({ 
            'request_url'   : 'count', 
            'reply_size'    : 'sum', 
            'user_id'       : 'nunique'
        }).sort_values("request_url", ascending=False)

        # calculate percentage of size
        if True:

            total_sum = view["request_url"].sum()
            if total_sum > 0:
                view["hits_percent"] = np.ceil(100 * view["request_url"] / total_sum).astype(int)
            else:
                view["hits_percent"] = 0

        # and return nicely
        return view

    def calculate_totals(self, view):

        return {
            'request_url_total': int(view['request_url'].sum()),
            'reply_size_total' : int(view['reply_size'].sum()),
        }



    def get_loader(self):

        parse_dates = []
        usecols     = [
            "request_url",
            "reply_size",
            "user_id",
            "ws_verdict_block",
            "fldomain", 
            "ws_policy",
        ]
        dtype       = {
            "reply_size"       : "uint64",
            "ws_verdict_block" : "uint8"
        }
        
        return Loader(parse_dates, usecols, dtype)


