import os
import numpy  as np

#
#
#
from django.template import loader, Context

#
#
#
from daemon.utils import JsonWriter

#
#
#
from .reporter import Reporter
from .renderer import Renderer
from .loader   import Loader

#
#
#
class TopUsersBySize(Reporter):

    template_path = "top_users_by_size.html"

    def report_impl(self, folder, config, df):

        # generate
        (view, totals) = self.generate(config, df)

        # save view
        view.to_csv(os.path.join(folder, "data.csv"), encoding='utf-8')

        # save totals
        JsonWriter(os.path.join(folder, "data.json")).write(totals)

        # now render them as html
        text = Renderer().render(
            self.template_path, {
                "view"   : view,
                "totals" : totals,
                "config" : config,
                "meta"   : self.get_meta()
            }
        )

        # and save
        with open(os.path.join(folder, "index.html"), "w") as fout:
            fout.write(text)

    def generate(self, config, df):

        # calculate view
        view   = self.calculate_view(df)
        totals = self.calculate_totals(view)

        # get only top N
        view = view[:config.filters.limit_n_entries]

        # and return
        return (view, totals)

    def calculate_view(self, df):

        # group records by domain with accumulated reply_size
        view = df.groupby(
            ["user_id"]
        ).agg(
            { 'reply_size': 'sum' }
        ).sort_values("reply_size", ascending=False)

        # calculate percentage of size
        if True:

            total_sum = view["reply_size"].sum()
            if total_sum > 0:
                view["bandwidth_percent"] = np.ceil(100 * view["reply_size"] / total_sum).astype(int)
            else:
                view["bandwidth_percent"] = 0

        # and return nicely
        return view

    def calculate_totals(self, view):

        return {
            'reply_size_total': int(view['reply_size'].sum()),
        }


    def get_loader(self):

        parse_dates = []
        usecols     = [
            "reply_size",
            "fldomain", 
            "user_id",
            "ws_policy"
        ]
        dtype       = {
            "reply_size"    : "uint64"
        }
        
        return Loader(parse_dates, usecols, dtype)


