import os
import numpy  as np

#
#
#
from django.template import loader, Context

#
#
#
from daemon.utils  import JsonWriter
from daemon.safety import categories_to_strings

# for now like this ugly
from daemon.stats.reporter.builders._impl.specific.generators import VideosGenerator

#
#
#
from .reporter import Reporter
from .renderer import Renderer
from .loader   import Loader

#
#
#
class YouTubeVideos(Reporter):

    template_path = "youtube_videos.html"

    def report_impl(self, folder, config, df):

        # generate records of all video accesses
        view = VideosGenerator().generate(df)

        # save it
        view.to_csv(os.path.join(folder, "data.csv"), encoding='utf-8')

        # generate list of videos and viewers
        videos  = self.list_of_videos(view)

        # for each video add a viewers table
        self.generate_viewers(config, videos, view)

        # now render them as html
        text = Renderer().render(
            self.template_path, {
                "videos" : videos,
                "config" : config,
                "meta"   : self.get_meta()
            }
        )

        # and save
        with open(os.path.join(folder, "index.html"), "w", encoding='utf-8') as fout:
            fout.write(text)

    def list_of_videos(self, df):

        # group by video id
        videos = df.groupby(
            ["video_id"]
        ).agg(
            { 
                'video_title'  : 'first',
                'video_desc'   : 'first',
                'ws_iid'       : 'count',
                'user_id'      : 'nunique'
            }
        ).sort_values("ws_iid", ascending=False)

        # rename some colums to look better
        videos.rename(columns={'ws_iid':'views', 'user_id': 'users'}, inplace=True)

        # and return nicely
        return videos

    def generate_viewers(self, config, videos, df):

        # the number of records to store in a drill down per video id
        limit_n = config.filters.limit_n_drilldown

        # this is the column to append to videos dataframe
        column = []

        for index, row in videos.iterrows():

            # select the viewers
            viewers = df.loc[df['video_id'] == index ]

            # group by user id
            users = viewers.groupby(
                ["user_id"]
            ).agg(
                { 'ws_iid' : 'count' }
            ).sort_values("ws_iid", ascending=False)

            # we only take N entries 
            users.head(limit_n)

            # and save them to column
            column.append(users)

        # finally add the column into videos dataframe
        assert(len(column) == len(videos.index))

        # assign
        videos["viewers"] = column

        # and return
        return

    def get_loader(self):

        parse_dates = []
        usecols     = [
            "ws_iid", 
            "timestamp", 
            "user_id", 
            "request_url", 
            "request_params",
            "fldomain", 
            "ws_policy"
        ]
        dtype = {
            "ws_iid" : "uint64"
        }
        
        return Loader(parse_dates, usecols, dtype)


