import os
import datetime
import calendar
import pandas as pd
import numpy  as np

#
#
#
class HitsPerMonthAggregator:

    def aggregate(self, df):

        # sort df by timestamp
        df = df.sort_values("timestamp", ascending=True)

        # leave only needed columns and set index to timestamp
        df = df[ 
            ["timestamp", "ws_verdict_pass", "ws_verdict_adapt", "ws_verdict_block", "ws_verdict_skip"] 
        ].set_index("timestamp")

        # now resample the table into 1 day intervals - each verdict type now is summed
        resampled = df.resample('24H').sum()

        # we now need to extend the dataframe to whole month, so lets see the min date and then take the first
        date = datetime.datetime.date(df.index.min())        
        date = date.replace(day=1)

        # see how many days we have in this month
        count = 0
        if True:

            # min and max day in the month
            (m1, m2) = calendar.monthrange(date.year, date.month)

            # and range
            count = m2 - m1
            count = count + 1

        # create new index for the full month based on the first date and count of days
        index = pd.period_range(date, periods=count, freq='D').to_timestamp()

        # and reindex the resampled data
        resampled1 = resampled.reindex(index=index, fill_value=0)

        # set the index name for better looking csv
        resampled1.index.name = "timestamp"

        # add total column
        resampled1['total']= resampled1.sum(axis=1)

        # add epoch column (timestamp converted to epoch, needed for javascript)
        resampled1['epoch'] = resampled1.index.astype(np.int64) / 1000000

        # and return nicely
        return resampled1


#
#
#
class HitsPerYearAggregator:

    def aggregate(self, df):

        # sort df by timestamp
        df = df.sort_values("timestamp", ascending=True)

        # leave only needed columns and set index to timestamp
        df = df[ 
            ["timestamp", "ws_verdict_pass", "ws_verdict_adapt", "ws_verdict_block", "ws_verdict_skip"] 
        ].set_index("timestamp")

        # now resample the table into 1 day intervals - each verdict type now is summed
        resampled = df.resample('M').sum()

        # we now need to extend the dataframe to whole month, so lets see the min date and then take the first
        date = datetime.datetime.date(df.index.min())
        date = date.replace(day=1, month=1)

        # create new index for the full year based on the first date and count of days
        index = pd.date_range(date, periods=12, freq='M')

        # and reindex the resampled data
        resampled1 = resampled.reindex(index=index, fill_value=0)

        # set the index name for better looking csv
        resampled1.index.name = "timestamp"

        # add total column
        resampled1['total']= resampled1.sum(axis=1)

        # add epoch column (timestamp converted to epoch, needed for javascript)
        resampled1['epoch'] = resampled1.index.astype(np.int64) / 1000000

        # and return nicely
        return resampled1
