import pandas as pd
import numpy  as np


#
#
#
class RequestorAggregator:

    def domains(self, df):

        # construct table and totals
        view   = self.calculate_table(df)
        totals = self.calculate_totals(view)

        # and return nicely
        return (view, totals)

    def calculate_table(self, df):

        # group records by domain, then calculate hits and offensive hits per group
        view = df.groupby(
            ["fldomain"]
        ).agg(
            { 
                "requests"          : 'sum',
                "reply_size"        : 'sum',
                "request_size"      : 'sum',
                "ws_verdict_pass"   : 'sum',
                "ws_verdict_adapt"  : 'sum',
                "ws_verdict_block"  : 'sum',
                "ws_verdict_rescan" : 'sum',
                "ws_verdict_skip"   : 'sum',
                "ws_offensive"      : 'sum'
            }
        )

        # calculate percentage of requests
        if True:

            total_sum = view["requests"].sum()
            if total_sum > 0:
                view["requests_percent"] = np.ceil(100 * view["requests"]/total_sum ).astype(int)
            else:
                view["requests_percent"] = 0

        # calculate percentage of downloads
        if True:

            total_sum = view["reply_size"].sum()
            if total_sum > 0:
                view["bandwidth_percent"] = np.ceil(100 * view["reply_size"] /total_sum ).astype(int)
            else:
                view["bandwidth_percent"] = 0

        # calculate percentage of uploads
        if True:

            total_sum = view["request_size"].sum()
            if total_sum > 0:
                view["request_size_percent"] = np.ceil(100 * view["request_size"] /total_sum ).astype(int)
            else:
                view["request_size_percent"] = 0

        # rename columns
        # view.rename(columns={"fldomain":'requests'}, inplace=True)

        # sort them
        view = view.sort_values('reply_size', ascending=False)

        # and return nicely
        return view

    def calculate_totals(self, view):

        return {
            'total_requests'          : int(view['requests'].sum() ),
            'total_reply_size'        : int(view['reply_size'].sum() ),
            'total_request_size'      : int(view['request_size'].sum() ),
            'total_ws_verdict_pass'   : int(view['ws_verdict_pass'].sum()),
            'total_ws_verdict_adapt'  : int(view['ws_verdict_adapt'].sum()),
            'total_ws_verdict_block'  : int(view['ws_verdict_block'].sum()),
            'total_ws_verdict_rescan' : int(view['ws_verdict_rescan'].sum()),
            'total_ws_verdict_skip'   : int(view['ws_verdict_skip'].sum()),
            'total_ws_offensive'      : int(view['ws_offensive'].sum())
        }
