import os
import logging

#
#
#
from .builder_summary  import *
from .builder_hits     import *
from .builder_bytes    import *
from .builder_items    import *
from .builder_item     import *


#
# periodic builder - builds reports by aggregating data from multiple days
#
class Builder:

    def __init__(self, builder_type, folder):

        # make the folder if not yet there
        self.builder_type = builder_type
        self.folder       = folder

    def build(self, periods):

        # summary
        SummaryBuilder(self.folder).build(periods)

        # hits per day
        HitsBuilder(self.builder_type, self.folder).build(periods)

        # bytes per day
        BytesBuilder(self.builder_type, self.folder).build(periods)

        # run various builders
        self.build_users(periods)
        self.build_domains(periods)

    def build_users(self, periods):

        users = UsersBuilder(self.folder).build(periods)
        for user in users:

            r = UserBuilder(self.builder_type, self.folder, user)
            r.build(periods)

    def build_domains(self, periods):

        # finally dump all domains
        domains = DomainsBuilder(self.folder).build(periods)
        for domain in domains:

            # and then report each domain
            r = DomainBuilder(self.builder_type, self.folder, domain)
            r.build(periods)
    