import os
import pandas as pd

#
#
#
from .aggregators import BytesPerMonthAggregator, BytesPerYearAggregator

#
#
#
class BytesBuilder:


    def __init__(self, builder_type, folder):

        self.folder       = folder
        self.builder_type = builder_type

    def build(self, periods):

    	# combine all bytes into one big data frame
        df = self.load_bytes(periods)

        # call actual aggregator based on the type of build
        if self.builder_type == "month":
            n = BytesPerMonthAggregator().aggregate(df)
        else:
            n = BytesPerYearAggregator().aggregate(df)

        # save to csv
        n.to_csv(
            os.path.join(self.folder, "bytes.csv"), encoding='utf-8'
        )

    def load_bytes(self, periods):

        # list of loaded dataframes
        loaded = []

        # load all folders
        for period in periods:

            file = os.path.join(period, "bytes.csv")
            if not os.path.isfile(file):
                continue

            loaded.append(
                self.load_byte(file)
            )

        # merge into one dataframe
        return pd.concat(loaded, axis=0, ignore_index=True, sort=False)

    def load_byte(self, file):

        return pd.read_csv(file, parse_dates=[0])
