import os
import pandas as pd

#
#
#
from .aggregators import HitsPerMonthAggregator, HitsPerYearAggregator

#
#
#
class HitsBuilder:

    def __init__(self, builder_type, folder):

        self.folder       = folder
        self.builder_type = builder_type

    def build(self, periods):

    	# combine all hits into one big data frame
        df = self.load_hits(periods)

        # call actual aggregator based on the type of build
        if self.builder_type == "month":
            n = HitsPerMonthAggregator().aggregate(df)
        else:
            n = HitsPerYearAggregator().aggregate(df)

        # save to csv
        n.to_csv(
            os.path.join(self.folder, "hits.csv"), encoding='utf-8'
        )

    def load_hits(self, periods):

        # list of loaded dataframes
        loaded = []

        # load all folders
        for period in periods:

            file = os.path.join(period, "hits.csv")
            if not os.path.isfile(file):
                continue

            loaded.append(
                self.load_hit(file)
            )

        # merge into one dataframe
        return pd.concat(loaded, axis=0, ignore_index=True, sort=False)

    def load_hit(self, file):

        return pd.read_csv(file, parse_dates=[0])
