import os

#
#
#
from daemon.utils import JsonWriter, remake_folder

#
#
#
from .aggregators import *



#
#
#
class ItemsBuilder:

    def __init__(self, folder):

        # construct subfolder to store generated files
        self.folder = os.path.join(folder, self.sub_folder)

        # ensure it exists
        remake_folder(self.folder)

    def build(self, periods):

        # combine data from periods into one big dataframe
        df = self.load_dataframes(periods)

        # create the aggregator
        a = self.aggregators_class()

        # generate items and total
        view, totals = a.aggregate(df)

        # save them
        self.save(view, totals)

        # create the list of items
        result = []

        # loop over each entry in the view
        for entry, row in view.iterrows():
            result.append(entry)

        # and return it
        return result

    def load_dataframes(self, periods):

        # list of loaded dataframes
        loaded = []

        # load all folders
        for period in periods:

            loaded.append(
                self.load_dataframe(period)
            )

        # merge into one dataframe
        return pd.concat(loaded, axis=0, ignore_index=True, sort=False)

    def load_dataframe(self, period):

        return pd.read_csv(
            os.path.join(period, self.sub_folder, "items.csv")
        )

    def save(self, view, totals):

        # save table
        view.to_csv(
            os.path.join(self.folder, "items.csv"), encoding='utf-8'
        )

        # save totals
        JsonWriter(os.path.join(self.folder, "items.json")).write(totals)


#
#
#
class UsersBuilder(ItemsBuilder):

    aggregators_class = UsersAggregator
    sub_folder        = "users"
    
#
#
#
class DomainsBuilder(ItemsBuilder):

    aggregators_class = DomainsAggregator
    sub_folder        = "domains"