import os
import json
import codecs
import pandas as pd

#
#
#
from daemon.utils import JsonWriter

#
#
#
from .aggregators import SummaryAggregator


#
#
#
class SummaryBuilder:

    def __init__(self, folder):

        # save the target folder to write data into
        self.folder = folder

    def build(self, periods):

        # combine summaries from all periods into one big data frame
        df = self.load_summaries(periods)

        # call the actual aggregator
        n = SummaryAggregator().aggregate(df)

        # save it as JSON 
        JsonWriter(os.path.join(self.folder, "summary.json")).write(n)


    def load_summaries(self, periods):

        # list of loaded dataframes
        loaded = []

        # load all periods
        for period in periods:

            file = os.path.join(period, "summary.json")
            if not os.path.isfile(file):
                continue
            else:
                loaded.append(self.load_summary(file))

        # merge into one dataframe
        return pd.concat(loaded, axis=0, ignore_index=True, sort=False)

    def load_summary(self, file):

        # load values
        with codecs.open(file, "r", encoding="utf-8") as fin:
            data = json.load(fin)

        # create a new dataframe
        df = pd.DataFrame(
            [data]
        ) 

        # and return it
        return df