import os
import logging
import pandas as pd

#
#
#
from daemon.utils import JsonWriter

#
#
#
from .generators   import *
from .builder_item import ItemBuilder

    
#
# builder for user
#
class UserBuilder(ItemBuilder):

    def build_impl(self, df):

        # generate requestor independent files
        self.build_domains_all(df)
        self.build_domains_blocked(df)
        self.build_domains_offensive(df)
        self.build_time_of_day(df)
        self.build_youtube_videos(df)
        self.build_searches(df)
        
    def build_domains_all(self, df):

        # create names
        name_csv  = os.path.join(self.folder, "domains_all.csv")
        name_json = os.path.join(self.folder, "domains_all.json")

        # build
        self.build_domains(df, name_csv, name_json)

    def build_domains_blocked(self, df):

        # create names
        name_csv  = os.path.join(self.folder, "domains_blocked.csv")
        name_json = os.path.join(self.folder, "domains_blocked.json")

        # select only blocked domains
        sub_df = df.loc[
            df['ws_verdict_block'] == 1
        ]

        # build
        self.build_domains(sub_df, name_csv, name_json)

    def build_domains_offensive(self, df):

        # create names
        name_csv  = os.path.join(self.folder, "domains_offensive.csv")
        name_json = os.path.join(self.folder, "domains_offensive.json")

        # select only offensive domains
        sub_df = df.loc[
            df['ws_offensive'] == 1
        ]

        # build
        self.build_domains(sub_df, name_csv, name_json)

    def build_domains(self, df, name_csv, name_json):

        # create generator class
        g = UserGenerator()

        # generate
        (v, t) = g.generate(df)

        # save view
        v.to_csv(name_csv, encoding='utf-8')

        # save totals
        JsonWriter(name_json).write(t)

    def build_time_of_day(self, df):

        # create generator class
        g = UserGenerator()

        # generate and save
        g.hourly_requests_per_domain(df).to_csv(
            os.path.join(self.folder, "hourly_requests_per_domain.csv"), encoding='utf-8'
        )
        g.hourly_bandwidth_per_domain(df).to_csv(
            os.path.join(self.folder, "hourly_bandwidth_per_domain.csv"), encoding='utf-8'
        )
        g.hourly_uploads_per_domain(df).to_csv(
            os.path.join(self.folder, "hourly_uploads_per_domain.csv"), encoding='utf-8'
        )

    def build_youtube_videos(self, df):

        g = VideosGenerator()
        g.generate(df).to_csv(
            os.path.join(self.folder, "videos.csv"), encoding='utf-8'
        )

    def build_searches(self, df):

        g = SearchesGenerator()
        g.generate(df).to_csv(
            os.path.join(self.folder, "searches.csv"), encoding='utf-8'
        )
   
#
# builder for individual domain
#
class DomainBuilder(ItemBuilder):

    def build_impl(self, df):

        # users
        self.build_requestors(
            df, 
            "requestor_name",
            os.path.join(self.folder, "users.csv"), 
            os.path.join(self.folder, "users.json")
        )

        # ips
        self.build_requestors(
            df, 
            "requestor_ip",
            os.path.join(self.folder, "ips.csv"), 
            os.path.join(self.folder, "ips.json")
        )

        # macs
        self.build_requestors(
            df, 
            "requestor_mac",
            os.path.join(self.folder, "macs.csv"), 
            os.path.join(self.folder, "macs.json")
        )

    def build_requestors(self, df, group_by, name_csv, name_json):

        # generate
        (v, t) = DomainGenerator().generate(df, group_by)

        # save view
        v.to_csv(name_csv, encoding='utf-8')

        # save totals
        JsonWriter(name_json).write(t)



    