import numpy  as np
import pandas as pd

#
#
#
class BytesPerHourGenerator:

    def generate(self, dataframe):

        # create a new dataframe with needed columns and set the index to timestamp
        df = dataframe[ 
            ["timestamp", "reply_size", "request_size"] 
        ].set_index("timestamp")

        # now resample the table into 1 hour intervals - each verdict type now is summed
        resampled = df.resample('1H').sum()

        # select the first value (we need in to generate the full day index)
        date = dataframe["date"].iloc[0]

        # create new index for the full day based on the date
        index = pd.period_range(date, periods=24, freq='H').to_timestamp()

        # and reindex the resampled data
        resampled1 = resampled.reindex(index=index, fill_value=0)

        # set the index name for better looking csv
        resampled1.index.name = "timestamp"

        # add epoch column (timestamp converted to epoch, needed for javascript)
        resampled1['epoch'] = resampled1.index.astype(np.int64) / 1000000

        return resampled1


#
#
#
class BytesPer10MinutesGenerator:

    def generate(self, dataframe):

        # create a new dataframe with needed columns and set the index to timestamp
        df = dataframe[ 
            ["timestamp", "reply_size", "request_size"]
        ].set_index("timestamp")

        # now resample the table into 1 hour intervals - each verdict type now is summed
        resampled = df.resample('600s').sum()

        # select the first value (we need in to generate the full day index)
        date = dataframe["date"].iloc[0]

        # create new index for the full day based on the date
        index = pd.period_range(date, periods=144, freq='600s').to_timestamp()

        # and reindex the resampled data
        resampled1 = resampled.reindex(index=index, fill_value=0)

        # set the index name for better looking csv
        resampled1.index.name = "timestamp"

        # add epoch column (timestamp converted to epoch, needed for javascript)
        resampled1['epoch'] = resampled1.index.astype(np.int64) / 1000000

        return resampled1



#
# used in Surfing Now 
#
class BytesPerMinuteGenerator:

    def generate(self, dataframe):

        # create a new dataframe with needed columns and set the index to timestamp
        df = dataframe[ 
            ["timestamp", "reply_size", "request_size"] 
        ].set_index("timestamp")

        # now resample the table into 1 minute intervals - each verdict type now is summed
        resampled = df.resample('15s').sum()

        # add epoch column (timestamp converted to epoch, needed for javascript)
        resampled['epoch'] = resampled.index.astype(np.int64) / 1000000

        return resampled