import numpy  as np
import pandas as pd

#
#
#
from daemon.safety import categories_to_strings

#
# generates the list of domains based on daily dataframe (a little different from requestors generator)
#
class DomainsGenerator:

    def generate(self, df):

        # calculate view
        view   = self.calculate_view(df)
        totals = self.calculate_totals(view)

        # and return
        return (view, totals)

    def calculate_view(self, df):

        # group records by the domain, then calculate aggregates
        view = df.groupby(
            ['fldomain']
        ).agg(
            { 
                'request_url'       : 'count', 
                'reply_size'        : 'sum',
                'request_size'      : 'sum',
                'user_id'           : 'nunique',
                'ws_verdict_pass'   : 'sum',
                'ws_verdict_adapt'  : 'sum',
                'ws_verdict_block'  : 'sum',
                'ws_verdict_rescan' : 'sum',
                'ws_verdict_skip'   : 'sum',
                'ws_offensive'      : 'sum',
                "ws_categories"     : 'max'
            }
        ).sort_values("reply_size", ascending=False)

        # convert the categories flags into categories
        view['ws_categories_str'] = view.apply(lambda row: categories_to_strings(int(row["ws_categories"])), axis=1)
        
        # calculate percentage of requests
        if True:

            total_sum = view["request_url"].sum()
            if total_sum > 0:
                view["requests_percent"] = np.ceil(100 * view["request_url"] / total_sum).astype(int)
            else:
                view["requests_percent"] = 0

        # calculate percentage of download
        if True:

            total_sum = view["reply_size"].sum()
            if total_sum > 0:
                view["bandwidth_percent"] = np.ceil(100 * view["reply_size"] / total_sum).astype(int)
            else:
                view["bandwidth_percent"] = 0

        # calculate percentage of upload
        if True:

            total_sum = view["request_size"].sum()
            if total_sum > 0:
                view["request_size_percent"] = np.ceil(100 * view["request_size"] / total_sum).astype(int)
            else:
                view["request_size_percent"] = 0

        # and return it
        return view

    def calculate_totals(self, view):

        return {
            'total_requests'          : int(view['request_url'].sum()),
            'total_reply_size'        : int(view['reply_size'].sum()),
            'total_request_size'      : int(view['request_size'].sum()),            
            'total_ws_verdict_pass'   : int(view['ws_verdict_pass'].sum()),
            'total_ws_verdict_adapt'  : int(view['ws_verdict_adapt'].sum()),
            'total_ws_verdict_block'  : int(view['ws_verdict_block'].sum()),
            'total_ws_verdict_rescan' : int(view['ws_verdict_rescan'].sum()),
            'total_ws_verdict_skip'   : int(view['ws_verdict_skip'].sum()),
            'total_ws_offensive'      : int(view['ws_offensive'].sum()),
            'total_user_id'           : int(view['user_id'].nunique()),
        }
