import numpy  as np
import pandas as pd

#
# hits resampled per hour
#
class HitsPerHourGenerator:

    def generate(self, dataframe):

        # create a new dataframe with needed columns and set the index to timestamp
        df = dataframe[ 
            ["timestamp", "ws_verdict_pass", "ws_verdict_adapt", "ws_verdict_block", "ws_verdict_skip"] 
        ].set_index("timestamp")

        # now resample the table into 1 hour intervals - each verdict type now is summed
        resampled = df.resample('1H').sum()

        # select the first value (we need in to generate the full day index)
        date = dataframe["date"].iloc[0]

        # create new index for the full day based on the date
        index = pd.period_range(date, periods=24, freq='H').to_timestamp()

        # and reindex the resampled data
        resampled1 = resampled.reindex(index=index, fill_value=0)

        # set the index name for better looking csv
        resampled1.index.name = "timestamp"

        # add total column
        resampled1['total']= resampled1.sum(axis=1)

        # add epoch column (timestamp converted to epoch, needed for javascript)
        resampled1['epoch'] = resampled1.index.astype(np.int64) / 1000000

        return resampled1

#
# hits resamped per 10 minutes - used in hits per day report, the dataframe is also filled in till whole [0:23] interval
#
class HitsPer10MinutesGenerator:

    def generate(self, dataframe):

        # create a new dataframe with needed columns and set the index to timestamp
        df = dataframe[ 
            ["timestamp", "ws_verdict_pass", "ws_verdict_adapt", "ws_verdict_block", "ws_verdict_skip"] 
        ].set_index("timestamp")

        # now resample the table into 10 minutes intervals - each verdict type now is summed
        resampled = df.resample('600s').sum()

        # select the first value (we need in to fill in the dataframe for the whole day)
        date = dataframe["date"].iloc[0]

        # create new index for the full day based on the date
        index = pd.period_range(date, periods=144, freq='600s').to_timestamp()

        # and reindex the resampled data
        resampled1 = resampled.reindex(index=index, fill_value=0)

        # set the index name for better looking csv
        resampled1.index.name = "timestamp"

        # add total column
        resampled1['total']= resampled1.sum(axis=1)

        # add epoch column (timestamp converted to epoch, needed for javascript)
        resampled1['epoch'] = resampled1.index.astype(np.int64) / 1000000

        return resampled1

#
# hits resamped per minute - used in the Surfing Now report
#
class HitsPerMinuteGenerator:

    def generate(self, dataframe):

        # create a new dataframe with needed columns and set the index to timestamp
        df = dataframe[ 
            ["timestamp", "ws_verdict_pass", "ws_verdict_adapt", "ws_verdict_block", "ws_verdict_skip"] 
        ].set_index("timestamp")

        # now resample the table into 1 minute intervals - each verdict type now is summed
        resampled = df.resample('15s').sum()

        # add total column
        resampled['total']= resampled.sum(axis=1)

        # add epoch column (timestamp converted to epoch, needed for javascript)
        resampled['epoch'] = resampled.index.astype(np.int64) / 1000000

        return resampled

