import os
import requests
import numpy  as np
import pandas as pd
import urllib.parse

#
#
#
from daemon.core  import DaemonPath
from daemon.utils import JsonReader
#
#
#
def is_search(row):

    request_url = row['request_url']

    if request_url.find(".google.") != -1 and request_url.find("search") != -1:
        return True

    return False

#
#
#
class SearchesGenerator:

    def generate(self, dataframe):

        # select only google searches
        df = dataframe[
            dataframe.apply(is_search, axis=1)
        ]

        # leave only needed columns
        df = df[ ["ws_iid", "timestamp", "user_id", "request_url", "request_params"] ]

        # set all NaN into empty string to look better
        df.fillna("", inplace=True)

        # enrich the dataframe
        df = self.enrich(df)

        # and return
        return df

    def question_is_found(self, q, questions):

        for item in questions:
            if q == item:
                return True

            if item.find(q) != -1:
                return True

        return False

    def enrich(self, dataframe):

        # loop over all records in the dataframe and parse out the search terms
        raw_questions = []
        if True:
            for index, row in dataframe.iterrows():

                raw_questions.append(
                    self.parse_q(row["request_params"])
                )

        # sort them out by length of the question
        sorted_questions = sorted(raw_questions, key=len)

        # take one by one and add to final array if unique
        unique_questions = []
        while len(sorted_questions) > 0:

            q = sorted_questions.pop(0)
            if len(q) == 0:
                continue

            # see if q is a substring of any question, if so it is not interesting
            if self.question_is_found(q, sorted_questions):
                continue

            # ok this is a unique question
            unique_questions.append(q)

        # create new dataframe
        df1 = pd.DataFrame(unique_questions)

        df1.index.name = 'number'
        df1.rename(columns={0: 'question'}, inplace=True)

        # and return
        return df1

    def parse_q(self, request_params):

        # any error in parsing means question is empty
        question = ""

        if len(request_params) > 0:

            try:
                qs = urllib.parse.parse_qs(request_params)
                q  = qs.get("q", [""])

                question = q[0]

                if not question:
                    pq       = qs.get("pq", [""])
                    question = pq[0]

                # we are not interested in questions for URLs
                if question.startswith("https://") or question.startswith("http://"):
                    question = ""

            except Exception as e:
                question = ""

        return question

    def lookup_video(self, video_id):

        # any error in lookup is handled
        title = ""
        desc  = ""

        try:

            if not video_id:
                raise Exception("Empty video id")
            if not self.api_key:
                raise Exception("YouTube API key is empty")

            url    = 'https://www.googleapis.com/youtube/v3/videos'
            params = dict(part='snippet', id=video_id, key=self.api_key)
            resp   = requests.get(url=url, params=params)
            data   = resp.json()

            if data["kind"] != "youtube#videoListResponse":
                raise Exception("Wrong data kind %s" % data["kind"])

            items = data["items"]
            if len(items) != 1:
                raise Exception("Wrong count of items %d instead of 1" % len(items))

            # get first snippet
            item = items[0]

            # and assign
            title = item["snippet"]["title"]
            desc  = item["snippet"]["description"]

        except Exception as e:

            title = "unable to lookup"
            desc  = str(e)

        # sanitize
        title = title.replace("\n", " ")
        title = title.replace("\r", " ")

        desc = desc.replace("\n", " ")
        desc = desc.replace("\r", " ")

        # and return nicely (we do not need large)
        return (title[:32], desc[:64])



