import numpy  as np
import pandas as pd

#
# generates the list of users based on daily dataframe
#
class UsersGenerator:

    group_by = 'user_id'

    def generate(self, df):

        # calculate view
        view   = self.calculate_view(self.group_by, df)
        totals = self.calculate_totals(view)

        # and return
        return (view, totals)

    def calculate_view(self, group_by, df):

        # group records by the group_field, then calculate hits and offensive hits per group
        view = df.groupby(
            [group_by]
        ).agg(
            { 
                'request_url'       : 'count',
                'request_size'      : 'sum',
                'reply_size'        : 'sum',
                'ws_verdict_pass'   : 'sum',
                'ws_verdict_adapt'  : 'sum',
                'ws_verdict_block'  : 'sum',
                'ws_verdict_rescan' : 'sum',
                'ws_verdict_skip'   : 'sum',
                'ws_offensive'      : 'sum'
            }
        ).sort_values("reply_size", ascending=False)
        
        # calculate percentage of requests
        if True:

            total_sum = view["request_url"].sum()
            if total_sum > 0:
                view["requests_percent"] = np.ceil(100 * view["request_url"] / total_sum).astype(int)
            else:
                view["requests_percent"] = 0

        # calculate percentage of download
        if True:

            total_sum = view["reply_size"].sum()
            if total_sum > 0:
                view["bandwidth_percent"] = np.ceil(100 * view["reply_size"] / total_sum).astype(int)
            else:
                view["bandwidth_percent"] = 0

        # calculate percentage of upload
        if True:

            total_sum = view["request_size"].sum()
            if total_sum > 0:
                view["request_size_percent"] = np.ceil(100 * view["request_size"] / total_sum).astype(int)
            else:
                view["request_size_percent"] = 0

        # and return it
        return view

    def calculate_totals(self, view):

        return {
            'request_url_total'       : int( view['request_url'].sum() ),
            'request_size_total'      : int(view['request_size'].sum()),
            'reply_size_total'        : int( view['reply_size'].sum() ),
            'ws_verdict_pass_total'   : int( view['ws_verdict_pass'].sum() ),
            'ws_verdict_adapt_total'  : int( view['ws_verdict_adapt'].sum() ),
            'ws_verdict_block_total'  : int( view['ws_verdict_block'].sum() ),
            'ws_verdict_rescan_total' : int( view['ws_verdict_rescan'].sum() ),
            'ws_verdict_skip_total'   : int( view['ws_verdict_skip'].sum() ),
            'ws_offensive_total'      : int(view['ws_offensive'].sum())
        }
