import os
import logging


#
#
#
from daemon.utils import Timer

#
#
#
from .day   import DayReporter
from .month import MonthReporter
from .year  import YearReporter

#
#
#
class CronReporter:

    def __init__(self, config):

        # debug check dirs are fine
        assert(os.path.isdir(config.stats_dir))        
        assert(os.path.isdir(config.reports_dir))        

        # save
        self.stats_dir   = config.stats_dir
        self.reports_dir = config.reports_dir
        self.config      = config

    def report(self, force):

        logging.info("Report build initiated...")

        # create timer
        timer = Timer()

        # days based on statistics storage
        d = DayReporter(self.config)
        d.report(force)

        # months by aggregating days
        m = MonthReporter(self.config)
        m.report(force)

        # years by aggregating months
        y = YearReporter(self.config)
        y.generate(force)

        # ok completed, dump the log
        logging.info("Report built completed.")
        logging.info("Statistics:")
        logging.info("  Elapsed time    : %d seconds" % timer.elapsed())
        
