import os
import shutil
import logging
import datetime
import dateutil.relativedelta

#
#
#
from daemon.stats.collector import History

#
#
#
from .builders.monthly import Builder
from .history          import is_valid_report_dir

#
#
#
class MonthReporter:

    def __init__(self, config):

        # debug check dirs are fine
        assert(os.path.isdir(config.stats_dir))
        assert(os.path.isdir(config.reports_dir))

        # save
        self.stats_dir   = config.stats_dir
        self.reports_dir = config.reports_dir

    def report(self, force):

        logging.info("Generating monthly reports:")

        # see what year and month info we have available
        history = History(self.stats_dir)

        # loop over each year
        for year in history.years():

            # then over each month in a year
            for month in history.months(year):

                # construct the prefix
                prefix = "%04d-%02d" % (year, month)

                # see if admin insists
                if force:

                    # oh yes just do it
                    self.generate_month(prefix, month, year)

                else:

                    # see if the report is already there
                    if not is_valid_report_dir(os.path.join(self.reports_dir, prefix)):

                        # no such report, generate then
                        self.generate_month(prefix, month, year)

                    else:

                        # well report is there but we always regenerate previous month
                        today = datetime.date.today()
                        start = today - datetime.timedelta(days=31)

                        if datetime.date(year, month, 1) < start:

                            # log and skip
                            logging.info("\tskip %s (already generated)" % prefix)

                        else:

                            # do regenerate indeed
                            self.generate_month(prefix, month, year)

    def generate_month(self, prefix, month, year):

        # dump to log
        logging.info("\tbuild %s monthly report ..." % prefix)

        # collect all available day folders for a given month
        folders = []

        if True:
            
            history = History(self.stats_dir)
            for day in history.days(year, month):

                d = os.path.join(self.reports_dir, "%04d-%02d-%02d" % (year, month, day))
                if is_valid_report_dir(d):
                    folders.append(d)

        # the monthly report will be saved here
        out_dir = os.path.join(self.reports_dir, prefix)
        tmp_dir = out_dir + ".temp"

        # clear the temp folder
        if os.path.exists(tmp_dir):
            shutil.rmtree(tmp_dir)

        # create it
        os.mkdir(tmp_dir)

        # and build
        r = Builder(tmp_dir)
        r.build(folders)

        # remove the output dir just in case
        if os.path.exists(out_dir):
            shutil.rmtree(out_dir)

        # and rename temp
        os.rename(tmp_dir, out_dir)