import os
import re
import logging
import shutil


#
#
#
from daemon.stats.collector import History

#
#
#
class HistorySearcher:

    def day_found(self, history, year, month, day):

        years = history.years()
        if year in years:

            months = history.months(year)
            if month in months:

                days = history.days(year, month)
                if day in days:
                    return True

        return False

    def month_found(self, history, year, month):

        years = history.years()
        if year in years:

            months = history.months(year)
            if month in months:
                return True

        return False

    def year_found(self, history, year):

        years = history.years()
        if year in years:
            return True

        return False

#
#
#
class CronPurger:

    def __init__(self, config):

        # debug check dirs are fine
        assert(os.path.isdir(config.stats_dir))        
        assert(os.path.isdir(config.reports_dir))        

        # save
        self.stats_dir   = config.stats_dir
        self.reports_dir = config.reports_dir

    def purge(self):

        logging.info("Report purger is starting...")

        # construct the history object on the statistics storage
        history = History(self.stats_dir)

        # purge days
        self.purge_days(history)

        # purge months
        self.purge_months(history)

        # purge years
        self.purge_years(history)

        logging.info("Report purger finished successfully.")

    def purge_days(self, history):

        # enumerate the days in the reporting storage
        entries = [f.lower() for f in os.listdir(self.reports_dir)]

        # collection of folders to remove
        to_remove = []

        # loop over entries
        for entry in entries:

            # see if it is like YYYY-MM-DD (for example, 2019-04-16)
            match = re.match(r'^(\d\d\d\d)-(\d\d)-(\d\d)$', entry, re.M | re.I)
            if not match:
                continue

            # get parts as ints
            y = int(match.group(1))
            m = int(match.group(2))
            d = int(match.group(3))

            # see if this date is in the statistics
            searcher = HistorySearcher()
            if not searcher.day_found(history, y, m, d):

                # ok this folder needs to be removed
                to_remove.append(os.path.join(self.reports_dir, entry))

        # now just purge all
        for folder in to_remove:
            # log it
            logging.info("\tPurging report %s...", folder)

            # and drop
            shutil.rmtree(folder)

        if len(to_remove):
            logging.info("\tDaily reports are purged")

    def purge_months(self, history):

        # enumerate the months in the reporting storage
        entries = [f.lower() for f in os.listdir(self.reports_dir)]

        # collection of folders to remove
        to_remove = []

        # loop over entries
        for entry in entries:

            # see if it is like YYYY-MM (for example, 2019-04)
            match = re.match(r'^(\d\d\d\d)-(\d\d)$', entry, re.M | re.I)
            if not match:
                continue

            # get parts as ints
            y = int(match.group(1))
            m = int(match.group(2))

            # see if this date is in the statistics
            searcher = HistorySearcher()
            if not searcher.month_found(history, y, m):

                # ok this folder needs to be removed
                to_remove.append(os.path.join(self.reports_dir, entry))

        # now just purge all
        for folder in to_remove:
            # log it
            logging.info("\tPurging report %s...", folder)

            # and drop
            shutil.rmtree(folder)

        if len(to_remove):
            logging.info("\tMonthly reports are purged")

    def purge_years(self, history):

        # enumerate the years in the reporting storage
        entries = [f.lower() for f in os.listdir(self.reports_dir)]

        # collection of folders to remove
        to_remove = []

        # loop over entries
        for entry in entries:

            # see if it is like YYYY (for example, 2019)
            match = re.match(r'^(\d\d\d\d)$', entry, re.M | re.I)
            if not match:
                continue

            # get parts as ints
            y = int(match.group(1))

            # see if this date is in the statistics
            searcher = HistorySearcher()
            if not searcher.year_found(history, y):

                # ok this folder needs to be removed
                to_remove.append(os.path.join(self.reports_dir, entry))

        # now just purge all
        for folder in to_remove:
            # log it
            logging.info("\tPurging report %s...", folder)

            # and drop
            shutil.rmtree(folder)

        if len(to_remove):
            logging.info("\tYearly reports are purged")
