import os
import shutil
import logging
import datetime
import dateutil.relativedelta

#
#
#
from daemon.stats.collector import History

#
#
#
from .builders.yearly import Builder
from .history         import is_valid_report_dir

#
#
#
class YearReporter:

    def __init__(self, config):

        # debug check dirs are fine
        assert(os.path.isdir(config.stats_dir))
        assert(os.path.isdir(config.reports_dir))

        # save
        self.stats_dir   = config.stats_dir
        self.reports_dir = config.reports_dir

    def generate(self, force):

        logging.info("Generating yearly reports:")

        # see what year info we have available
        history = History(self.stats_dir)

        # loop over each year
        for year in history.years():

            # we always regenerate years (it is fast)
            self.generate_year(year)

    def generate_year(self, year):

        # construct the folder name for this year
        folder = "%04d" % year

        # dump to log
        logging.info("\tbuild %s yearly report ..." % folder)

        # collect all available month reports for a given year
        folders = []

        if True:
            
            history = History(self.stats_dir)        
            for month in history.months(year):

            	m = os.path.join(self.reports_dir, "%04d-%02d" % (year, month))
            	if is_valid_report_dir(m):
                	folders.append(m)

        # the yearly report will be saved here
        out_dir = os.path.join(self.reports_dir, folder)
        tmp_dir = out_dir + ".temp"

        # clear the temp folder
        if os.path.exists(tmp_dir):
            shutil.rmtree(tmp_dir)

        # create it
        os.mkdir(tmp_dir)

        # and run the accumulating reporter
        r = Builder(tmp_dir)
        r.build(folders)

        # remove the output dir just in case
        if os.path.exists(out_dir):
            shutil.rmtree(out_dir)

        # and rename temp
        os.rename(tmp_dir, out_dir)