import os
import shutil
import platform
import subprocess

#
#
#
class FsMgr:

    
    def remove_file(self, target_file):

        # we can only remove files within our directory
        if not self.target_is_acceptable(target_file):
            raise Exception("Cannot remove target file '%s'" % target_file)

        # it should be a file indeed
        if not os.path.isfile(target_file):
            raise Exception("Target file '%s' is *not* a file" % target_file)

        # ok drop it
        os.unlink(target_file)


    def copy_file(self, source_file, target_file):

        # verify the source file looks ok
        if not self.source_is_acceptable(source_file):
            raise Exception("Source file '%s' is wrong" % source_file)

        # it should be a file indeed
        if not os.path.isfile(source_file):
            raise Exception("Source file '%s' is *not* a file" % source_file)

        # construct names
        target_file_bak = target_file + ".bak"
        
        # drop the old bak file if it exists
        if os.path.isfile(target_file_bak):
            os.unlink(target_file_bak)

        # move the current one into bak if it exists
        if os.path.isfile(target_file):
            shutil.move(target_file, target_file_bak)

        # and copy the sorce into new one
        shutil.copyfile(source_file, target_file)

    def remove_folder(self, target_folder):

        # we can only remove files within our directory
        if not self.target_is_acceptable(target_folder):
            raise Exception("Cannot remove target folder '%s'" % target_folder)

        # it should be a folder indeed
        if not os.path.isdir(target_folder):
            raise Exception("Target folder '%s' is *not* a folder" % target_folder)

        # ok drop it
        shutil.rmtree(target_folder)

    def copy_folder(self, source_folder, target_folder):

        # verify the source folder looks ok
        if not self.source_is_acceptable(source_folder):
            raise Exception("Source folder '%s' is wrong" % source_folder)

        # it should be a folder indeed
        if not os.path.isdir(source_folder):
            raise Exception("Source folder '%s' is *not* a folder" % source_folder)

        target_folder_bak = target_folder + ".bak"
        
        # drop the old bak folder if it exists
        if os.path.isdir(target_folder_bak):
            shutil.rmtree(target_folder_bak)

        # move the current one into bak
        if os.path.isdir(target_folder):
            shutil.move(target_folder, target_folder_bak)

        # and copy the sorce into new one
        shutil.copytree(source_folder, target_folder)

    def source_is_acceptable(self, path):

        if path.startswith("/opt/websafety-ui/var/temp"):

            # for example the temp folders with generated configuration is here
            return True

        if path.startswith("/opt/websafety-ui/etc/"):

            # for example the periodic cron.d file is generated here and 
            # squid's logrotate.d file too
            return True

        if platform.system().lower() == "windows":
            return True

        return False

    def target_is_acceptable(self, path):

        if path.startswith("/opt/websafety/"):
            return True

        if platform.system().lower() == "windows":
            return True

        return False

    def reset_owner(self, target_folder, user, group):

        # verify the target folder looks ok
        if not self.target_is_acceptable(target_folder):
            raise Exception("Target folder '%s' is wrong" % target_folder)

        # do nothing on windows
        if platform.system().lower() == "windows":
            return

        # debug check
        assert(user  in ["proxy", "squid"])
        assert(group in ["proxy", "squid"])

        # reset the owner
        args = ["chown", "-R", "%s:%s" % (user, group), target_folder]
        code = subprocess.call(args)
        if code != 0:

            raise Exception("Cannot change owner of a configuration folder '%s' with command '%s'." %
                (target_folder, " ".join(args))
            )

        # all good then