#!/usr/bin/env python3
import sys
import argparse

#
#
#
from daemon.utils import FsMgr
from daemon.core  import SquidUser

#
# remove action
#
def perform_remove(args):

    fsmgr = FsMgr()

    if args.target_folder:
        fsmgr.remove_folder(args.target_folder)    

    if args.target_file:
        fsmgr.remove_file(args.target_file)


#
# copy action
#
def perform_copy(args):

    # create actor, user and group names
    fsmgr = FsMgr()
    user  = SquidUser.name()
    group = SquidUser.name()

    # see what we have do to
    if args.source_folder and args.target_folder:

        # copy
        fsmgr.copy_folder(
            args.source_folder, 
            args.target_folder
        )    

        # and reset
        fsmgr.reset_owner(args.target_folder, user, group)

    elif args.source_file and args.target_file:

        # copy
        fsmgr.copy_file(
            args.source_file, 
            args.target_file
        )

        # and reset
        fsmgr.reset_owner(args.target_file, user, group)

    else:
        raise Exception("Arguments are invalid")

#
# action dispatcher
#
def perform_action(args):

    if args.remove:
        perform_remove(args)
    else:
        perform_copy(args)

    

#
# main
#
def main():

    # create parser
    parser = argparse.ArgumentParser(description='FsMgr.py file manipulation script.')

    # add args
    parser.add_argument("--source-folder", help="full path to source folder")
    parser.add_argument("--target-folder", help="full path to target folder")
    parser.add_argument("--source-file", help="full path to source file")
    parser.add_argument("--target-file", help="full path to target file")
    parser.add_argument("--remove", action='store_true', help="if set removes target folder or/and file")
    
    # parse actual arguments
    args = parser.parse_args()

    # ok let's go
    perform_action(args)

if __name__ == "__main__":

    # call the entry code
    main()

    # if we got here everything is fine
    sys.exit(0)
