#!/usr/bin/env python3
import os
import sys
import time
import logging
import datetime
import argparse

#
#
#
def perform_action_linux(distrib, hostname):

    file = '/etc/hostname'
    logging.debug("Opening the '%s' for writing..." % file)
    with open(file, 'w') as fout:
        fout.write(hostname)

    logging.debug("Host name '%s' is written to file '%s' successfully" % (hostname, file))    

#
#
#
def perform_action(system_name, distrib, hostname):

    if len(hostname) == 0:
        raise Exception("Host name cannot be empty!")

    logging.debug("System: %s, distrib: %s, setting host name to '%s' ... " % (system_name, distrib, hostname))

    if system_name == "linux":
        perform_action_linux(distrib, hostname)
    else:
        raise Exception("Provided system name '%s' is not valid!" % system_name)

    # if we got here everything is fine
    logging.debug("Host name information updated successfully!")

#
# main
#
def main():

    verbose = False
    start   = time.time()
    
    # parse command line args
    parser = argparse.ArgumentParser(description='Host name manipulation script.')
    parser.add_argument("--verbose", help="print additional debug information", action="store_true")
    parser.add_argument("--hostname", help="sets hostname to this value", required=True)
    parser.add_argument("--system", help="linux", required=True)
    parser.add_argument("--distrib", help="debian10|ubuntu20", required=True)
    
    args  = parser.parse_args()       
    level = logging.INFO
    if args.verbose:
        level = logging.DEBUG
        
    logging.basicConfig(format='%(asctime)s %(message)s', level=level)
            
    logging.debug("Hostname manipulation script is started at %s" % datetime.datetime.now().strftime("%Y-%m-%d, %H:%M:%S"))    
    perform_action(args.system, args.distrib, args.hostname)    
    logging.debug("Hostname manipulation script is finished SUCCESSFULLY in %d seconds!" % (time.time() - start))

if __name__ == "__main__":

    # call the entry code
    main()

    # if we got here everything is fine
    sys.exit(0)
