#!/usr/bin/env python3
import os
import sys
import time
import shutil
import logging
import datetime
import argparse

#
#
#
def ovewrite_file(source_file, target_file):

    # read the contents of source file
    with open(source_file, 'r') as fin:
        contents = fin.read()

    cur_file = target_file
    bak_file = cur_file + ".bak"
    new_file = cur_file + ".new"

    logging.debug("Current file '%s'" % cur_file)
    logging.debug("Bak file '%s'" % bak_file)
    logging.debug("New file '%s'" % new_file)

    # clear the environment
    if os.path.exists(bak_file):
        os.unlink(bak_file)
    if os.path.exists(new_file):
        os.unlink(new_file)

    # copy current file to bak
    shutil.copyfile(cur_file, bak_file)
    shutil.copyfile(source_file, new_file)        

    logging.debug("Moving new file '%s' to target file '%s' ..." % (new_file, cur_file))

    # move
    shutil.move(new_file, cur_file)

    logging.debug("Success, contents of file '%s' are updated." % cur_file)


#
#
#
def perform_action(system_name, distrib, source_file):

    # verify the source file
    if len(source_file) == 0:
        raise Exception("Source file '%s' cannot be empty!" % source_file)

    if source_file.find("..") != -1:
        raise Exception("Source file '%s' must be absolute!" % source_file)

    # check the source file is correctly placed
    if not source_file.startswith("/opt/websafety/etc/") and not source_file.startswith("/opt/websafety-ui/etc/"):
        raise Exception("Source file '%s' is not located correctly." % source_file)

    # ok dump the info
    logging.debug("System: %s, distrib: %s, source file '%s' ... " % (system_name, distrib, source_file))

    # we are only working on linux
    if system_name != "linux":
        raise Exception("Provided system name '%s' is not valid!" % system_name)

    # set the target file name
    target_file = ""

    if distrib in ["debian10"]:
        target_file = "/etc/resolv.conf"
    else:
        raise Exception("Distrib name '%s' is not valid!" % distrib)

    # if we got here, everything is fine, overwrite!
    ovewrite_file(source_file, target_file)

    # if we got here everything is fine
    logging.debug("Resolve settings are updated successfully!")


#
# main
#
def main():
    verbose = False
    start   = time.time()
    
    # parse command line args
    parser = argparse.ArgumentParser(description='Debian 10 /etc/resolv.conf manipulation script.')
    parser.add_argument("--verbose", help="print additional debug information", action="store_true")
    parser.add_argument("--file", help="path to file which will be copied to /etc/resolv.conf", required=True)
    parser.add_argument("--system", help="linux", required=True)
    parser.add_argument("--distrib", help="debian10", required=True)
    
    args  = parser.parse_args()       
    level = logging.INFO
    if args.verbose:
        level = logging.DEBUG
        
    logging.basicConfig(format='%(asctime)s %(message)s', level=level)
            
    logging.debug("Resolve manipulation script is started at %s" % datetime.datetime.now().strftime("%Y-%m-%d, %H:%M:%S"))    
    perform_action(args.system, args.distrib, args.file)    
    logging.debug("Resolve settings manipulation script is finished SUCCESSFULLY in %d seconds!" % (time.time() - start))

if __name__ == "__main__":

    # call the entry code
    main()

    # if we got here everything is fine
    sys.exit(0)
