import os
import tempfile

#
#
#
from daemon.core import UiPath, System, Distrib
from _domain.utils import CommandElevated

#
#
#
class DebianNetworkSettingsGenerator:

    def generate(self, contents):

        # debug check
        assert(len(contents) > 0)

        # first generate a temporary file in /opt/websafety-ui/etc/node folder (it will be automatically removed after being closed)
        prefix = "etc_network_interfaces."
        folder = os.path.join(UiPath.etc_dir(), "node")

        # write to temp file        
        with tempfile.NamedTemporaryFile(prefix=prefix, dir=folder, delete=True) as temp:

            temp.write(contents.encode())
            temp.flush()

            # call the sudoing binary
            exe  = os.path.join(UiPath.bin_dir(), "netmgr.py")
            arg1 = "--file=%s" % temp.name
            arg2 = "--system=%s" % System.name()
            arg3 = "--distrib=%s" % Distrib.name()
            args = [exe, arg1, arg2, arg3]

            # and run it
            (exit_code, stdout, stderr) = CommandElevated().run(args)
            if exit_code != 0:
                raise Exception("Cannot generate network settings. Error: %d, STDOUT: %s, STDERR: %s" % (exit_code, stdout, stderr))

#
#
#
class EtcResolvConfGenerator:

    def generate(self, contents):

        # debug check
        assert(len(contents) > 0)

        # first generate a temporary file in our etc folder (it will be automatically removed after being closed)
        prefix = "etc_resolv_conf."
        folder = UiPath.etc_dir()

        # write to temp file        
        with tempfile.NamedTemporaryFile(prefix=prefix, dir=folder, delete=True) as temp:

            temp.write(contents.encode())
            temp.flush()

            # call the sudoing binary
            exe  = os.path.join(UiPath.bin_dir(), "resolve.py")
            arg1 = "--file=%s" % temp.name
            arg2 = "--system=%s" % System.name()
            arg3 = "--distrib=%s" % Distrib.name()
            args = [exe, arg1, arg2, arg3]

            # and run it
            (exit_code, stdout, stderr) = CommandElevated().run(args)
            if exit_code != 0:
                raise Exception("Cannot generate network settings. Error: %d, STDOUT: %s, STDERR: %s" % (exit_code, stdout, stderr))