import pandas as pd
import numpy  as np
#
# domain
#
from _domain.safety import BuildInCategories

#
#
#
class HitsType(object):

    ALL_AVAILABLE = 0
    ONLY_BLOCKED  = 1

#
#
#
class TableDomain(object):

    def __init__(self, frame, domain):

        # construct view for that specific domain
        self.view = frame[
            frame["tld"] == domain
        ]

    def retrieve(self, what):

        # get the view
        view = self.view

        # adjust it based on what parameter
        if what == HitsType.ONLY_BLOCKED:

            # having ads and tracking domains gives a lot of noise
            exclude_category_mask = BuildInCategories.CATEGORY_ADVERTISING | BuildInCategories.CATEGORY_USER_TRACKING

            # make a sub view
            view = view[
                (view['verdict'] == "block") 
                & 
                ~view['module'].isin( ["adblock", "privacy"] )                
                &
                (np.bitwise_and(view["categoryflags"], exclude_category_mask) == 0)
            ]

        # group by tld
        view = view.groupby( ["user_name", "user_ip"] ).agg(
            { 
                "tld": "count"
            }
        )
        
        # sort them by the number of hits and get first 30
        view = view.sort_values("tld", ascending=False)
        view = view[:30]
        
        # and return        
        return view

class TableDomainUser(object):

    def __init__(self, frame, domain, user):

        # construct view for that specific domain
        self.view = frame[            

            (frame["tld"] == domain)            
            &
            ((frame["user_name"] == user) | (frame["user_ip"] == user))
        
        ]

    def retrieve(self, what):

        # get the view
        view = self.view

        # adjust it based on what parameter
        if what == HitsType.ONLY_BLOCKED:
            view = view[
                (view['verdict'] == "block") & ~view['module'].isin( ["adblock", "privacy"] )
            ]

        # sort by timestamp and leave first 20 records
        view = view.sort_values("timestamp", ascending=False)
        view = view.head(20)

        # and return        
        return view

#
#
#
class DomainsSummary(object):

    def __init__(self, frame):

        # save the frame
        self.frame = frame

    def top_n_browsing(self, n=30):

        # if no data return empty view
        if self.frame.empty:
            return pd.DataFrame([])

        view = self.frame[ 
            ["tld", "user_name", "user_ip"] 
        ]

        view = view.groupby(["tld"]).agg(
            {
                "tld"      : "count",
                "user_ip"  : pd.Series.nunique,
                "user_name": pd.Series.nunique
            }
        )
        view.columns = ["hits", "user_ip", "user_name"]
        view = view.sort_values("hits", ascending=False)
        view = view[:n]

        return view

    def top_n_blocked(self, n=30):

        # if no data return empty view
        if self.frame.empty:
            return pd.DataFrame([])

        # get only blocked entries
        frame = self.frame

        # having ads and tracking domains gives a lot of noise
        exclude_category_mask = BuildInCategories.CATEGORY_ADVERTISING | BuildInCategories.CATEGORY_USER_TRACKING

        # make a view
        view = frame[
            (frame['verdict'] == "block") 
            & 
            ~frame['module'].isin( ["adblock", "privacy"] )                
            &
            (np.bitwise_and(frame["categoryflags"], exclude_category_mask) == 0)
        ]

        # group by        
        view = view.groupby(["tld"]).agg(
            {
                "tld"      : "count",
                "user_ip"  : pd.Series.nunique,
                "user_name": pd.Series.nunique
            }
        )
        view.columns = ["hits", "user_ip", "user_name"]
        view = view.sort_values("hits", ascending=False)
        view = view[:n]
        
        return view
