import json
import numpy
import pandas as pd

#
# domain
#
from _domain.safety import BuildInCategories

#
#
#
class SummaryMgr(object):

    def __init__(self, frame):

        # save the frame
        self.frame = frame

    def summary(self):

        # default values are all zeros
        result = {
            "hits" : {
                "total" : self.frame.shape[0],
                "block" : 0,
                "adapt" : 0,
                "pass"  : 0,
            },
            "tdelta" : {
                "days"    : 0,
                "hours"   : 0,
                "minutes" : 0,
                "seconds" : 0
            },
            "users_total"   : 0,
            "ips_total"     : 0,
            "domains_total" : 0,
            "policies_total": 0,
        }

        # fill tdelta 
        try:
            view   = self.frame["timestamp"]#.astype("datetime64[ns]")
            tstart = view.iloc[0]
            tend   = view.iloc[-1]
            tdelta = tend - tstart

            result["tdelta"]["days"]    = tdelta.components.days
            result["tdelta"]["hours"]   = tdelta.components.hours
            result["tdelta"]["minutes"] = tdelta.components.minutes
            result["tdelta"]["seconds"] = tdelta.components.seconds

        except Exception as e:
            pass


        # fill hits
        try:
            verdicts = self.frame[ ["verdict"] ].groupby('verdict').size()            
            result["hits"]["block"] = verdicts.loc['block']
            result["hits"]["adapt"] = verdicts.loc['adapt']
            result["hits"]["pass"]  = verdicts.loc['pass']
        except Exception as e:
            pass

        # fill users
        try:
            users = self.frame[ ["user_name"] ].groupby('user_name').size()    
            result["users_total"] = users.shape[0]
        except Exception as e:
            pass

        # fill ips
        try:
            ips = self.frame[ ["user_ip"] ].groupby('user_ip').size()    
            result["ips_total"] = ips.shape[0]
        except Exception as e:
            pass

        # fill domains
        try:
            domains = self.frame[ ["tld"] ].groupby('tld').size()    
            result["domains_total"] = domains.shape[0]
        except Exception as e:
            pass

        # fill policies
        try:
            policies = self.frame[ ["policy"] ].groupby('policy').size()    
            result["policies_total"] = policies.shape[0]
        except Exception as e:
            pass

        return result

    def modules(self):

        # if no data return empty view
        if self.frame.empty:
            return pd.DataFrame([])

        table = self.frame[ ["module", "verdict"] ]
        table = table.groupby(["module"]).agg(
            {
                "verdict" : 'count',
            }
        )
        table = table.sort_values("verdict", ascending=False)
        
        return table

    def categories(self, n=15):

        # if no data return empty dict
        if self.frame.empty:
            return {}

        result = {
            "none" : 0
        }

        table = self.frame[ ["categoryflags"] ]
        for index, row in table.iterrows():
            
            categories = BuildInCategories.unmap_name(
                row['categoryflags']
            )
            
            if len(categories) == 0:
                result["none"] += 1
            else:
                for category in categories:

                    if category not in result:
                        result[category] = 0

                    result[category] += 1

        # now sort the final dictionary
        sorted_result = []
        for c in sorted(result, key=result.get, reverse=True):
            sorted_result.append( (c, result[c]) )

        return sorted_result[:n]

    def hits_over_time(self):

        # if no data return empty view
        if self.frame.empty:
            return pd.DataFrame([])

        # make a copy of the frame
        table = self.frame.copy()

        # convert the timestamp column
        table["timestamp"] = table["timestamp"].astype("datetime64[ns]")

        # get a table of timestamps to verdicts (and set the index to timestamp)        
        table = table[ ["timestamp", "verdict"] ].set_index("timestamp")

        # add three columns to match verdict enum
        table['clean'] = table["verdict"] == "pass"
        table['block'] = table["verdict"] == "block"       
        table['adapt'] = table["verdict"] == "adapt"

        # convert from bool to int
        table.clean = table.clean.astype(int)
        table.block = table.block.astype(int)
        table.adapt = table.adapt.astype(int)

        # now resample the table into 30second intervals - each verdict type now is summed
        resampled = pd.DataFrame()
        resampled = table.resample('1min').sum()

        # and add total column
        resampled['total'] = resampled['clean'] + resampled['block'] + resampled['adapt']

        # good print it
        # seconds.to_csv("output.csv", sep=',', encoding='utf-8')

        return resampled
