import dateutil.parser

#
# domain
#
from _domain.safety import BuildInCategories

#
# local
#
from .provider import Provider, Options

#
# helper function
#
class ScanFlagsParser(object):

    @staticmethod
    def parse(flags):
    
        # these are bits for flags - must be in sync with m:/websafety/src/include/ws/blogic_types.hpp!
        scan_for_adult      = 1
        scan_for_categories = 1<<1
        scan_for_file       = 1<<2
        scan_for_adblock    = 1<<3
        scan_for_privacy    = 1<<4
        scan_for_http       = 1<<5

        # construct the dict
        result = { 
            "adult":      False, 
            "categories": False,
            "file":       False,
            "adblock":    False,
            "privacy":    False,
            "http":       False
        }

        if flags & scan_for_adult:
            result["adult"] = True

        if flags & scan_for_categories:
            result["categories"] = True

        if flags & scan_for_file:
            result["file"] = True

        if flags & scan_for_adblock:
            result["adblock"] = True

        if flags & scan_for_privacy:
            result["privacy"] = True

        if flags & scan_for_http:
            result["http"] = True

        return result

    @staticmethod
    def unmap(flags):
        
        array = []
        flags = ScanFlagsParser.parse(flags)
        
        for key in flags.keys():
            s = key
            if flags[key]:
                s += "+"
            else:
                s += "-"
            array.append(s)
            
        return ", ".join(array)


#
#
#
class TableMgr(object):

    def __init__(self):
        pass

    def retrieve(self, options=Options()):

        # get data
        provider = Provider()
        items    = provider.retrieve(options)
        
        # fill the contents array with the constructed attributes
        for item in items:

            # categories as array of strings
            item["categories"] = ", ".join(BuildInCategories.unmap_name(item["categoryflags"]))

            # scan flags
            item["scanflags_as_string"] = ScanFlagsParser.unmap(item["scanflags"])

        # and return nicely            
        return items