import psutil
#
#
#
from django.views import generic
from django.urls import reverse
from django.contrib.messages.views import SuccessMessageMixin

#
#
#
from antivirus.models import AvSettings


#
#
#
class ViewAvSettingsMode(SuccessMessageMixin, generic.edit.UpdateView):

    model           = AvSettings
    fields          = ['enable', 'av_type', 'bypass_to_localnet']
    success_message = "need_squid_restart"
    template_name   = "antivirus/settings/mode_form.html"

    def get_object(self):
        return AvSettings.objects.first()

    def get_success_url(self):
        return reverse("av:ViewAvSettingsMode")

    def get_context_data(self, **kwargs):

        ram_memory = psutil.virtual_memory().total
        low_memory = False
        if True:
            
            if ram_memory < 8 * 1024 * 1024 * 1024:
                low_memory = True

        context = super(ViewAvSettingsMode, self).get_context_data(**kwargs)        
        context['low_memory'] = low_memory
        context['ram_memory'] = ram_memory
        
        return context

#
#
#
class ViewAvSettingsEcapActions(SuccessMessageMixin, generic.edit.UpdateView):

    model  = AvSettings
    fields = [ 
        'ecap_clamav_async', 
        'ecap_clamav_message_size_max', 
        'ecap_clamav_on_error_action',
        'ecap_reqmod_bypass',
        'ecap_respmod_bypass'
    ]
    success_message = "need_squid_restart"
    template_name   = "antivirus/settings/ecap_actions_form.html"

    def get_object(self):
        return AvSettings.objects.first()

    def get_success_url(self):
        return reverse("av:ViewAvSettingsEcapActions")

#
#
#
class ViewAvSettingsEcapTrickling(SuccessMessageMixin, generic.edit.UpdateView):

    model  = AvSettings
    fields = [ 
        'ecap_clamav_trickling_enable', 
        'ecap_clamav_trickling_drop_size', 
        'ecap_clamav_trickling_period', 
        'ecap_clamav_trickling_type', 
        'ecap_clamav_trickling_size_max', 
        'ecap_clamav_trickling_start_delay', 
    ]
    success_message = "need_squid_restart"
    template_name   = "antivirus/settings/ecap_trickling_form.html"

    def get_object(self):
        return AvSettings.objects.first()

    def get_success_url(self):
        return reverse("av:ViewAvSettingsEcapTrickling")

#
#
#
class ViewAvSettingsIcap(SuccessMessageMixin, generic.edit.UpdateView):

    model  = AvSettings
    fields = [
        'avicap_address', 
        'avicap_port', 
        'avicap_reqpath', 
        'avicap_respath', 
        'avicap_res_bypass', 
        'avicap_options'
    ]
    success_message = "need_squid_restart"
    template_name   = "antivirus/settings/icap_form.html"

    def get_object(self):
        return AvSettings.objects.first()

    def get_success_url(self):
        return reverse("av:ViewAvSettingsIcap")
