#!/usr/bin/env python3
#
# (c) Diladele B.V. 2020, Web Safety for Squid Proxy
#  
# Sample automation script
#
import os
import sys
import django
import shutil
import datetime
import logging
import argparse

#
# in order for Django to work correctly with our monitor app, we add ourselves
# to Python's search path
sys.path.append(os.path.abspath(__file__))
sys.path.insert(0, os.path.join(os.path.abspath(__file__), "..", "..", "bin"))

#
# tell Django where to read settings from
#
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "console.settings")

#
#
#
django.setup()

#
# do imports
#
from daemon.core   import UiPath, Build
from safety.models import *
from squid.models  import *
    
#
#
#
def backup(top_folder):

    # the admin must make sure the folder does not exist
    if os.path.isdir(top_folder):
        raise Exception("folder '%s' already exists" % top_folder)

    folder  = os.path.join(top_folder, "temp")
    etc_dir = os.path.join(folder, "etc")
    var_dir = os.path.join(folder, "var")

    # and recreate
    os.makedirs(var_dir)

    # copy db
    if True:

        p1 = os.path.join(UiPath.var_dir(), "db", "config.sqlite")
        p2 = os.path.join(UiPath.var_dir(), "db", "config.dump.json")

        shutil.copy2(p1, os.path.join(var_dir, "config.sqlite"))
        if os.path.isfile(p2):
            shutil.copy2(p2, os.path.join(var_dir, "config.dump.json"))

    # always copy ui version
    shutil.copy2(os.path.join(UiPath.etc_dir(), "version.json"), os.path.join(var_dir, "version.json"))

    # construct patters to ignore
    to_ignore = []
    if False:
        to_ignore.append("license.pem")
    if False:
        to_ignore.extend(["myca.pem", "myca.der"])
    if False:
        to_ignore.extend(["krb5.keytab", "krb5.conf", "ldaps.pem"])

    shutil.copytree(
        DaemonPath.etc_dir(), 
        etc_dir, 
        ignore = shutil.ignore_patterns(*to_ignore)
    )

    # pack some other files
    if True:

        videos_json = os.path.join(DaemonPath.var_dir(), "spool", "youtube_cache", "videos.json")
        if os.path.isfile(videos_json):
            shutil.copy2(videos_json, os.path.join(var_dir, "videos.json"))

    # pack it
    today    = datetime.date.today()
    arc_name = "websafety_backup_%s_%s" % (Build.version(), today.strftime("%Y_%m_%d"))
    arc_path = os.path.join(top_folder, arc_name)
    shutil.make_archive(arc_path, 'zip', folder)

    # and return the name
    return arc_path + ".zip"


#
# main
#
def main():
    
    # define arguments
    parser = argparse.ArgumentParser(description='Automation sample for Web Safety')
    parser.add_argument("--folder", help="full path to folder to save backup to", required=True)

    # parse them
    args = parser.parse_args()    

    logging.basicConfig(format='%(asctime)s %(message)s', level=logging.INFO)   
        
    # do the backup
    zip_file = backup(args.folder)

    # and dump
    logging.info("Backed up to '%s' successfully." % zip_file)
        
if __name__ == '__main__':

    main()
