import os
import shutil


#
# django
#
from django.db import models
from django.core.management import call_command


#
# business logic
#
from daemon.core  import DaemonPath, UiPath
from daemon.utils import FileWriter, JsonDumper

#
# ours
#
from squid.generator     import Generator as SquidGenerator
from safety.generator    import Generator as SafetyGenerator
from traffic.generator   import Generator as TrafficGenerator
from antivirus.generator import Generator as AntivirusGenerator

#
# one and only object, so called root of the websafety-core configuration
#    
class WebSafetyCore(models.Model):   

    # config information (needed for cluster sync)
    version   = models.IntegerField(default=0)
    timestamp = models.IntegerField(default=0)

    def generate(self, target_dir):

        # run all generators
        SafetyGenerator(os.path.join(target_dir, "safety")).generate()        
        SquidGenerator(os.path.join(target_dir, "squid")).generate()        
        AntivirusGenerator(os.path.join(target_dir, "antivirus")).generate()

        # and update config timestamp
        d = {
            'version'   : self.version,
            'timestamp' : self.timestamp
        }
        FileWriter(target_dir).write('config.json', JsonDumper().dumps(d))

    def dump_db(self):

        output_cur = os.path.join(UiPath.var_dir(), "db", "config.dump.json")
        output_new = output_cur + ".new"
        if os.path.isfile(output_new):
            os.unlink(output_new)

        try:
            call_command('dumpdata', '--output=%s' % output_new)

            if os.path.isfile(output_cur):
                os.unlink(output_cur)

            shutil.move(output_new, output_cur)

        except Exception as e:
            print(str(e))

#
# one and only object, root of websafety-ui configuration
#
class WebSafetyUi(models.Model):   

    version   = models.IntegerField(default=0)
    timestamp = models.IntegerField(default=0)

    def generate(self, target_dir):

        # this is the folder to generate to
        traffic_dir = os.path.join(target_dir, "traffic")
        if not os.path.exists(traffic_dir):

            # sometime we end up without the traffic dir at all, so create it here
            os.mkdir(traffic_dir)

        # generate traffic
        TrafficGenerator(traffic_dir).generate()

        # update config timestamp
        d = {
            'version'   : self.version,
            'timestamp' : self.timestamp
        }
        FileWriter(target_dir).write('config.json', JsonDumper().dumps(d))
