import os
import re

#
#
#
from django.db import models, connection, transaction

#
#
#
from squid.upgrader      import Upgrader  as SquidUpgrader
from safety.upgrader     import Upgrader  as SafetyUpgrader
from traffic.upgrader    import Upgrader  as TrafficUpgrader
from node.upgrader       import Upgrader  as NodeUpgrader
from antivirus.upgrader  import Upgrader  as AntivirusUpgrader
from _domain.utils       import read_json_array

#
# upgrading
#
class Upgrader(object):

    def __init__(self, dry_run, version):

        # set default values that we upgrade from
        major = "7"
        minor = "6"

        # if version hint is set, verify and update major and minor
        if version:

            # see if the hint indicates supported version
            match = re.match(r"(\d+)\.(\d+)", version, re.M|re.I)
            if not match:
                raise Exception("Unsupported version to import: %s" % version)

            major = match.group(1)
            minor = match.group(2)

            if major == "3":
                if minor not in ["3"]:
                    raise Exception("Unsupported minor number for major %s, minor %s: %s" % (major, minor, version))

            elif major == "4":
                if minor not in ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"]:
                    raise Exception("Unsupported minor number for major %s, minor %s: %s" % (major, minor, version))
        
            elif major == "5":
                if minor not in ["0","1","2"]:
                    raise Exception("Unsupported minor number for major %s, minor %s: %s" % (major, minor, version))

            elif major == "6":
                if minor not in ["0", "1", "2", "3", "4", "5"]:
                    raise Exception("Unsupported minor number for major %s, minor %s: %s" % (major, minor, version))
            elif major == "7":
                if minor not in ["0", "1", "2", "3", "4", "5", "6"]:
                    raise Exception("Unsupported minor number for major %s, minor %s: %s" % (major, minor, version))
            else:
                raise Exception("Major number of version to import is unknown: %s" % version)

        # store params
        self.dry_run = dry_run
        self.major   = int(major)
        self.minor   = int(minor)

    @transaction.atomic
    def upgrade(self, etc_dir, var_dir):

        # config database *may* be also stored in the var dir as big JSON
        config_json = {}
        if True:
            config_path = ""
            if var_dir and os.path.isdir(var_dir):
                config_path = os.path.join(var_dir, "config.dump.json")
                if not os.path.isfile(config_path):
                    config_json = ""

            if config_path:
                config_json = read_json_array(config_path)

        # import node (sharing)
        NodeUpgrader(self.major, self.minor).upgrade(self.dry_run, etc_dir)

        # import squid
        SquidUpgrader(self.major, self.minor).upgrade(etc_dir, config_json)

        # import traffic
        TrafficUpgrader(self.major, self.minor).upgrade(etc_dir, config_json)

        # import safety
        SafetyUpgrader(self.major, self.minor).upgrade(etc_dir)

        # import antivirus
        AntivirusUpgrader(self.major, self.minor).upgrade(etc_dir)

        # see if we are just testing
        if self.dry_run:

            # oh yes, rollback
            transaction.set_rollback(True)