import os
import traceback

#
# django
#
from django.views     import generic
from django.shortcuts import render

#
# our
#
from daemon.core  import DaemonPath
from daemon.utils import ElevatedCommand
from frame.blogic import UiPublisher, CorePublisher

#
#
#
class ApplyView(generic.TemplateView):

    template_name = "frame/apply.html"

    def get_context_data(self, **kwargs):
        context = super(ApplyView, self).get_context_data(**kwargs)
        context['hide_apply'] = True

        return context

#
#
#
class CommandRestart:

    @staticmethod
    def run(only_reload):

        name = "restart.sh"
        if only_reload:
            name = "reload.sh"

        args = [os.path.join(DaemonPath.bin_dir(), name)]
        
        return ElevatedCommand.run(args)

#
#
#
class RestartReloadViewBase(generic.View):

    def post(self, request, *args, **kwargs):

        is_reload  = self.only_reload
        hide_apply = True

        try:

            # publish our configuration into /opt/websafety-ui/etc for reporting
            UiPublisher.publish()

            # publish the configuration into /opt/websafety/etc
            CorePublisher.publish()

            # and run the restart/reload command
            (exit_code, stdout, stderr) = CommandRestart.run(self.only_reload)
            if exit_code != 0:

                raise Exception(
                    "Cannot restart/reload daemons, exit code %d, stdout %s, stderr %s" % (
                        exit_code, stdout, stderr
                    )
                )

            # good if we got here everything is just fine
            return render(request, 'frame/apply_success.html', locals())

        except Exception as exception:

            all_locals = locals()
            all_locals['traceback'] = traceback.format_exc()

            return render(request, 'frame/apply_failure.html', all_locals)

#
#
#
class ReloadView(RestartReloadViewBase):
    only_reload = True

#
#
#
class RestartView(RestartReloadViewBase):
    only_reload = False
