#!/usr/bin/env python
#
# (c) Diladele B.V. 2018, Web Safety for Squid Proxy
#  
import os
import sys
import django
import logging
import argparse

#
# in order for Django to work correctly with our monitor app, we add ourselves to Python's search path
# 
sys.path.append(os.path.abspath(__file__))

#
# tell Django where to read settings from
#
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "console.settings")

#
#
#
django.setup()

#
# do imports
#
from daemon.core  import DaemonPath, UiPath
from frame.models import WebSafetyCore, WebSafetyUi
    
#
#
#
def generate(args):

    if args.core:

        obj = WebSafetyCore()
        obj.generate(DaemonPath.etc_dir())

    elif args.ui:

        obj = WebSafetyUi()
        obj.generate(UiPath.etc_dir())

    else:
        pass

#
# main
#
def main():

    # define arguments
    parser = argparse.ArgumentParser(
        description='''
            Generator of configuration files from UI's SQLite config storage.
            Note: run as "sudo -u proxy python3 generate.py --core" for websafety-core 
            config files and "sudo -u websafety python3 generate.py --ui" for 
            websafety-ui config files. Usually is only run once upon installation.
        '''
    )

    parser.add_argument("--core", required=False, default=False, action='store_true', help="for websafety-core config")
    parser.add_argument("--ui", required=False, default=False, action='store_true', help="for websafety-ui config")
    
    # parse them (none but anyway)
    args = parser.parse_args()    

    logging.basicConfig(format='%(asctime)s %(message)s', level=logging.INFO)   
        
    # add domain
    generate(args)

    logging.info("Generator finished successfully. Please run /opt/websafety/bin/restart.sh or /opt/websafety/bin/reload.sh as desired!")
        
if __name__ == '__main__':

    main()