#
# django
#
from django.db import models

#
# business logic
#
from daemon.core import System, Distrib
from _domain.utils import LicenseManager

#
#
#
class Node(models.Model):   

    def can_manage_network(self):

        # no, if run on azure - will uncomment when building image for azure
        #return False

        # for debug
        if System.WS_WINDOWS == System.name():
            return True

        if System.WS_LINUX == System.name():
            if Distrib.name() in [Distrib.WS_DEBIAN11, Distrib.WS_UBUNTU20]:
                return True

        return False

    def can_manage_timezone(self):

        # for debug
        if System.WS_WINDOWS == System.name():
            return True

        if System.WS_LINUX == System.name():
            if Distrib.name() in [Distrib.WS_DEBIAN11, Distrib.WS_UBUNTU20]:
                return True

        return False

#
#
#
class NetAdvanced(models.Model):

    NETWORK_TYPE_NONE       = 0
    NETWORK_TYPE_INTERFACES = 1
    NETWORK_TYPE_NETPLAN    = 2
    NETWORK_TYPE_CHOICES    = (
        (NETWORK_TYPE_NONE,       "Not managed by this UI"),
        (NETWORK_TYPE_INTERFACES, "Debian 11 (/etc/network/interfaces)"),
        (NETWORK_TYPE_NETPLAN,    "Ubuntu 20 (netplan)"),
    )

    network_type = models.IntegerField(default=NETWORK_TYPE_NONE,choices=NETWORK_TYPE_CHOICES)
    manual_text  = models.TextField(blank=True)  # settings for manual /etc/network/interfaces or /etc/netplan/*.yml file

#
#
#
class NetDevice(models.Model):

    CONFIG_TYPE_NONE     = 0
    CONFIG_TYPE_DHCP     = 1
    CONFIG_TYPE_STATIC   = 2
    #CONFIG_TYPE_LOOPBACK = 3
    #CONFIG_TYPE_MANUAL   = 4
    CONFIG_TYPE_CHOICES  = (
        (CONFIG_TYPE_NONE,     "None"),
        (CONFIG_TYPE_DHCP,     "DHCP"),
        (CONFIG_TYPE_STATIC,   "Static"),
        #(CONFIG_TYPE_LOOPBACK, "Loopback"),
        #(CONFIG_TYPE_MANUAL,   "Manual")
    )

    name  = models.CharField(max_length=64,  unique=True)
    mac   = models.CharField(max_length=128, blank=True)
    state = models.CharField(max_length=16,  blank=True)

    # ip4
    ip4_config     = models.IntegerField(default=CONFIG_TYPE_DHCP,choices=CONFIG_TYPE_CHOICES)
    ip4_address    = models.CharField(max_length=254, blank=True)
    ip4_netmask    = models.CharField(max_length=254, blank=True)
    ip4_gateway    = models.CharField(max_length=254, blank=True)
    ip4_dns_srv1   = models.CharField(max_length=254, blank=True)
    ip4_dns_srv2   = models.CharField(max_length=254, blank=True)
    ip4_dns_search = models.CharField(max_length=254, blank=True)
    
    def ip4_netmask_as_int(self):

        values = {
            
            30 : "255.255.255.252",
            29 : "255.255.255.248",
            28 : "255.255.255.240",
            27 : "255.255.255.224",
            26 : "255.255.255.192",
            25 : "255.255.255.128",
            24 : "255.255.255.0",
            23 : "255.255.254.0",
            22 : "255.255.252.0",
            21 : "255.255.248.0",
            20 : "255.255.240.0",
            19 : "255.255.224.0",
            18 : "255.255.192.0",
            17 : "255.255.128.0",
            16 : "255.255.0.0",
        }

        for value in values:
            if self.ip4_netmask == values[value]:
                return value

        raise Exception("Cannot convert IPv4 netmask '%s' into integer" % self.ip4_netmask)

    def ip4_config_type_etc_network_interfaces_str(self):

        if NetDevice.CONFIG_TYPE_DHCP == self.ip4_config:
            return "dhcp"
        if NetDevice.CONFIG_TYPE_STATIC == self.ip4_config:
            return "static"
        #if NetDevice.CONFIG_TYPE_LOOPBACK == self.ip4_config:
        #    return "loopback"
        #if NetDevice.CONFIG_TYPE_MANUAL == self.ip4_config:
        #    return "manual"
        return "none"


    # ip6
    ip6_config     = models.IntegerField(default=CONFIG_TYPE_NONE,choices=CONFIG_TYPE_CHOICES)
    ip6_address    = models.CharField(max_length=254, blank=True)
    ip6_subnet     = models.IntegerField(default=64)
    ip6_gateway    = models.CharField(max_length=254, blank=True)
    ip6_dns_srv1   = models.CharField(max_length=254, blank=True)
    ip6_dns_srv2   = models.CharField(max_length=254, blank=True)
    ip6_dns_search = models.CharField(max_length=254, blank=True)
    
    def ip6_config_type_etc_network_interfaces_str(self):

        if NetDevice.CONFIG_TYPE_DHCP == self.ip6_config:
            return "dhcp"
        if NetDevice.CONFIG_TYPE_STATIC == self.ip6_config:
            return "static"
        #if NetDevice.CONFIG_TYPE_LOOPBACK == self.ip6_config:
        #    return "loopback"
        #if NetDevice.CONFIG_TYPE_MANUAL == self.ip6_config:
        #    return "manual"
        return "none"

    def __str__(self):
        return self.name

