import os

#
#
#
from _domain.utils import read_json_array, read_json_object
from _domain.node  import Sharing, ClusterClient, ClusterServer


#
#
#
class SharingImporter:

    def upgrade(self, dry_run, etc_dir):

        # construct the path
        path = os.path.join(etc_dir, "safety", "sharing.json")        
        if not os.path.exists(path):            
            path = os.path.join(etc_dir, "node", "sharing.json")            
            if not os.path.exists(path):
                return

        # the import is here straight forward, just load data from file and re-save it
        obj = Sharing()
        obj.load_from(path)

        if not dry_run:
            obj.save()

#
#
#
class ClusterImporter:

    def upgrade(self, dry_run, etc_dir):

        self.upgrade_client(dry_run, etc_dir)
        self.upgrade_server(dry_run, etc_dir)

    def upgrade_client(self, dry_run, etc_dir):

        path = os.path.join(etc_dir, "node", "cluster_client.json")
        if not os.path.exists(path):
            return

        # the import is here straight forward, just load data from file and re-save it
        obj = ClusterClient()
        obj.load_from(path)
        
        if not dry_run:
            obj.save()

    def upgrade_server(self, dry_run, etc_dir):

        path = os.path.join(etc_dir, "node", "cluster_server.json")
        if not os.path.exists(path):
            return

        # the import is here straight forward, just load data from file and re-save it
        obj = ClusterServer()
        obj.load_from(path)
        
        if not dry_run:
            obj.save()


        

#
#
#
class Upgrader(object):

    def __init__(self, major, minor):

        self.major   = major
        self.minor   = minor

    def upgrade(self, dry_run, etc_dir):

        SharingImporter().upgrade(dry_run, etc_dir)
        ClusterImporter().upgrade(dry_run, etc_dir)

