import os
import json

#
#
#
from daemon.core  import DaemonPath, UiPath, SquidUser, SquidPath
from daemon.utils import folder_size

#
#
#
from django.views import generic
from django.http import HttpResponse

#
#
#
from .api_ps import CommandPs
from .api_system import CommandDiskFree

#
#
#
class ViewApiDashWsmgrd(generic.View):

    def get(self, request, *args, **kwargs):

        # allocate default response
        data = { 'error' : False, 'desc' : '', 'info' : {}, 'stats' : {} }

        # dump amount of free place
        (success, info) = CommandDiskFree().run(os.path.join(DaemonPath.var_dir(), "monitor"))
        if success:
            disk = {
                'free'  : info['avail'],
                'used'  : info['used'],
                'total' : info['size'],
                'ratio' : info['ratio'].replace('%', '')
            }
            data['stats']['disk'] = disk

        # we also need to know how many files to upload
        data['stats']['var_log_squid'] = folder_size(SquidPath.log_dir())
        data['stats']['stats_dir']     = folder_size(os.path.join(UiPath.var_dir(), "stats"))
        data['stats']['reports_dir']   = folder_size(os.path.join(UiPath.var_dir(), "reports"))
        
        # get processes wsmgrd daemon
        processes = []
        try:
            processes = CommandPs(SquidUser.name()).run()
        except Exception as e:
            data['error'] = True
            data['desc']  = str(e)

        # see if wsmgrd binary is there and fill the stats
        found = False
        for process in processes:
            if process['path'].find(DaemonPath.bin_dir() + "/wsmgrd") != -1:
                found = True
                data['info']['path']       = process['path']
                data['info']['pid']        = process['pid']
                data['info']['user']       = process['user']
                data['stats']['cpu_time']  = process['cpu_time']
                data['stats']['cpu_usage'] = process['cpu_usage']
                data['stats']['mem_size']  = int(process['mem_size'])
                data['stats']['mem_usage'] = process['mem_usage']

        # no wsmgrd daemon means something is really bad tell the caller
        if not found:
            data['error'] = True
            data['desc']  = 'the wsmgrd daemon is not running'

        # add the processes anyway
        data['info']['processes'] = processes

        # and store as array
        return HttpResponse(json.dumps([data], ensure_ascii=True), content_type='application/json')