import os
import json
import shutil
import traceback

#
#
#
from daemon.core import UiPath, UiUser
from _domain.safety import WsYtgdStatus

#
# django
#
from django.views import generic
from django.http import HttpResponse

#
#
#
from .api_ps        import CommandPs
from .api_systemctl import CommandSystemCtl

#
#
#
class WsmondInfo(object):

    def __init__(self):
        
        self.path = ""
        self.pid  = ""
        self.user = ""

#
#
#
class WsmondStats(object):

    def __init__(self):
        
        self.cpu_time   = ""
        self.cpu_usage  = ""
        self.mem_size   = 0
        self.mem_usage  = ""
        self.hdd_size   = 0
        self.hdd_usage  = ""

#
#
#
class ViewApiDashWsmondRuntime(generic.View):

    def get_status(self):

        try:
            return CommandSystemCtl("wsmond").run()
        except Exception as e:
            return str(e)

    def get_process(self):

        # get processes of wsmond daemon
        processes = CommandPs(UiUser.name()).run()
        
        # see if realtime.py binary is there and fill the stats
        for process in processes:

            if process['path'].find(UiPath.bin_dir() + "/realtime.py") != -1:

                # ok found, as we know the wsmond process is single per system, fill info
                info = WsmondInfo()
                if True:
                    info.path = process['path']
                    info.pid  = process['pid']
                    info.user = process['user']

                # fill stats
                stats = WsmondStats()
                if True:

                    stats.cpu_time  = process['cpu_time']
                    stats.cpu_usage = process['cpu_usage']
                    stats.mem_size  = int(process['mem_size'])
                    stats.mem_usage = process['mem_usage']

                    # fill in the hdd usage (based on stats folder but it actually does not matter as usage is calculated per mounted filesystem)
                    usage = shutil.disk_usage(os.path.join(UiPath.var_dir(), "stats"))

                    stats.hdd_size  = usage.used
                    stats.hdd_usage = int( (float(usage.used) / float(usage.total)) * 100)

                # perfect now return
                return (info, stats, [process])

        # no daemon means something is really bad tell the caller
        raise Exception("The wsmond daemon is not running")

    def get(self, request, *args, **kwargs):

        # allocate default response
        data = { 'error' : False, 'desc' : '', 'info' : {}, 'stats' : {}, 'runtime' : {}, 'status' : '' }

        try:

            # save status
            data['status'] = self.get_status()

            # get process info
            info, stats, processes = self.get_process()

            # convert info
            data['info'] = {
                'path' : info.path,
                'pid'  : info.pid,
                'user' : info.user
            }

            # convert stats to json and store in data
            data['stats'] = {
                'cpu_time'  : stats.cpu_time,
                'cpu_usage' : stats.cpu_usage,
                'mem_size'  : stats.mem_size,
                'mem_usage' : stats.mem_usage,
                'hdd_size'  : stats.hdd_size,
                'hdd_usage' : stats.hdd_usage,
            }

            # and store processes too
            data['processes'] = processes
            
        except Exception as e:
            
            data['error']     = True
            data['desc']      = str(e)
            data['traceback'] = "%s" % traceback.format_exc()
        
        return HttpResponse(json.dumps([data], ensure_ascii=True), content_type='application/json')
