import os
import json

#
#
#
from django import forms
from django.views import generic
from django.urls import reverse_lazy
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.contrib import messages
from django.contrib.messages.views import SuccessMessageMixin

#
#
#
from _domain.node  import Sharing, SharingFactory

#
#
#
class SharingForm(forms.Form):

    upload_recategorization   = forms.BooleanField(required=False)
    upload_telemetry_basic    = forms.BooleanField(required=False)
    upload_telemetry_extended = forms.BooleanField(required=False)

#
#
#
class ViewDataChoices(SuccessMessageMixin, generic.edit.UpdateView):

    template_name   = "node/tools/sharing.html"
    success_message = "need_squid_restart"
    form_class      = SharingForm

    def get_success_url(self):
        return reverse_lazy("node:ViewDataChoices")

    def get(self, request, *args, **kwargs):

        obj  = SharingFactory().construct()
        form = SharingForm(initial = obj.as_data())

        return render(request, self.template_name, {'form': form })

    def post(self, request, *args, **kwargs):

        form  = SharingForm(request.POST)
        if form.is_valid():

            # ok form is valid, fill the data
            obj = SharingFactory().construct()
            obj.upload_recategorization   = form.cleaned_data['upload_recategorization']
            obj.upload_telemetry_basic    = form.cleaned_data['upload_telemetry_basic']
            obj.upload_telemetry_extended = form.cleaned_data['upload_telemetry_extended']
            obj.save()

            # mark as needing relstart
            messages.info(request, "need_squid_restart")

            return HttpResponseRedirect(self.get_success_url()) 

        return render(request, self.template_name, {'form': form })

