import os
import datetime

#
#
#
from django.views import generic

#
#
#
from daemon.core   import DaemonPath, UiPath, Build, CoreBuild, Distrib, System
from daemon.utils  import JsonReader
from _domain.utils import FolderInfo

#
#
#
class ViewSystemInfo(generic.TemplateView):

    template_name = "node/tools/system_info.html"

    def get_context_data(self, **kwargs):
        
        context = super(ViewSystemInfo, self).get_context_data(**kwargs)

        # add hardcoded settings
        context['WEBSAFETY_ARCH']    = Distrib.arch()
        context['WEBSAFETY_DISTRIB'] = Distrib.name()
        context['WEBSAFETY_SYSTEM']  = System.name()

        # add UI settings
        context['ui'] = {
            "etc_dir_path" : UiPath.etc_dir(),
            "etc_dir_size" : int(FolderInfo(UiPath.etc_dir()).get_size()),
            "var_dir_path" : UiPath.var_dir(),
            "var_dir_size" : int(FolderInfo(UiPath.var_dir()).get_size()),
            "bin_dir_path" : UiPath.bin_dir(),
            "bin_dir_size" : int(FolderInfo(UiPath.bin_dir()).get_size()),
            "version"      : Build.version()
        }

        # read the configuration file
        core_version   = CoreBuild.version()
        conf_timestamp = ""
        conf_version   = ""
        conf_date      = ""
        conf_error     = ""
        try:
            
            # read json file
            v = JsonReader(
                os.path.join(DaemonPath.etc_dir(), "config.json")
            ).read_object()

            # and assign
            conf_timestamp = v["timestamp"]
            conf_version   = v["version"]
            conf_date      = str(datetime.datetime.fromtimestamp(int(conf_timestamp) / 1000))

        except Exception as e:
            conf_error = str(e)            

        context['core'] = {

            "etc_dir_path" : DaemonPath.etc_dir(),
            "etc_dir_size" : int(FolderInfo(DaemonPath.etc_dir()).get_size()),
            "var_dir_path" : DaemonPath.var_dir(),
            "var_dir_size" : int(FolderInfo(DaemonPath.var_dir()).get_size()),
            "bin_dir_path" : DaemonPath.bin_dir(),
            "bin_dir_size" : int(FolderInfo(DaemonPath.bin_dir()).get_size()),
            "version"      : core_version, 
            "config_info"  : {
                "timestamp" : conf_timestamp,
                "version"   : conf_version,
                "error"     : conf_error,
                "date"      : conf_date
            }
        }

        return context