#
# django
#
from django.db import models

#
# our
#
from _domain.utils  import LicenseManager
from _domain.safety import BuildInCategories

#
#
#
class Safety(models.Model):

    def get_uicolor(self):

        try:
            (result, license) = LicenseManager().get()

            if result is True:

                if license['valid'] == "1":

                    if license['type'] == "community":

                        # updated the flag (used in views!)
                        self.is_community = True

                        # and return the community color
                        return 'black-blue'

                    if license['expires_soon'] == "1":
                        return 'cyan'
                    else:
                        return 'default'

        except Exception as e:
            pass

        return 'red'

    





#
#
#
class Network(models.Model):

    enable_icap        = models.BooleanField(default=True)                 # sets icap_enable on|off
    bypass_to_localnet = models.BooleanField(default=True)                 # connections to hosts in our LAN does not need to be filtered

    WSICAP_LOG_DEBUG   = 0
    WSICAP_LOG_INFO    = 1
    WSICAP_LOG_WARN    = 2
    WSICAP_LOG_ERROR   = 3
    WSICAP_LOG_CHOICES = (
        (WSICAP_LOG_DEBUG, "debug"),
        (WSICAP_LOG_INFO,  "info"),
        (WSICAP_LOG_WARN,  "warning"),
        (WSICAP_LOG_ERROR, "error")
    )

    wsicap_address = models.CharField(max_length=200, default="127.0.0.1") # address of ICAP server for web safety
    wsicap_port    = models.IntegerField(default=1344)                     # port
    wsicap_debug   = models.BooleanField(default=False)                    # special debug mode of web safety when it dumps ICAP req/resp contents on disk   
    wsicap_threads = models.IntegerField(default=9)
    wsicap_logging = models.IntegerField(choices=WSICAP_LOG_CHOICES,default=WSICAP_LOG_INFO)


#
#
#
class Apps(models.Model):    
    google_apps_allowed_domains = models.CharField(max_length=254, blank=True, unique=True)

#
#
#
class YouTube(models.Model):

    bypass_to_localnet  = models.BooleanField(default=True)     # connections to hosts in our LAN does not need to be filtered
    api_key             = models.CharField(max_length=200)      # get it from youtube api site
    daemon_port         = models.IntegerField(default=18891)    # default port on 127.0.0.1 that the wsytgd daemon listens on for requests
    verbose             = models.BooleanField(default=False)
    cache_expiration    = models.IntegerField(default=432000)   # 5 days in seconds
    

#
#
#
class Annoyances(models.Model):

    # general settings
    apply_replace_heuristics = models.BooleanField(default=True)
    replace_with_image       = models.BooleanField(default=True)
    hide_tags                = models.BooleanField(default=True)
    mangle_id                = models.BooleanField(default=False)
    mangle_class             = models.BooleanField(default=False)
    scan_external_links      = models.BooleanField(default=False)

    # adblock subscriptions
    enable_bulgarian         = models.BooleanField(default=False)
    enable_chinese           = models.BooleanField(default=False)
    enable_czech             = models.BooleanField(default=False)
    enable_danish            = models.BooleanField(default=False)
    enable_dutch             = models.BooleanField(default=False)
    enable_english           = models.BooleanField(default=True)
    enable_estonian          = models.BooleanField(default=False)
    enable_french            = models.BooleanField(default=False)
    enable_german            = models.BooleanField(default=False)
    enable_greek             = models.BooleanField(default=False)
    enable_hungarian         = models.BooleanField(default=False)
    enable_indian            = models.BooleanField(default=False)
    enable_israeli           = models.BooleanField(default=False)
    enable_italian           = models.BooleanField(default=False)
    enable_japanese          = models.BooleanField(default=False)
    enable_korean            = models.BooleanField(default=False)
    enable_latvian           = models.BooleanField(default=False)
    enable_lithuanian        = models.BooleanField(default=False)
    enable_polish            = models.BooleanField(default=False)
    enable_portuguese        = models.BooleanField(default=False)
    enable_romanian          = models.BooleanField(default=False)
    enable_russian           = models.BooleanField(default=False)
    enable_slovak            = models.BooleanField(default=False)
    enable_spanish           = models.BooleanField(default=False)
    enable_swedish           = models.BooleanField(default=False)
    enable_turkish           = models.BooleanField(default=False)
    enable_vietnamese        = models.BooleanField(default=False)

    # privacy subscriptions
    enable_adware            = models.BooleanField(default=False)
    enable_annoyance         = models.BooleanField(default=False)
    enable_antiadb           = models.BooleanField(default=False)
    enable_cookies           = models.BooleanField(default=False)
    enable_privacy           = models.BooleanField(default=False)
    enable_social            = models.BooleanField(default=False)
    enable_tracking          = models.BooleanField(default=False)
    
    # youtube subscriptions
    enable_yt_comments       = models.BooleanField(default=False)
    enable_yt_suggestions    = models.BooleanField(default=False)
    enable_yt_other          = models.BooleanField(default=False)

    # custom subscriptions
    enable_custom1           = models.BooleanField(default=True)
    enable_custom2           = models.BooleanField(default=True)    

#
#
#
class CustomCategory(models.Model):

    class Meta:
        ordering = ["name"]

    name     = models.CharField(max_length=254, primary_key=True, unique=True)
    title    = models.CharField(max_length=254)
    trust    = models.BooleanField(default=False)   # the category may be marked as trusted (i.e. skip adult language detection on it)
    sslbump  = models.BooleanField(default=True)    # the category may be marked as non bumpable (i.e. skip SSL bump on it)

    def __unicode__(self):
        return self.title

#
#
#
class Categories(models.Model):

    trust_adult_themes_sexuality       = models.BooleanField(default=False)
    trust_advertising                  = models.BooleanField(default=True)
    trust_alcohol_tobacco              = models.BooleanField(default=False)
    trust_anime_manga_webcomic         = models.BooleanField(default=False)
    trust_arts_society_culture         = models.BooleanField(default=False)
    trust_automotive                   = models.BooleanField(default=True)
    trust_banking_insurance_finance    = models.BooleanField(default=True)
    trust_blogs_personal               = models.BooleanField(default=False)
    trust_business_services_industry   = models.BooleanField(default=True)
    trust_classifieds_auctions         = models.BooleanField(default=False)
    trust_content_delivery_networks    = models.BooleanField(default=False)
    trust_cracks_hacking_illegal       = models.BooleanField(default=False)
    trust_dating                       = models.BooleanField(default=False)
    trust_drugs                        = models.BooleanField(default=False)
    trust_ecommerce_shopping           = models.BooleanField(default=True)
    trust_education_science_research   = models.BooleanField(default=True)
    trust_entertainment_humor_hobby    = models.BooleanField(default=False)
    trust_expired_parked               = models.BooleanField(default=False)
    trust_fashion_beauty_cosmetics     = models.BooleanField(default=True)
    trust_file_storage                 = models.BooleanField(default=False)
    trust_food_dining_restaurants      = models.BooleanField(default=True)
    trust_forums_message_boards        = models.BooleanField(default=False)
    trust_gambling                     = models.BooleanField(default=False)
    trust_games                        = models.BooleanField(default=False)
    trust_generic_non_categorized      = models.BooleanField(default=False)
    trust_government                   = models.BooleanField(default=True)
    trust_hate_discrimination_violence = models.BooleanField(default=False)
    trust_health_medicine_fitness      = models.BooleanField(default=True)
    trust_jobs_employment_career       = models.BooleanField(default=True)
    trust_messaging_chat               = models.BooleanField(default=False)
    trust_movies                       = models.BooleanField(default=True)
    trust_music_radio                  = models.BooleanField(default=True)
    trust_network_infrastructure       = models.BooleanField(default=True)
    trust_news_media                   = models.BooleanField(default=True)
    trust_nudity_pornography           = models.BooleanField(default=False)
    trust_p2p_file_sharing             = models.BooleanField(default=False)
    trust_photo_sharing                = models.BooleanField(default=False)
    trust_politics                     = models.BooleanField(default=True)
    trust_portals                      = models.BooleanField(default=False)
    trust_proxy_anonymizer             = models.BooleanField(default=False)
    trust_real_estate_home_interior    = models.BooleanField(default=True)
    trust_religious                    = models.BooleanField(default=True)
    trust_search_engines               = models.BooleanField(default=False)
    trust_self_harm                    = models.BooleanField(default=False)
    trust_social_networking            = models.BooleanField(default=True)
    trust_software_technology_hardware = models.BooleanField(default=True)
    trust_sports                       = models.BooleanField(default=True)
    trust_television                   = models.BooleanField(default=True)
    trust_travel                       = models.BooleanField(default=True)
    trust_user_tracking                = models.BooleanField(default=False)
    trust_video_sharing                = models.BooleanField(default=False)
    trust_weapons                      = models.BooleanField(default=False)
    trust_webmail                      = models.BooleanField(default=True)


    def categories_as_array(self):

        array = []
        for name in BuildInCategories.CATEGORY_NAMES:

            attrname = "trust_%s" % name
            assigned = getattr(self, attrname)
            
            if assigned:
                array.append(name)

        return array

    def categories_str(self):

        values = self.categories_as_array()
        return ", ".join(values)


class RecategorizedDomain(models.Model):

    name                                = models.CharField(max_length=256, db_index=True, unique=True) # UNIQUE NOT NULL COLLATE NOCASE    

    assign_adult_themes_sexuality       = models.BooleanField(default=False)
    assign_advertising                  = models.BooleanField(default=False)
    assign_alcohol_tobacco              = models.BooleanField(default=False)
    assign_anime_manga_webcomic         = models.BooleanField(default=False)
    assign_arts_society_culture         = models.BooleanField(default=False)
    assign_automotive                   = models.BooleanField(default=False)
    assign_banking_insurance_finance    = models.BooleanField(default=False)
    assign_blogs_personal               = models.BooleanField(default=False)
    assign_business_services_industry   = models.BooleanField(default=False)
    assign_classifieds_auctions         = models.BooleanField(default=False)
    assign_content_delivery_networks    = models.BooleanField(default=False)
    assign_cracks_hacking_illegal       = models.BooleanField(default=False)
    assign_dating                       = models.BooleanField(default=False)
    assign_drugs                        = models.BooleanField(default=False)
    assign_ecommerce_shopping           = models.BooleanField(default=False)
    assign_education_science_research   = models.BooleanField(default=False)
    assign_entertainment_humor_hobby    = models.BooleanField(default=False)
    assign_expired_parked               = models.BooleanField(default=False)
    assign_fashion_beauty_cosmetics     = models.BooleanField(default=False)
    assign_file_storage                 = models.BooleanField(default=False)
    assign_food_dining_restaurants      = models.BooleanField(default=False)
    assign_forums_message_boards        = models.BooleanField(default=False)
    assign_gambling                     = models.BooleanField(default=False)
    assign_games                        = models.BooleanField(default=False)
    assign_generic_non_categorized      = models.BooleanField(default=False)
    assign_government                   = models.BooleanField(default=False)
    assign_hate_discrimination_violence = models.BooleanField(default=False)
    assign_health_medicine_fitness      = models.BooleanField(default=False)
    assign_jobs_employment_career       = models.BooleanField(default=False)
    assign_messaging_chat               = models.BooleanField(default=False)
    assign_movies                       = models.BooleanField(default=False)
    assign_music_radio                  = models.BooleanField(default=False)
    assign_network_infrastructure       = models.BooleanField(default=False)
    assign_news_media                   = models.BooleanField(default=False)
    assign_nudity_pornography           = models.BooleanField(default=False)
    assign_p2p_file_sharing             = models.BooleanField(default=False)
    assign_photo_sharing                = models.BooleanField(default=False)
    assign_politics                     = models.BooleanField(default=False)
    assign_portals                      = models.BooleanField(default=False)
    assign_proxy_anonymizer             = models.BooleanField(default=False)
    assign_real_estate_home_interior    = models.BooleanField(default=False)
    assign_religious                    = models.BooleanField(default=False)
    assign_search_engines               = models.BooleanField(default=False)
    assign_self_harm                    = models.BooleanField(default=False)
    assign_social_networking            = models.BooleanField(default=False)
    assign_software_technology_hardware = models.BooleanField(default=False)
    assign_sports                       = models.BooleanField(default=False)
    assign_television                   = models.BooleanField(default=False)
    assign_travel                       = models.BooleanField(default=False)
    assign_user_tracking                = models.BooleanField(default=False)
    assign_video_sharing                = models.BooleanField(default=False)
    assign_weapons                      = models.BooleanField(default=False)
    assign_webmail                      = models.BooleanField(default=False)


    def __unicode__(self):
        return self.name

    def categories_as_array(self):

        array = []
        for name in BuildInCategories.CATEGORY_NAMES:

            attrname = "assign_%s" % name
            assigned = getattr(self, attrname)
            
            if assigned:
                array.append(name)

        return array

    def categories_str(self):

        values = self.categories_as_array()
        return ", ".join(values)

#
#
#
class FileType(models.Model):
    name = models.CharField(max_length=254, primary_key=True, unique=True)

#
#
#
class PhraseEngine(models.Model):

    PHRASE_ENGINE_COUNT_MULTIPLE = 0
    PHRASE_ENGINE_COUNT_ONCE     = 1
    PHRASE_ENGINE_COUNT_CHOICES = (
        (PHRASE_ENGINE_COUNT_MULTIPLE, "Count all phrase occurance in text"),
        (PHRASE_ENGINE_COUNT_ONCE,     "Count phrase occurance only once in text")
    )
    count_type = models.IntegerField(default=PHRASE_ENGINE_COUNT_MULTIPLE,choices=PHRASE_ENGINE_COUNT_CHOICES)

#
#
#
class Policy(models.Model):

    name     = models.CharField(max_length=254, unique=True)
    priority = models.IntegerField(default=1)

    def __unicode__(self):
        return self.name

    #
    # block page bypass settings of a policy
    #
    
    # type of bypass
    bypass_allowed        = models.BooleanField(default=False)  # enable of disable bypass
    bypass_strip_www      = models.BooleanField(default=True)   # www.example.lan is bypassed if we bypass example.lan and vice versa
    bypass_children       = models.BooleanField(default=True)   # cdn.example.lan is bypassed if we bypass example.lan
    bypass_category       = models.BooleanField(default=False)  # not implemented
    bypass_all            = models.BooleanField(default=False)  # not implemented
    bypass_referers       = models.BooleanField(default=False)  # all URLs that have example.lan in referer are also bypassed
    bypass_duration       = models.IntegerField(default=60)     # by default each 60 minutes you will see the click_to_bypass screen
    bypass_token_required = models.BooleanField(default=False)  # bypass is ONLY allowed with the token

    # list of modules that can be bypassed (same as in exclusions) by default - everything is bypassed
    bypass_adult          = models.BooleanField(default=True)
    bypass_categories     = models.BooleanField(default=True)
    bypass_file           = models.BooleanField(default=True)
    bypass_adblock        = models.BooleanField(default=True)
    bypass_privacy        = models.BooleanField(default=True)
    bypass_http           = models.BooleanField(default=True)

    #
    # bandwidth limitations of a policy
    #
    bandwidth_limit_enable = models.BooleanField(default=False)  # enable of disable bandwidth limitations
    bandwidth_policy_limit = models.IntegerField(default=1024)   # all members of this policy cannot consume more than N KB per second
    bandwidth_member_limit = models.IntegerField(default=100)    # each member of this policy cannot consume more than N KB per second

    def get_delay_pool_class(self):

        # there is no sence in calling this function if no bandwidth limits are set
        if not self.bandwidth_limit_enable:
            raise Exception("Bandwidth limits are not set, cannot determine the delay_pool class")

        # we are enforcing the bandwidth limitations per policy by raising special tag in out wsicap daemon, so always return 5
        # return 5

        # we only set total and individual limits, so class is 2 
        return 2

    def get_delay_pool_parameters(self):

        # for more information, see the delay_parameters description in delay_pool.conf template file

        # the speed limit parameters for the aggregate bucket; assume no limits
        aggregate = "none"

        if self.bandwidth_policy_limit != 0:
            aggregate = "%d/%d" % (self.bandwidth_policy_limit * 1024, self.bandwidth_policy_limit*1024*2)

        # the speed limit parameters for the individual
        individual = "none"
        if self.bandwidth_member_limit != 0:
            individual = "%d/%d" % (self.bandwidth_member_limit * 1024, self.bandwidth_member_limit*1024*2)

        # note I do NOT understand the n/n format in aggregate/individual values - the fact that second
        # valud is twice as big as the first - I found it somewhere on the internet :(, sorry.

        # and combine the two
        return aggregate + " " + individual

#
#
#
class PolicyUtils(object):

    # we now constuct the corresponding one <-> one related objects that MUST exist prior to us be able to use the policy in the UI
    # another way would be to create each one <-> one object upon accessing it but it gives a lot of too-dynamic behaviour for
    # the system and I do not like that
    @staticmethod
    def create_default_rules(policy):

        # NOTE: the policy object MUST be already saved into the database! It gets the PK by doing that        
        policy.advanced = Advanced(
            policy=policy, 
            enable=False, 
            comment="New filtering policy. Specify meaningful comment here. Please adjust this policy settings and enable it afterwards."
        )
        policy.advanced.save()

        policy.ruleannoyances = RuleAnnoyances(policy=policy)
        policy.ruleannoyances.save()

        policy.ruleadult = RuleAdult(policy=policy)
        policy.ruleadult.save()

        policy.ruleapps = RuleApps(policy=policy)
        policy.ruleapps.save()

        policy.ruleyoutube = RuleYouTube(policy=policy)
        policy.ruleyoutube.save()

        policy.ruleyoutubecategory = RuleYouTubeCategory(policy=policy)
        policy.ruleyoutubecategory.save()

        policy.rulefilesize = RuleFileSize(policy=policy)
        policy.rulefilesize.save()

        policy.rulecategory = RuleCategory(policy=policy)
        policy.rulecategory.save()

        policy.ruledynamiccategory = RuleDynamicCategory(policy=policy)
        policy.ruledynamiccategory.save()



    




#
# token list (global, not related to the policy) - ideally this needs to be the User model!
#
class ByPassToken(models.Model):

    class Meta:
        ordering = ["name"]

    value   = models.CharField(max_length=254, db_index=True, unique=True)    # token thats need to be typed into bypass popup (unique)
    name    = models.CharField(max_length=254, db_index=True)                 # name of the user who is allowed to bypass (visible in reports only)    
    comment = models.TextField(blank=True)


#
# schedule (like in Squid acl)
#
class Schedule(models.Model):

    policy = models.ForeignKey(Policy, on_delete=models.CASCADE)

    # schedule - days
    on_mon = models.BooleanField(default=True)
    on_tue = models.BooleanField(default=True)
    on_wed = models.BooleanField(default=True)
    on_thu = models.BooleanField(default=True)
    on_fri = models.BooleanField(default=True)
    on_sat = models.BooleanField(default=True)
    on_sun = models.BooleanField(default=True)

    # schedule - from 00:00 to 23:59
    from_hours = models.IntegerField(default=0)
    from_mins  = models.IntegerField(default=0)
    to_hours   = models.IntegerField(default=23)
    to_mins    = models.IntegerField(default=59)

#
#
#
class Advanced(models.Model):

    policy               = models.OneToOneField(Policy, primary_key=True, on_delete=models.CASCADE)
    comment              = models.TextField()
    enable               = models.BooleanField(default=True)
    sslbump              = models.BooleanField(default=True)    
    exclude_by_referer   = models.BooleanField(default=False)
    ignore_case          = models.BooleanField(default=True)
    hide_history         = models.BooleanField(default=False)
    hide_result_info     = models.BooleanField(default=True)    # if True - explicit results of what has triggered block are hidden on the blocked page
    tunnel_block         = models.BooleanField(default=True)    # True - postpone blocking after CONNECT; False - block CONNECTs right away
    show_debug           = models.BooleanField(default=False)   # if True DEBUG_INFO macro is expanded in the blocked page

    def __unicode__(self):
        return "Advanced for Policy %s" % self.policy




#
#
#
class Member(models.Model):
    class Meta:
        abstract  = True
        unique_together = (("policy", "value"),)
        ordering = ["value"]

    policy  = models.ForeignKey(Policy, on_delete=models.CASCADE)
    value   = models.CharField(max_length=254)
    comment = models.TextField()

    def __unicode__(self):
        return self.value

class MemberName(Member):
    pass    
class MemberIp(Member):
    pass
class MemberRange(Member):
    pass
class MemberSubnet(Member):
    pass
class MemberHeader(Member):
    pass

#
#
#
class MemberLdap(models.Model):
    class Meta:
        unique_together = (("policy", "dn"),)
        ordering = ["name"]

    policy    = models.ForeignKey(Policy, on_delete=models.CASCADE)
    name      = models.CharField(max_length=254)
    dn        = models.CharField(max_length=4096)
    recursive = models.BooleanField(default=False)
    comment   = models.TextField(blank=True)

#
#
#
class Exclusion(models.Model):

    class Meta:
        abstract  = True
        unique_together = (("policy", "value"),)
        ordering = ["value"]

    policy          = models.ForeignKey(Policy, on_delete=models.CASCADE)
    value           = models.CharField(max_length=254)
    scan_adult      = models.BooleanField(default=False)
    scan_categories = models.BooleanField(default=False)
    scan_file       = models.BooleanField(default=False)
    scan_adblock    = models.BooleanField(default=False)
    scan_privacy    = models.BooleanField(default=False)
    scan_http       = models.BooleanField(default=False)
    comment         = models.TextField()

class ExclusionDomain(Exclusion):
    pass
class ExclusionIp(Exclusion):
    pass
class ExclusionRange(Exclusion):
    pass
class ExclusionSubnet(Exclusion):
    pass
class ExclusionUrl(Exclusion):
    pass
class ExclusionContentType(Exclusion):
    pass

#
#
#
class RuleAnnoyances(models.Model):
    policy                   = models.OneToOneField(Policy, primary_key=True, on_delete=models.CASCADE)
    block_ads                = models.BooleanField(default=True)
    protect_privacy          = models.BooleanField(default=False)

#
#
#
class RuleAdult(models.Model):

    policy                   = models.OneToOneField(Policy, primary_key=True, on_delete=models.CASCADE)
    trust_allowed_categories = models.BooleanField(default=True)
    enable_iwf               = models.BooleanField(default=True)   # iwf - it is possible to enable/disable in web safety 7.3+
    enable_heuristics        = models.BooleanField(default=False)
    heuristics_maximum_weight= models.IntegerField(default=40)
    enable_image_filtering   = models.BooleanField(default=False)
    
    # phrases
    enable_phrases           = models.BooleanField(default=True)
    phrases_maximum_weight   = models.IntegerField(default=80)
    phrases_maximum_size     = models.IntegerField(default=4194304) # 4MB - oh YouTube!
    phrases_scan_links       = models.BooleanField(default=True)
    phrases_scan_javascript  = models.BooleanField(default=False)
    phrases_scan_css         = models.BooleanField(default=False)
    phrases_parse_links      = models.BooleanField(default=False)
        
#
#
#
class RuleApps(models.Model):

    policy             = models.OneToOneField(Policy, primary_key=True, on_delete=models.CASCADE)
    enable_google_apps = models.BooleanField(default=False)
    enable_safe_search = models.BooleanField(default=True)

#
# youtube rules
#
class RuleYouTube(models.Model):

    YOUTUBE_RESTRICT_NONE     = 0
    YOUTUBE_RESTRICT_MODERATE = 1
    YOUTUBE_RESTRICT_STRICT   = 2
    YOUTUBE_RESTRICT_CHOICES  = (
        (YOUTUBE_RESTRICT_NONE,     "No restrictions"),
        (YOUTUBE_RESTRICT_MODERATE, "Moderate restrictions"),
        (YOUTUBE_RESTRICT_STRICT,   "Strict restrictions")
    )

    YOUTUBE_VIDEO_ALLOW = 0
    YOUTUBE_VIDEO_DENY  = 1
    YOUTUBE_VIDEO_CHOICES = (
        (YOUTUBE_VIDEO_ALLOW,  "Allow videos by default"),
        (YOUTUBE_VIDEO_DENY,   "Block videos by default")
    )

    enable           = models.BooleanField(default=False)                
    policy           = models.OneToOneField(Policy, primary_key=True, on_delete=models.CASCADE)
    action           = models.IntegerField(choices=YOUTUBE_VIDEO_CHOICES,default=YOUTUBE_VIDEO_ALLOW)
    restrictions     = models.IntegerField(choices=YOUTUBE_RESTRICT_CHOICES,default=YOUTUBE_RESTRICT_MODERATE)
    hide_comments    = models.BooleanField(default=False)
    hide_suggestions = models.BooleanField(default=False)
    hide_other       = models.BooleanField(default=False)


class RuleYouTubeVideo(models.Model):

    class Meta:        
        abstract        = True
        ordering        = ["video_id"] 
        unique_together = ('policy', 'video_id')


    policy      = models.ForeignKey(Policy, on_delete=models.CASCADE)
    video_id    = models.CharField(max_length=200)
    channel_id  = models.CharField(max_length=200, blank=True)
    category_id = models.CharField(max_length=200, blank=True)
    title       = models.CharField(max_length=200)
    description = models.TextField(blank=True)

class RuleYouTubeAllowVideo(RuleYouTubeVideo):
    pass

class RuleYouTubeBlockVideo(RuleYouTubeVideo):
    pass

class RuleYouTubeChannel(models.Model):

    class Meta:        
        abstract        = True
        ordering        = ["channel_id"] 
        unique_together = ('policy', 'channel_id')

    policy      = models.ForeignKey(Policy, on_delete=models.CASCADE)
    channel_id  = models.CharField(max_length=200)
    title       = models.CharField(max_length=200)
    description = models.TextField(blank=True)

class RuleYouTubeAllowChannel(RuleYouTubeChannel):
    pass

class RuleYouTubeBlockChannel(RuleYouTubeChannel):
    pass

class RuleYouTubeCategory(models.Model):

    policy                        = models.OneToOneField(Policy, primary_key=True, on_delete=models.CASCADE)
    block_action_adventure        = models.BooleanField(default=False)    
    block_anime_animation         = models.BooleanField(default=False)    
    block_autos_and_vehicles      = models.BooleanField(default=False)    
    block_classics                = models.BooleanField(default=False)    
    block_comedy                  = models.BooleanField(default=False)    
    block_documentary             = models.BooleanField(default=False)    
    block_drama                   = models.BooleanField(default=False)    
    block_education               = models.BooleanField(default=False)    
    block_entertainment           = models.BooleanField(default=False)    
    block_family                  = models.BooleanField(default=False)    
    block_film_and_animation      = models.BooleanField(default=False)    
    block_foreign                 = models.BooleanField(default=False)    
    block_gaming                  = models.BooleanField(default=False)    
    block_horror                  = models.BooleanField(default=False)    
    block_howto_and_style         = models.BooleanField(default=False)    
    block_movies                  = models.BooleanField(default=False)    
    block_music                   = models.BooleanField(default=False)    
    block_news_and_politics       = models.BooleanField(default=False)    
    block_nonprofits_and_activism = models.BooleanField(default=False)    
    block_people_and_blogs        = models.BooleanField(default=False)    
    block_pets_and_animals        = models.BooleanField(default=False)    
    block_sci_fi_fantasy          = models.BooleanField(default=False)    
    block_science_and_technology  = models.BooleanField(default=False)    
    block_short_movies            = models.BooleanField(default=False)    
    block_shorts                  = models.BooleanField(default=False)    
    block_shows                   = models.BooleanField(default=False)    
    block_sports                  = models.BooleanField(default=False)    
    block_thriller                = models.BooleanField(default=False)    
    block_trailers                = models.BooleanField(default=False)
    block_travel_and_events       = models.BooleanField(default=False)    
    block_videoblogging           = models.BooleanField(default=False)    
    

#
#
#
class RuleCategoryCustom(models.Model):

    class Meta:
        unique_together = ('policy', 'category')

    policy   = models.ForeignKey(Policy, on_delete=models.CASCADE)
    category = models.CharField(max_length=254)         # we cannot use foregn key to custom categories as they are rebuilt every time
    enable   = models.BooleanField(default=False)       # should we block the category or not

class RuleCategory(models.Model):

    policy                             = models.OneToOneField(Policy, primary_key=True, on_delete=models.CASCADE)
    block_noncategorized_domains       = models.BooleanField(default=False)  # complete block all non categorized (unknown) domains; used to create safe
                                                                             # browsing environments where only site the expert looked at are allowed
    block_ctiru                        = models.BooleanField(default=False)  # block ctiru URL list
    
    block_adult_themes_sexuality       = models.BooleanField(default=False)
    block_advertising                  = models.BooleanField(default=False)
    block_alcohol_tobacco              = models.BooleanField(default=False)
    block_anime_manga_webcomic         = models.BooleanField(default=False)
    block_arts_society_culture         = models.BooleanField(default=False)
    block_automotive                   = models.BooleanField(default=False)
    block_banking_insurance_finance    = models.BooleanField(default=False)
    block_blogs_personal               = models.BooleanField(default=False)
    block_business_services_industry   = models.BooleanField(default=False)
    block_classifieds_auctions         = models.BooleanField(default=False)
    block_content_delivery_networks    = models.BooleanField(default=False)
    block_cracks_hacking_illegal       = models.BooleanField(default=True)
    block_dating                       = models.BooleanField(default=False)
    block_drugs                        = models.BooleanField(default=False)
    block_ecommerce_shopping           = models.BooleanField(default=False)
    block_education_science_research   = models.BooleanField(default=False)
    block_entertainment_humor_hobby    = models.BooleanField(default=False)
    block_expired_parked               = models.BooleanField(default=False)
    block_fashion_beauty_cosmetics     = models.BooleanField(default=False)
    block_file_storage                 = models.BooleanField(default=False)
    block_food_dining_restaurants      = models.BooleanField(default=False)
    block_forums_message_boards        = models.BooleanField(default=False)
    block_gambling                     = models.BooleanField(default=False)
    block_games                        = models.BooleanField(default=False)
    block_generic_non_categorized      = models.BooleanField(default=False)
    block_government                   = models.BooleanField(default=False)
    block_hate_discrimination_violence = models.BooleanField(default=True)
    block_health_medicine_fitness      = models.BooleanField(default=False)
    block_jobs_employment_career       = models.BooleanField(default=False)
    block_messaging_chat               = models.BooleanField(default=False)
    block_movies                       = models.BooleanField(default=False)
    block_music_radio                  = models.BooleanField(default=False)
    block_network_infrastructure       = models.BooleanField(default=False)
    block_news_media                   = models.BooleanField(default=False)
    block_nudity_pornography           = models.BooleanField(default=True)
    block_p2p_file_sharing             = models.BooleanField(default=False)
    block_photo_sharing                = models.BooleanField(default=False)
    block_politics                     = models.BooleanField(default=False)
    block_portals                      = models.BooleanField(default=False)
    block_proxy_anonymizer             = models.BooleanField(default=True)
    block_real_estate_home_interior    = models.BooleanField(default=False)
    block_religious                    = models.BooleanField(default=False)
    block_search_engines               = models.BooleanField(default=False)
    block_self_harm                    = models.BooleanField(default=True)
    block_social_networking            = models.BooleanField(default=False)
    block_software_technology_hardware = models.BooleanField(default=False)
    block_sports                       = models.BooleanField(default=False)
    block_television                   = models.BooleanField(default=False)
    block_travel                       = models.BooleanField(default=False)
    block_user_tracking                = models.BooleanField(default=False)
    block_video_sharing                = models.BooleanField(default=False)
    block_weapons                      = models.BooleanField(default=False)
    block_webmail                      = models.BooleanField(default=False)

    def categories_as_array(self):

        array = []
        for name in BuildInCategories.CATEGORY_NAMES:

            attrname = "block_%s" % name
            assigned = getattr(self, attrname)
            
            if assigned:
                array.append(name)

        return array

    def categories_str(self):

        values = self.categories_as_array()
        return ", ".join(values)

#
#
#
class RuleDynamicCategory(models.Model):

    policy           = models.OneToOneField(Policy, primary_key=True, on_delete=models.CASCADE)
    enabled          = models.BooleanField(default=False)
    analyze_request  = models.BooleanField(default=True)
    analyze_response = models.BooleanField(default=True)

    CLASSIFY_ONLY_UNKNOWN_SITES    = 0
    CLASSIFY_UNKNOWN_AND_UNTRUSTED = 1
    CLASSIFY_ALL_SITES             = 2
    CLASSIFY_CHOICES  = (
        (CLASSIFY_ONLY_UNKNOWN_SITES, "only unknown sites"),
        (CLASSIFY_UNKNOWN_AND_UNTRUSTED, "unknown and untrusted sites"),
        (CLASSIFY_ALL_SITES, "all sites")
    )

    classify_type = models.IntegerField(choices=CLASSIFY_CHOICES,default=CLASSIFY_UNKNOWN_AND_UNTRUSTED)

    # we have all the categories here in the model but actually we have much less classifiers 
    classify_adult_themes_sexuality       = models.BooleanField(default=True)
    classify_advertising                  = models.BooleanField(default=False)
    classify_alcohol_tobacco              = models.BooleanField(default=True)
    classify_anime_manga_webcomic         = models.BooleanField(default=False)
    classify_arts_society_culture         = models.BooleanField(default=False)
    classify_automotive                   = models.BooleanField(default=False)
    classify_banking_insurance_finance    = models.BooleanField(default=False)
    classify_blogs_personal               = models.BooleanField(default=False)
    classify_business_services_industry   = models.BooleanField(default=False)
    classify_classifieds_auctions         = models.BooleanField(default=False)
    classify_content_delivery_networks    = models.BooleanField(default=False)
    classify_cracks_hacking_illegal       = models.BooleanField(default=False)
    classify_dating                       = models.BooleanField(default=True)
    classify_drugs                        = models.BooleanField(default=True)
    classify_ecommerce_shopping           = models.BooleanField(default=False)
    classify_education_science_research   = models.BooleanField(default=False)
    classify_entertainment_humor_hobby    = models.BooleanField(default=False)
    classify_expired_parked               = models.BooleanField(default=False)
    classify_fashion_beauty_cosmetics     = models.BooleanField(default=False)
    classify_file_storage                 = models.BooleanField(default=False)
    classify_food_dining_restaurants      = models.BooleanField(default=False)
    classify_forums_message_boards        = models.BooleanField(default=False)
    classify_gambling                     = models.BooleanField(default=True)
    classify_games                        = models.BooleanField(default=True)
    classify_generic_non_categorized      = models.BooleanField(default=False)
    classify_government                   = models.BooleanField(default=False)
    classify_hate_discrimination_violence = models.BooleanField(default=True)
    classify_health_medicine_fitness      = models.BooleanField(default=False)
    classify_jobs_employment_career       = models.BooleanField(default=False)
    classify_messaging_chat               = models.BooleanField(default=False)
    classify_movies                       = models.BooleanField(default=False)
    classify_music_radio                  = models.BooleanField(default=False)
    classify_network_infrastructure       = models.BooleanField(default=False)
    classify_news_media                   = models.BooleanField(default=False)
    classify_nudity_pornography           = models.BooleanField(default=True)
    classify_p2p_file_sharing             = models.BooleanField(default=False)
    classify_photo_sharing                = models.BooleanField(default=False)
    classify_politics                     = models.BooleanField(default=False)
    classify_portals                      = models.BooleanField(default=False)
    classify_proxy_anonymizer             = models.BooleanField(default=False)
    classify_real_estate_home_interior    = models.BooleanField(default=False)
    classify_religious                    = models.BooleanField(default=False)
    classify_search_engines               = models.BooleanField(default=False)
    classify_self_harm                    = models.BooleanField(default=True)
    classify_social_networking            = models.BooleanField(default=False)
    classify_software_technology_hardware = models.BooleanField(default=False)
    classify_sports                       = models.BooleanField(default=True)
    classify_television                   = models.BooleanField(default=False)
    classify_travel                       = models.BooleanField(default=False)
    classify_user_tracking                = models.BooleanField(default=False)
    classify_video_sharing                = models.BooleanField(default=False)
    classify_weapons                      = models.BooleanField(default=True)
    classify_webmail                      = models.BooleanField(default=False)

    def categories_as_array(self):

        array = []
        for name in BuildInCategories.CATEGORY_NAMES:

            attrname = "classify_%s" % name
            assigned = getattr(self, attrname)
            
            if assigned:
                array.append(name)

        return array

    def categories_str(self):

        values = self.categories_as_array()
        return ", ".join(values)

    

#
#
#
class RuleValue(models.Model):

    class Meta:
        abstract = True
        unique_together = ('policy', 'value')

    policy  = models.ForeignKey(Policy, on_delete=models.CASCADE)
    value   = models.CharField(max_length=254)
    comment = models.TextField()

class RuleDomain(RuleValue):
    pass
class RuleUrl(RuleValue):
    pass
class RuleUserAgent(RuleValue):
    pass
class RuleContentType(RuleValue):
    pass
class RuleCharset(RuleValue):
    pass
class RuleFileName(RuleValue):
    pass
class RuleFileContent(RuleValue):
    pass

class RuleFileSize(models.Model):
    policy   = models.OneToOneField(Policy, primary_key=True, on_delete=models.CASCADE)
    enable   = models.BooleanField(default=False)
    max_size = models.IntegerField(default=1048576) # 1MB
