#
#
#
from django import forms

#
#
#
from utility.validator import AddressIP, AddressSubnet, AddressRange, AsciiOnly, NoWildcards, NotUrl, Strip

#
#
#
class ItemForm(forms.ModelForm):

    def clean_value(self):
        return Strip.clean(self.cleaned_data['value'])

    def clean_comment(self):
        return AsciiOnly.clean(self.cleaned_data['comment'])
        

#
#
#
class DomainNameForm(ItemForm):

    def clean_value(self):
        
        value = Strip.clean(self.cleaned_data['value'])
        if len(value) < 2:
            raise forms.ValidationError("Domain '%s' cannot be added, becase it is too short." % (value) )

        value = AsciiOnly.clean(value)
        value = NoWildcards.clean(value)
        value = NotUrl.clean(value)
        
        return value

#
#
#
class IpForm(ItemForm):

    def clean_value(self):      

        value = Strip.clean(self.cleaned_data['value'])
        value = AddressIP().check(value)

        return value

#
#
#
class SubnetForm(ItemForm):

    def clean_value(self):

        value = Strip.clean(self.cleaned_data['value'])
        value = AddressSubnet().check(value)

        return value

#
#
#
class RangeForm(ItemForm):

    def clean_value(self):

        value = Strip.clean(self.cleaned_data['value'])
        value = AddressRange().check(value)

        return value

#
#
#
class AdvancedForm(forms.ModelForm):

    def clean_value(self):
        return Strip.clean(self.cleaned_data['value'])

    def clean_comment(self):
        return AsciiOnly.clean(self.cleaned_data['comment'])
