#
#
#
import os
import traceback

#
#
#
from django import forms
from django.views import generic
from django.conf  import settings
from django.http  import HttpResponse
from django.shortcuts import render
from django.contrib import messages
from django.http import HttpResponseRedirect
from django.urls import reverse


#
# squid tools form
#
IMPORT_FILE_CHOICES = (

    # exclusions
    ('ExcludeDomainName',   'Exclude Domain Name'),
    ('ExcludeDomainIp',     'Exclude Domain IP'),
    ('ExcludeDomainSubnet', 'Exclude Domain Subnet'),
    ('ExcludeDomainRange',  'Exclude Domain Range'),    
    ('ExcludeUserIp',       'Exclude User Ip'),    
    ('ExcludeUserSubnet',   'Exclude User Subnet'),
    ('ExcludeUserRange',    'Exclude User Range'),
    ('ExcludeUserAgent',    'Exclude User Agent'),
    ('ExcludeContentType',  'Exclude Content Type'),    
    ('SslTargetDomain',     'Ssl Target Domain'),
    ('SslTargetIp',         'Ssl Target IP Address'),
    ('SslTargetSubnet',     'Ssl Target IP Subnet'),
    ('SslErrorDomain',      'Ssl Error Domain'),
    ('SslErrorIp',          'Ssl Error IP Address'),
    ('SslErrorSubnet',      'Ssl Error IP Subnet'),
)



#
#
#
class UploadFileForm(forms.Form):

    file    = forms.FileField()
    type    = forms.ChoiceField(choices=IMPORT_FILE_CHOICES)
    comment = forms.CharField(max_length=512, widget=forms.Textarea(
        attrs={'size':'90', 'rows':'3', 'class':'input-block-level'}),
        required=False
    )


#
# squid tools
#    
class ToolsUploadSquidFile(generic.View):

    def get(self, request, *args, **kwargs):
        return render(request, 'squid/tools/upload.html', {'form': UploadFileForm() })

    def collect(self, data):

        collected = ""
        for chunk in data.chunks():
            collected += chunk.decode("utf-8")

        result = []
        array  = collected.split('\n')
        for line in array:
            line = line.strip('\n')
            line = line.strip('\r')
            line = line.strip(' ')

            if len(line) > 0:
                result.append(line)    

        return result

    def drop_comments(self, items):

        result = []
        for item in items:
            pos = item.find("#")
            if pos != -1:
                item = item[:pos].strip()
            if len(item) > 0:
                result.append(item)            
        return result

    def import_file(self, model_name, array, comment):

        from django.apps import apps

        model = apps.get_model("squid.%s" % model_name)
        query = model.objects

        for item in array:
            found = query.filter(value__exact=item)
            if len(found) > 0:
                continue
            
            new_entry = model(value=item)
            try:
                new_entry.comment = comment
            except Exception as e:
                pass

            new_entry.save()

    def post(self, request, *args, **kwargs):

        # get the form
        form = UploadFileForm(request.POST, request.FILES)
        if not form.is_valid():
            return render(request, 'squid/tools/upload.html', {'form': form })

        # try to import and redisplay if failed
        try:

            # process data
            array = []
            if 'file' in self.request.FILES:
                array = self.collect(self.request.FILES['file'])

            # drop all comments
            array = self.drop_comments(array)

            # get comment in form
            comment = form.cleaned_data.get('comment', '')

            # see if we need to import squid or policy
            self.import_file(form.cleaned_data['type'], array, comment)
            
        except Exception as e:
            return render(request, 'squid/tools/upload.html', {'form': form, 'error': str(e), 'stack': traceback.format_exc() })

        # mark as needing reload
        messages.info(self.request, "need_squid_restart")

        # and redirect
        return HttpResponseRedirect(reverse("squid:ToolsUploadSquidFile"))
