#!/usr/bin/env python
#
# (c) Diladele B.V. 2018, Web Safety for Squid Proxy
#  
# Sample automation script
#
import os
import sys
import django
import logging
import argparse

#
# in order for Django to work correctly with our monitor app, we add ourselves
# to Python's search path
sys.path.append(os.path.abspath(__file__))

#
# tell Django where to read settings from
#
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "console.settings")

#
#
#
django.setup()

#
# do imports
#
from _domain.traffic import *
from _domain.squid   import *
from squid.models    import AuthAd
from traffic.views   import *
    

def run(args):

    keytab_path = "/opt/websafety/etc/krb5.keytab"
    keytab_spn  = "HTTP/proxy.diladele.lan@DILADELE.LAN"

    print ("Dumping krb5.keytab...")
    dumper = KeyTabDumper()
    data   = dumper.dump(keytab_path)
    print(data)

    print ("Trying to initialize kerberos using krb5.keytab...")

    initializer = KeyTabInitializer()
    (result, output) = initializer.initialize(keytab_path, keytab_spn)
    if result:
        print("SUCCESS:")
        print(output)
    else:
        print("FAILURE:")
        print(output)

    print("Trying to check kerberos version:")

    creds    = LdapCreds("DC=diladele,DC=lan", "squid@diladele.lan", "Passw0rd")
    endpoint = LdapEndpoint("dc1.diladele.lan", AuthAd.LOOKUP_MODE_LDAP, False, creds)
    checker  = KerberosChecker(endpoint)

    (s1, u1, v1, v2) = checker.check(keytab_spn, keytab_path)
    print(s1)
    print(u1)
    print(v1)
    print(v2)

#
# main
#
def main():
    
    # define arguments
    parser = argparse.ArgumentParser(description='Test playground for Web Safety')
    #parser.add_argument("--domain", help="name of the domain to exclude from HTTPS decryption", required=True)

    # parse them
    args = parser.parse_args()    

    logging.basicConfig(format='%(asctime)s %(message)s', level=logging.INFO)   
        
    # add domain
    run(args)

    logging.info("Test playground works. Everything is ok!")
        
if __name__ == '__main__':

    main()