import os

#
#
#
from _domain.utils import read_json_object

#
#
#
from traffic.models import Traffic, Mailer, PeriodicReport

#
#
#
class Upgrader(object):

    def __init__(self, major, minor):

        self.major   = major
        self.minor   = minor

    def upgrade(self, etc_dir, config_json):

        # note reports were completely re-implemented in 7.2 so we do NOT support upgrading of reports from earlier than that
        if self.major > 7 or (self.major == 7 and self.minor >= 2):
            self.upgrade_traffic(config_json)
            self.upgrade_reports(config_json)
            self.upgrade_mailer(config_json)
        else:
            self.upgrade_monitor(etc_dir)

    def upgrade_traffic(self, config_json):

        for item in config_json:

            if "traffic.traffic" != item.get("model", ""):
                continue

            # get name value
            fields = item.get("fields", {})

            # try to store in the database
            try:
                obj = Traffic.objects.first()

                obj.realtime_regeneration_interval           = fields.get("realtime_regeneration_interval", 60)
                obj.realtime_last_line_count                 = fields.get("realtime_last_line_count", 1000)

                obj.collector_enable                         = fields.get("collector_enable", True)
                obj.collector_ignore_status_non_aborted      = fields.get("collector_ignore_status_non_aborted", True)
                obj.collector_ignore_status_407              = fields.get("collector_ignore_status_407", True)
                obj.collector_ignore_error_invalid_request   = fields.get("collector_ignore_error_invalid_request", True)
                obj.collector_ignore_error_transaction_end   = fields.get("collector_ignore_error_transaction_end", True)
                obj.collector_ignore_module_ads              = fields.get("collector_ignore_module_ads", False)
                obj.collector_ignore_module_privacy          = fields.get("collector_ignore_module_privacy", False)
                obj.collector_ignore_module_sslbump          = fields.get("collector_ignore_module_sslbump", False)
                obj.collector_ignore_module_clean            = fields.get("collector_ignore_module_clean", False)
                obj.collector_ignore_module_adult_heuristics = fields.get("collector_ignore_module_adult_heuristics", False)
                obj.collector_ignore_module_adult_safesearch = fields.get("collector_ignore_module_adult_safesearch", False)
                obj.collector_ignore_module_adult_youtube    = fields.get("collector_ignore_module_adult_youtube", False)
                obj.collector_ignore_module_adult_phrases    = fields.get("collector_ignore_module_adult_phrases", False)
                obj.collector_ignore_module_adult_image      = fields.get("collector_ignore_module_adult_image", False)
                obj.collector_ignore_module_apps             = fields.get("collector_ignore_module_apps", False)
                obj.collector_folders                        = fields.get("collector_folders", "")
                obj.collector_cluster                        = fields.get("collector_cluster", True)

                obj.reporter_max_detailed_users              = fields.get("reporter_max_detailed_users", 50)
                obj.reporter_max_detailed_domains            = fields.get("reporter_max_detailed_domains", 100)
                
                obj.save()

            except Exception as e:
                pass

            # as there is only one traffic object we can return now
            return 

    def upgrade_reports(self, config_json):

        for item in config_json:

            if "traffic.periodicreport" != item.get("model", ""):
                continue

            # get name value
            fields = item.get("fields", {})

            # try to store in the database
            try:
                obj, c = PeriodicReport.objects.get_or_create(name=fields.get("name"))
                obj.comments       = fields.get("comments", "")
                obj.enabled        = fields.get("enabled", False)
                obj.rep_type       = fields.get("rep_type", "top_sites_by_size")
                obj.schedule_min   = fields.get("schedule_min", "*")
                obj.schedule_hour  = fields.get("schedule_hour", "*")
                obj.schedule_dom   = fields.get("schedule_dom", "*")
                obj.schedule_month = fields.get("schedule_month", "*")
                obj.schedule_dow   = fields.get("schedule_dow", "*")

                obj.timeframe_type = fields.get("timeframe_type", "last_n_days")
                obj.timeframe_value = fields.get("timeframe_value", 7)
                obj.timeframe_from = fields.get("timeframe_from", "")
                obj.timeframe_to = fields.get("timeframe_to", "")
                obj.include_current = fields.get("include_current", True)
                obj.include_domains = fields.get("include_domains", "")
                obj.include_users = fields.get("include_users", "")
                obj.include_policies = fields.get("include_policies", "")
                obj.exclude_domains = fields.get("exclude_domains", "")
                obj.exclude_users = fields.get("exclude_users", "")
                obj.exclude_policies = fields.get("exclude_policies", "")
                obj.limit_n_entries = fields.get("limit_n_entries", 50)
                obj.limit_n_drilldown = fields.get("limit_n_drilldown", 50)
                obj.send_to_emails = fields.get("send_to_emails", "")
                obj.save()

            except Exception as e:
                pass

    def upgrade_mailer(self, config_json):

        for item in config_json:

            if "traffic.mailer" != item.get("model", ""):
                continue

            # get fields dict
            fields = item.get("fields", {})

            obj  = Mailer.objects.first()
            obj.smtp_username   = fields.get("smtp_username", "")
            obj.smtp_password   = fields.get("smtp_password", "")
            obj.smtp_server     = fields.get("smtp_server", "")
            obj.smtp_use_auth   = fields.get("smtp_use_auth", True)
            obj.smtp_port       = fields.get("smtp_port", 587)
            obj.smtp_test_email = fields.get("smtp_test_email", "")
            obj.save()

    def upgrade_monitor(self, etc_dir):

        # first try newer path
        path = os.path.join(etc_dir, "traffic", "monitor.json")
        if not os.path.exists(path):
            path = os.path.join(etc_dir, "monitor.json")

        # get json and upgrade
        data = read_json_object(path)

        # get the object
        o = Traffic.objects.first()

        if 'database' in data:

            o.collector_ignore_module_ads              = not data['database']['store_adblock']
            o.collector_ignore_module_privacy          = not data['database']['store_privacy']
            o.collector_ignore_module_sslbump          = not data['database']['store_sslbump']
            o.collector_ignore_module_clean            = not data['database']['store_clean']
            o.collector_ignore_module_adult_heuristics = not data['database']['store_adult_heuristics']
            o.collector_ignore_module_adult_safesearch = not data['database']['store_adult_safesearch']
            o.collector_ignore_module_adult_youtube    = not data['database']['store_adult_youtube']
            o.collector_ignore_module_adult_phrases    = not data['database']['store_adult_phrases']
            o.collector_ignore_module_adult_image      = not data['database']['store_adult_image']
            o.collector_ignore_module_apps             = not data['database']['store_apps']
        
        # and save
        o.save()
