import os
import traceback

#
#
#
from django.views import generic
from django.core.paginator import Paginator

#
#
#
from daemon.core            import DaemonPath, UiPath, SquidPath
from daemon.stats.collector import NowCollector, Config as CollectorConfig

#
#
#
from traffic.models import Traffic

#
#
#
class ViewActivityNow(generic.ListView):

    template_name = "traffic/reporter/now/activity.html"
    paginate_by   = 50

    def get_context_data(self, **kwargs):

        context = super(ViewActivityNow, self).get_context_data(**kwargs)
        try:

            # get include and exclude strings from request
            include, exclude = self.get_filters()

            # save them into context
            context['include'] = include
            context['exclude'] = exclude

        except Exception as e:

            context['error_text'] = str(e)
            context['traceback']  = str(traceback.format_exc())

        return context

    def get_queryset(self):

        # create collector config
        c1 = CollectorConfig()

        # read entries
        c = NowCollector(c1)
        e = c.read_entries(
            Traffic.objects.first().realtime_last_line_count
        )

        # and do the filtering
        return self.filter_entries(e)

    def get_filters(self):

        include = self.request.GET.get('include', '').strip().lower()
        exclude = self.request.GET.get('exclude', '').strip().lower()

        return (include, exclude)

    def filter_entries(self, entries):

        # get include and exclude strings from request
        include, exclude = self.get_filters()

        # quick exit
        if not include and not exclude:
            return entries

        # ok stupidly
        result = []
        for entry in entries:

            # get textual entry contents
            contents = entry.contents().lower()

            # see if we have to include
            if len(include) > 0:

                if include not in contents:
                    continue # the text to include was not found, this entry is not intersting

            # see if we have to exclude
            if len(exclude) > 0:

                if exclude in contents:
                    continue # the text to exclude was found, this entry is not interesting

            # if we got here this entry is interesting
            result.append(entry)

        # and return
        return result