import os
import traceback

#
# django
#
from django.views import generic

#
# local
#
from .mixins  import *
from .loaders import DomainLoader

#
#
#
class ViewDomainTimeOfDay(ReporterMixin, generic.TemplateView):

    def get_context_data(self, **kwargs):

        # get the context
        context = super(ViewDomainTimeOfDay, self).get_context_data(**kwargs)

        try:

            # the domain we are displaying
            entry = self.kwargs['domain']

            # create loader for a given entry
            l = DomainLoader(self.get_period(), entry)

            # and fill the context
            context['hourly_requests_per_user']  = l.hourly_requests_per_user()
            context['hourly_bandwidth_per_user'] = l.hourly_bandwidth_per_user()
            context['hourly_uploads_per_user']   = l.hourly_uploads_per_user()
             
        except Exception as e:

            context['error_text'] = str(e)
            context['traceback']  = str(traceback.format_exc())

        return context
