import os
import traceback

#
# django
#
from django.views import generic

#
# domain
#
# from _domain.traffic.report import DomainsReporter, RequestType

#
# local
#
from .mixins import ReporterMixin
from .loaders import DomainsLoader


#
# generic class view all requestors (labels/macs/ips)
#
class ViewDomains(ReporterMixin, generic.TemplateView):

    def get_context_data(self, **kwargs):

        # get the context
        context = super(ViewDomains, self).get_context_data(**kwargs)

        try:

            # create reporter
            r = DomainsLoader(self.get_period())

            # and fill the context
            context['object_list'] = r.list()
            context['totals']      = r.totals()
            
        except Exception as e:

            context['error_text'] = str(e)
            context['traceback']  = str(traceback.format_exc())

        return context
