import os
import subprocess

#
#
#
from urllib.parse import quote, urlencode

#
#
#
from daemon.core import System, UiPath, SquidPath

#
# local
#
from .loaders import *

#
# helper functions
#
def reports_dir():
    return os.path.join(UiPath.var_dir(), "reports")

def stats_dir():
    return os.path.join(UiPath.var_dir(), "stats")

def logs_dir():
    return SquidPath.log_dir()

#
#
#
def regenerate_now(force=False):

    # we run manually on windows only, because starting from 7.4 we have added systemd realtime daemon
    if System.name() != System.WS_WINDOWS:
        return

    # ok this is windows, just call the realtime.py manually once
    subprocess.call(["python", os.path.join(UiPath.bin_dir(), "realtime.py"), "--once"])

#
# common functionality as mixin - useful for all reporter views
#
class ReporterMixin:

    def get_period(self):

        # get the period
        period = self.kwargs['period']

        # and return it with constructed prefix
        if period == "now":

            # it is a hack!
            regenerate_now()

            # ok then return
            return (period, "now")

        elif period == "day":
            return (period, "%04d-%02d-%02d" % (self.kwargs['year'], self.kwargs['month'], self.kwargs['day']))
        elif period == "month":
            return (period, "%04d-%02d" % (self.kwargs['year'], self.kwargs['month']))
        elif period == "year":
            return (period, "%04d" % self.kwargs['year'])
        else:
            raise Exception("Unknown period")

    def get_request_type(self):

        request_types = {
            "all"       : RequestType.All,
            "blocked"   : RequestType.Blocked,
            "offensive" : RequestType.Offensive,
        }

        return request_types[self.kwargs['show']]

#
# common functionality for requestors classes (labels, ips, domains)
#
class RequestorMixin:

    def get_loader_class(self):

        loaders = {
            "user_id" : UserLoader
        }

        return loaders [
            self.kwargs['requestor_type']
        ]
