import os
import traceback

#
# django
#
from django.views import generic

#
# domain
#
from daemon.stats.collector import History
from daemon.stats.reporter  import is_valid_report_dir

#
# local
#
from .mixins import reports_dir, stats_dir

#
#
#
class ViewMonths(generic.TemplateView):

    def get_context_data(self, **kwargs):

        # get the context
        context = super(ViewMonths, self).get_context_data(**kwargs)

        try:

            # create history 
            history = History(stats_dir())

            # create data for context
            entries = {}

            # and fill it
            for year in history.years():

                # get the months of the given year
                months = history.months(year)

                # loop over all months
                for month in months:

                    # construct path to this month report dir
                    prefix = "%04d-%02d" % (year, month)

                    # see if report is available
                    if not is_valid_report_dir(os.path.join(reports_dir(), prefix)):
                        continue

                    # ok report is available, fill the entries dict
                    if year not in entries:                    
                        entries[year] = {}

                    if month not in entries[year]:
                        entries[year][month] = 0

                    # and update the active day count
                    entries[year][month] = {
                        "days"   : len(history.days(year, month)),
                        "period" : "%04d-%02d-00" % (year, month)
                    }
                    

            # and save the context
            context['entries'] = entries

        except Exception as e:

            context['error_text'] = str(e)
            context['traceback']  = str(traceback.format_exc())
        
        return context
