import os
#
#
#
from django.urls      import reverse_lazy
from django.http      import HttpResponseRedirect, HttpResponse
from django.views     import generic
from django.shortcuts import get_object_or_404
from django.contrib   import messages
from django.contrib.messages.views import SuccessMessageMixin

#
#
#
from daemon.core    import UiPath
from traffic.models import PeriodicReport
from traffic.forms  import PeriodicReportForm

#
# jobs
#
class ViewPeriodicReports(SuccessMessageMixin, generic.ListView):

    template_name = 'traffic/periodic/reports_list.html'
    model         = PeriodicReport

    def get_queryset(self):
        return PeriodicReport.objects.order_by('name')

    def post(self, request, *args, **kwargs):

        # drop all items of that model by primary key
        for selected_id in request.POST.getlist('selected_id'):
            item = get_object_or_404(PeriodicReport, pk = selected_id)
            item.remove_generated_dir()
            item.delete()

        # tell the user squid needs to be reloaded or restarted
        messages.info(self.request, "need_squid_restart")

        # and redirect to success url
        return HttpResponseRedirect(reverse_lazy('traffic:ViewPeriodicReports'))

    #def get_context_data(self, **kwargs):
    #    context  = super(ViewPeriodicReports, self).get_context_data(**kwargs)
    #    return context


#
#
#
class ViewPeriodicReportCreate(SuccessMessageMixin, generic.edit.CreateView):

    model           = PeriodicReport
    form_class      = PeriodicReportForm
    success_message = "need_squid_restart"
    template_name   = 'traffic/periodic/report_form.html'

    '''
    def get_context_data(self, **kwargs):
        context = super(ViewPeriodicReportCreate, self).get_context_data(**kwargs)
        context['type']        = self.kwargs['template']

        # various flags
        #context['show_limits'] = self.show_limits(self.kwargs['template'])
        #context['show_limits_domains'] = self.show_limits_domains(self.kwargs['template'])
        return context

    
    def show_limits(self, template_name):
        if self.show_limits_domains(template_name):
            return True
        return False

    def show_limits_domains(self, template_name):

        if template_name in [
            "icap-domains-detailed",
            "icap-users-detailed-by-domains",
            "icap-users-detailed-unproductive-by-domains",
            "icap-users-detailed-blocked-by-domains",
            "icap-dates-detailed-by-bandwidth",
            "icap-hours-detailed-by-bandwidth",
            "icap-categories-detailed"
        ]:
            return True
        return False
    '''    

    def get_success_url(self):
        return reverse_lazy('traffic:ViewPeriodicReports')

#
#
#
class ViewPeriodicReportUpdate(SuccessMessageMixin, generic.edit.UpdateView):

    model           = PeriodicReport
    form_class      = PeriodicReportForm
    success_message = "need_squid_restart"
    template_name   = 'traffic/periodic/report_form.html'

    def get_context_data(self, **kwargs):
        context = super(ViewPeriodicReportUpdate, self).get_context_data(**kwargs)
        #context['reporter'] = Reporter.objects.first()
        #context['type']     = self.kwargs['template']
        return context

    def get_success_url(self):
        return reverse_lazy('traffic:ViewPeriodicReports')


#
#
#
class ViewPeriodicReportView(generic.View):

    def get(self, request, *args, **kwargs):

        
        item = get_object_or_404(PeriodicReport, pk=self.kwargs["pk"])
        file = self.kwargs["file"]

        fullname = os.path.join(UiPath.var_dir(), "reports", "periodic", item.name, file)
        data     = None
        try:
            with open (fullname, "r", encoding="utf-8") as fin:
                data = fin.read()
        except Exception as e:
            raise

        mimetype = "text/html"
        if file.endswith(".json"):
            mimetype = "application/json"
        elif file.endswith(".css"):
            mimetype = "text/css"
        elif file.endswith(".log"):
            mimetype = "text/plain"

        return HttpResponse(data, content_type=mimetype)

#
#
#
class ViewPeriodicLog(generic.View):

    def get(self, request, *args, **kwargs):

        
        item = get_object_or_404(PeriodicReport, pk=self.kwargs["pk"])
        file = item.name + ".log"

        fullname = os.path.join(UiPath.var_dir(), "reports", "periodic", file)
        data     = ""
        try:
            with open (fullname, "r", encoding="utf-8") as fin:
                data = fin.read()
        except Exception as e:
            raise

        return HttpResponse(data, content_type="text/plain")