import os
import traceback

#
# django
#
from django.views import generic
from django.http  import HttpResponse
from django.core.paginator import Paginator

#
# domain
#
from daemon.safety import BuildInModules, BuildInCategories

#
# local
#
from .mixins  import *
from .loaders import *

#
#
#
class ViewRequestorActivity(ReporterMixin, generic.TemplateView):

    def get_current_page(self):

        # calculate what page to show
        page_n = 1

        try:
            value  = self.request.GET.get('page', 1)
            value  = int(value)            
            page_n = value
        except Exception as e:
            pass

        return page_n

    def get_contents(self, from_time, to_time, line_count, include, exclude):

        # the user_id we are displaying
        requestor = self.kwargs['requestor']
        
        # create loader for a given entry
        l = UserLoader(self.get_period(), requestor)

        # load activity for all domains
        df = l.activity_tail(from_time, to_time, line_count, exclude, include)

        # we need to reverse the dataframe so that it shows older records on top
        df = df.reindex(index=df.index[::-1])

        # and return nicely
        return df

    def get_context_data(self, **kwargs):

        context = super(ViewRequestorActivity, self).get_context_data(**kwargs)

        try:

            # get from and to
            from_time = self.request.GET.get('from', '00:00:00')
            to_time   = self.request.GET.get('to',   '23:59:59')

            # get include and exclude strings from request
            include = self.request.GET.get('include', '')
            exclude = self.request.GET.get('exclude', '')
            size    = 512
            try:
                size = int(self.request.GET.get('size', '512'))
                if size > 4096:
                    size = 4096
            except Exception as e:
                pass

            # as django paginator does not work with dataframe we must repack it
            object_list = []
            modules     = BuildInModules()
            if True:

                # load the recent activity table            
                dataframe = self.get_contents(from_time, to_time, size, include, exclude)
                for index, row in dataframe.iterrows():

                    # unmap categories
                    row["ws_categories_str"] = ", ".join(BuildInCategories.unmap_name(row["ws_categories"]))
                    row["ws_module_str"]     = modules.unmap(row["ws_module"])

                    # and append row
                    object_list.append(row)

            #sort_field = self.request.GET.get("o", "")
            #is_asc     = [True, False][self.request.GET.get("a", "-") == "-"]
            #order_by   = (
            #   sort_field, is_asc
            #)

            # construct the paginator, current page and items
            paginator    = Paginator(object_list, 100)
            current_page = self.get_current_page()
            items        = paginator.page(current_page)

            # list of pages for quick pagination
            if True:

                min_value = 1
                max_value = 10

                total_pages = paginator.num_pages
                if total_pages < 10:
                    max_value = total_pages


                min_value = max(0, current_page - 5)
                max_value = min(current_page + 5, total_pages)

            # save into context
            context['size']         = size
            context['include']      = include
            context['exclude']      = exclude
            context['from_time']    = from_time
            context['to_time']      = to_time
            context['page']         = items
            context['paginator']    = paginator
            context['pages']        = [i for i in range(min_value + 1, max_value + 1)]
            context['object_list']  = object_list
            
            #context['o']           = sort_field
            #context['a']           = ["+", "-"][is_asc == False]
            
        except Exception as e:

            context['error_text'] = str(e)
            context['traceback']  = str(traceback.format_exc())

        return context


#
#
#
class ViewRequestorActivityDownload(ReporterMixin, generic.View):

    def get(self, request, *args, **kwargs):

        # get period
        period, value  = self.get_period()
        if period not in ["day", "now"]:
            raise Exception("Wrong period %s" % period)

        # construct the path to file
        activity_csv = os.path.join(UiPath.var_dir(), "reports", value, "users", self.kwargs['requestor'], "activity.csv")
        csv_name     = "activity_%s_%s.csv" % (self.kwargs['requestor'].lower(), value) 

        # and send it
        response = None
        with open(activity_csv, "rb") as fin:
            response = HttpResponse(fin.read(), content_type="text/csv")
            response['Content-Disposition'] = "attachment; filename=\"%s\"" % csv_name

        # and return response
        return response