import os
import traceback

#
# django
#
from django.views import generic

#
# local
#
from .mixins import *

#
#
#
class ViewRequestorSummary(RequestorMixin, ReporterMixin, generic.TemplateView):

    def get_context_data(self, **kwargs):

        # get the context
        context = super(ViewRequestorSummary, self).get_context_data(**kwargs)

        try:

            # the label/ip/mac or domain we are displaying
            entry = self.kwargs['requestor']

            # create loader for a given entry
            C = self.get_loader_class()
            l = C(self.get_period(), entry)

            # and fill the context
            context['summary']  = l.summary()
            context['hits']     = l.hits()
            context['bytes']    = l.bytes()
            context['searches'] = l.searches()
            
        except Exception as e:

            context['error_text'] = str(e)
            context['traceback']  = str(traceback.format_exc())

        return context
