import os
import calendar
import collections
import traceback


#
# django
#
from django.views import generic
 
#
# domain
#
from daemon.stats.collector import History
from daemon.stats.reporter  import is_valid_report_dir
#from _domain.core import Paths

#
# local
#
from .calendar_ex import MyHTMLCalendar
from .mixins      import reports_dir, stats_dir


#
#
#
class ViewStatistics(generic.TemplateView):

    def render(self, data):
        
        # this is the rendered data
        rendered = collections.OrderedDict()

        # loop over available years        
        for year, months in data.items():

            # add this year to rendered
            rendered[year] = {}

            # loop from january to december
            for index in range(1,13):

                # no reports for this month available, skip it
                if index not in months:
                    continue

                # these are all days for which reports exist in a given month
                days = months[index]

                # render this month
                rendered[year][index] = self.render_month(year, index, days)

        return rendered

        
    def render_month(self, year, month, days):

        html = MyHTMLCalendar(calendar.MONDAY).formatmonth(
            year, month, days
        )
        return html

    def get_context_data(self, **kwargs):

        # get the context
        context = super(ViewStatistics, self).get_context_data(**kwargs)

        try:

            # create data for context
            entries = {}

            # create the history
            history = History(stats_dir())

            # loop over years
            for year in history.years():

                # get the months of the given year
                months = history.months(year)

                # loop over all months
                for month in months:

                    # get days of a give month
                    days = history.days(year, month)

                    # loop over given day
                    for day in days:

                        # construct path to this day report dir
                        prefix = "%04d-%02d-%02d" % (year, month, day)

                        # see if report is available
                        if not is_valid_report_dir(os.path.join(reports_dir(), prefix)):
                            continue

                        # ok report is available, fill the entries dict
                        if year not in entries:
                            entries[year] = {}

                        if month not in entries[year]:
                            entries[year][month] = []

                        entries[year][month].append(day)

            # and render entries as HTML 
            html = self.render(entries)

            # and save the context
            context['entries'] = html
            
        except Exception as e:

            context['error_text'] = str(e)
            context['traceback']  = str(traceback.format_exc())
        
        return context
