import os
import json
import codecs
import traceback
import pandas as pd
import numpy as np

#
# django
#
from django.views import generic

#
# domain
#
from daemon.core import UiPath #, DnsSafetyPaths

#
# local
#
from .mixins import ReporterMixin


#
#
#
class SummaryLoader:

    def __init__(self, period, prefix):

        self.folder = os.path.join(UiPath.var_dir(), "reports", prefix)

    def summary(self):

        with codecs.open(os.path.join(self.folder, "summary.json"), "r", encoding="utf-8") as fin:
            return json.load(fin)

    def hits(self):

        return pd.read_csv(os.path.join(self.folder, "hits.csv"), index_col='timestamp')

    def bytes(self):

        return pd.read_csv(os.path.join(self.folder, "bytes.csv"), index_col='timestamp')


#
# generic summary view (for now, day, month, year)
#
class ViewSummary(ReporterMixin, generic.TemplateView):

    def get_context_data(self, **kwargs):

        # get the context
        context = super(ViewSummary, self).get_context_data(**kwargs)

        try:
            # see what period is requested
            period, prefix = self.get_period()

            # load it
            l = SummaryLoader(period, prefix)

            # and fill the context
            context['summary'] = l.summary()
            context['hits']    = l.hits()
            context['bytes']   = l.bytes()
            

        except Exception as e:

            context['error_text'] = str(e)
            context['traceback']  = str(traceback.format_exc())

        return context
