import os
import traceback

#
# django
#
from django.views import generic


#
# local
#
from .mixins  import *
from .loaders import *

#
# class for users
#
class ViewUsers(ReporterMixin, generic.TemplateView):

    def get_context_data(self, **kwargs):

        # get the context
        context = super(ViewUsers, self).get_context_data(**kwargs)

        try:

            # create loader
            r = UsersLoader(self.get_period())

            # and fill the context
            context['object_list'] = r.list()
            context['totals']      = r.totals()

            # files are only valid for now and days
            period = self.kwargs['period']
            if period in ["now", "day"]:
                context['downloaded_files'] = r.downloaded_files()
                context['uploaded_files']   = r.uploaded_files()
            else:
                context['files'] = []

        except Exception as e:

            context['error_text'] = str(e)
            context['traceback']  = str(traceback.format_exc())

        return context