import os
import traceback

#
# django
#
from django.views import generic

#
# domain
#
from daemon.stats.collector import History
from daemon.stats.reporter  import is_valid_report_dir

#
# local
#
from .mixins import reports_dir, stats_dir

#
#
#
class ViewYears(generic.TemplateView):

    def get_context_data(self, **kwargs):

        # get the context
        context = super(ViewYears, self).get_context_data(**kwargs)

        try:

            # create history 
            history = History(stats_dir())

            # create data for context
            entries = []

            # and fill it
            for year in history.years():

                months = history.months(year)
                days   = 0
                for month in months:
                    days += len(history.days(year, month))

                entries.append({
                    "year"   : year,
                    "months" : len(months),
                    "days"   : days,
                })

            # and save the context
            context['entries'] = entries

        except Exception as e:

            context['error_text'] = str(e)
            context['traceback']  = str(traceback.format_exc())
        
        return context
