#!/usr/bin/env python3
#
# (c) Diladele B.V. 2019, Web Safety for Squid Proxy
#  
# Sample automation script
#
import os
import sys
import django
import logging
import argparse

#
# in order for Django to work correctly with our monitor app, we add ourselves
# to Python's search path
sys.path.append(os.path.abspath(__file__))

#
# tell Django where to read settings from
#
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "console.settings")

#
#
#
django.setup()

#
# do imports
#
from safety.models import *
from squid.models import *
from node.models import *
from node.blogic import NetworkGenerator, NetworkDeviceSyncer

#
#
#
def process_network(network):

    # get the network object
    item = NetAdvanced.objects.first()

    # for none just save in the UI
    if network in ["none", "manual"]:

        item.network_type = NetAdvanced.NETWORK_TYPE_NONE
        item.save()

        # and return
        return
        
    # for actual network settings, first set
    if network in ["debian10"]:
        item.network_type = NetAdvanced.NETWORK_TYPE_INTERFACES
    elif network in ["ubuntu20"]:
        item.network_type = NetAdvanced.NETWORK_TYPE_NETPLAN
    else:
        pass

    # and save it
    item.save()

    # then forcefully reread all NICs from the system
    force_sync = True
    NetworkDeviceSyncer().reread_from_system(force_sync)

    # and re-generate the contents on disk
    NetworkGenerator().generate(
        NetDevice.objects.all(),
        NetAdvanced.objects.first()
    )

    
#
#
#
def process_args(args):

    if args.network is not None:

        # dump to log
        logging.info("Resetting network configuration to '%s' ..." % args.network)

        # run it
        process_network(args.network)

        # and dump to log again
        logging.info("Done, network configuration is now '%s' ..." % args.network)

    else:
        logging.info("No argument specified, aborting.")

#
# main
#
def main():
    
    # define arguments
    parser = argparse.ArgumentParser(description='Utility script for post build steps for Web Safety')
    parser.add_argument("--network", help="none|debian10|ubuntu20", required=False)

    # parse them
    args = parser.parse_args()    

    logging.basicConfig(format='%(asctime)s %(message)s', level=logging.INFO)   
        
    # process
    process_args(args)

    logging.info("Utility script finished successfully!")
        
if __name__ == '__main__':

    main()